@@ -24,7 +24,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include "tk.h"
+#include "gltk.h"
 
 
 #define EXP_WIDTH 80
