@@ -25,7 +25,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include "tk.h"
+#include "gltk.h"
 
 
 #define PI 3.141592654
