@@ -28,7 +28,7 @@
 #include <string.h>
 #include <math.h>
 #include <stdlib.h>
-#include "tk.h"
+#include "gltk.h"
 
 
 #define FALSE 0
