@@ -32,7 +32,10 @@
 #define DODECASOLID	18
 #define CONESOLID	19
 
+#ifndef PI
 #define PI 3.1415926535897
+#endif
+
 
 /*	structure for each geometric object	*/
 typedef struct model {
