@@ -22,9 +22,18 @@
 
 
 /*
-$Id: quadric.c,v 1.9 1995/08/01 21:43:18 brianp Exp $
+$Id: quadric.c,v 1.12 1995/10/19 15:57:16 brianp Exp $
 
 $Log: quadric.c,v $
+ * Revision 1.12  1995/10/19  15:57:16  brianp
+ * changed PI to M_PI
+ *
+ * Revision 1.11  1995/09/12  13:37:54  brianp
+ * fixed a bug in gluQuadricCallback() per Chee Weng
+ *
+ * Revision 1.10  1995/09/05  13:37:36  brianp
+ * added some missing initializations in gluNewQuadric()
+ *
  * Revision 1.9  1995/08/01  21:43:18  brianp
  * lots more work on quadric functions
  *
@@ -71,20 +80,22 @@ $Log: quadric.c,v $
 
 
 
-#define PI 3.14159265
+#ifndef M_PI
+#  define M_PI (3.1415926)
+#endif
 
 
 /*
  * Convert degrees to radians:
  */
-#define DEG_TO_RAD(A)   ((A)*(PI/180.0))
+#define DEG_TO_RAD(A)   ((A)*(M_PI/180.0))
 
 
 /*
  * Sin and Cos for degree angles:
  */
-#define SIND( A )   sin( (A)*(PI/180.0) )
-#define COSD( A)    cos( (A)*(PI/180.0) )
+#define SIND( A )   sin( (A)*(M_PI/180.0) )
+#define COSD( A)    cos( (A)*(M_PI/180.0) )
 
 
 
@@ -124,6 +135,9 @@ GLUquadricObj *gluNewQuadric( void )
    if (q) {
       q->DrawStyle = GLU_FILL;
       q->Orientation = GLU_OUTSIDE;
+      q->TextureFlag = GL_FALSE;
+      q->Normals = GLU_SMOOTH;
+      q->ErrorFunc = NULL;
    }
    return q;
 }
@@ -176,7 +190,7 @@ void gluQuadricOrientation( GLUquadricObj *quadObject, GLenum orientation )
 void gluQuadricCallback( GLUquadricObj *qobj,
 			 GLenum which, void (*fn)() )
 {
-   if (qobj && fn && which==GLU_ERROR) {
+   if (qobj && which==GLU_ERROR) {
       qobj->ErrorFunc = fn;
    }
 }
@@ -219,7 +233,7 @@ void gluCylinder( GLUquadricObj *qobj,
       nsign = 1.0;
    }
 
-   da = 2.0*PI / slices;
+   da = 2.0*M_PI / slices;
    dr = (topRadius-baseRadius) / stacks;
    dz = height / stacks;
    nz = (baseRadius-topRadius) / height;  /* Z component of normal vectors */
@@ -351,8 +365,8 @@ void gluSphere( GLUquadricObj *qobj,
       nsign = 1.0;
    }
 
-   drho = PI / (GLfloat) stacks;
-   dtheta = 2.0 * PI / (GLfloat) slices;
+   drho = M_PI / (GLfloat) stacks;
+   dtheta = 2.0 * M_PI / (GLfloat) slices;
 
    if (qobj->DrawStyle==GLU_FILL) {
       /* draw +Z end as a triangle fan */
@@ -393,7 +407,7 @@ void gluSphere( GLUquadricObj *qobj,
       glBegin( GL_TRIANGLE_FAN );
       glNormal3f( 0.0, 0.0, -1.0 );
       glVertex3f( 0.0, 0.0, -radius*nsign );
-      rho = PI - drho;
+      rho = M_PI - drho;
       for (j=slices;j>=0;j--) {
 	 theta = (j==slices) ? 0.0 : j * dtheta;
 	 x = cos(theta) * sin(rho);
@@ -481,7 +495,7 @@ void gluDisk( GLUquadricObj *qobj,
       }
    }
 
-   da = 2.0*PI / slices;
+   da = 2.0*M_PI / slices;
    dr = (outerRadius-innerRadius) / (GLfloat) loops;
 
    switch (qobj->DrawStyle) {
@@ -516,13 +530,13 @@ void gluDisk( GLUquadricObj *qobj,
 	 /* draw rings */
 	 for (r=innerRadius; r<=outerRadius; r+=dr) {
 	    glBegin( GL_LINE_LOOP );
-	    for (a=0.0; a<2.0*PI; a+=da) {
+	    for (a=0.0; a<2.0*M_PI; a+=da) {
 	       glVertex2f( r*sin(a), r*cos(a) );
 	    }
 	    glEnd();
 	 }
 	 /* draw spokes */
-	 for (a=0.0; a<2.0*PI; a+=da) {
+	 for (a=0.0; a<2.0*M_PI; a+=da) {
 	    x = sin(a);
 	    y = cos(a);
 	    glBegin( GL_LINE_STRIP );
@@ -534,7 +548,7 @@ void gluDisk( GLUquadricObj *qobj,
 	 break;
       case GLU_POINT:
 	 glBegin( GL_POINTS );
-	 for (a=0.0; a<2.0*PI; a+=da) {
+	 for (a=0.0; a<2.0*M_PI; a+=da) {
 	    x = sin(a);
 	    y = cos(a);
 	    for (r=innerRadius; r<=outerRadius; r+=dr) {
@@ -546,7 +560,7 @@ void gluDisk( GLUquadricObj *qobj,
       case GLU_SILHOUETTE:
 	 if (innerRadius!=0.0) {
 	    glBegin( GL_LINE_LOOP );
-	    for (a=0.0; a<2.0*PI; a+=da) {
+	    for (a=0.0; a<2.0*M_PI; a+=da) {
 	       x = innerRadius * sin(a);
 	       y = innerRadius * cos(a);
 	       glVertex2f( x, y );
@@ -554,7 +568,7 @@ void gluDisk( GLUquadricObj *qobj,
 	    glEnd();
 	 }
 	 glBegin( GL_LINE_LOOP );
-	 for (a=0; a<2.0*PI; a+=da) {
+	 for (a=0; a<2.0*M_PI; a+=da) {
 	    x = outerRadius * sin(a);
 	    y = outerRadius * cos(a);
 	    glVertex2f( x, y );
