@@ -27,9 +27,19 @@
 
 
 /*
-$Id: attrib.c,v 1.7 1995/07/24 18:58:55 brianp Exp $
+$Id: attrib.c,v 1.10 1995/11/01 23:18:22 brianp Exp $
 
 $Log: attrib.c,v $
+ * Revision 1.10  1995/11/01  23:18:22  brianp
+ * update CC.Light.LastEnabled when poping GL_ENABLE_BIT
+ *
+ * Revision 1.9  1995/10/27  20:29:02  brianp
+ * added glPolygonOffsetEXT() support
+ *
+ * Revision 1.8  1995/08/31  21:32:45  brianp
+ * new TexGenEnabled bitfield
+ * introduced CC.NewState convention
+ *
  * Revision 1.7  1995/07/24  18:58:55  brianp
  * added CC.ClipSpans logic to glPopAttrib()
  *
@@ -177,15 +187,13 @@ void gl_push_attrib( GLbitfield mask )
       attr->Map2Vertex4 = CC.Eval.Map2Vertex4;
       attr->Normalize = CC.Transform.Normalize;
       attr->PointSmooth = CC.Point.SmoothFlag;
+      attr->PolygonOffset = CC.Polygon.OffsetEnabled;
       attr->PolygonSmooth = CC.Polygon.SmoothFlag;
       attr->PolygonStipple = CC.Polygon.StippleFlag;
       attr->Scissor = CC.Scissor.Enabled;
       attr->Stencil = CC.Stencil.Enabled;
       attr->Texture = CC.Texture.Enabled;
-      attr->TexGenS = CC.Texture.TexGenEnabledS;
-      attr->TexGenT = CC.Texture.TexGenEnabledT;
-      attr->TexGenR = CC.Texture.TexGenEnabledR;
-      attr->TexGenQ = CC.Texture.TexGenEnabledQ;
+      attr->TexGen = CC.Texture.TexGenEnabled;
       newnode = new_attrib_node( GL_ENABLE_BIT );
       newnode->data = attr;
       newnode->next = head;
@@ -372,7 +380,6 @@ void gl_pop_attrib( void )
          case GL_COLOR_BUFFER_BIT:
             MEMCPY( &CC.Color, attr->data,
 		    sizeof(struct gl_colorbuffer_attrib) );
-	    gl_update_pixel_logic();
             break;
          case GL_CURRENT_BIT:
             MEMCPY( &CC.Current, attr->data,
@@ -395,8 +402,12 @@ void gl_pop_attrib( void )
 	    CC.Depth.Test = enable->DepthTest;
 	    CC.Color.DitherFlag = enable->Dither;
 	    CC.Fog.Enabled = enable->Fog;
+            CC.Light.LastEnabled = -1;
 	    for (i=0;i<MAX_LIGHTS;i++) {
 	       CC.Light.Light[i].Enabled = enable->Light[i];
+               if (enable->Light[i]) {
+                  CC.Light.LastEnabled = i;
+               }
 	    }
 	    CC.Light.Enabled = enable->Lighting;
 	    CC.Line.SmoothFlag = enable->LineSmooth;
@@ -422,15 +433,13 @@ void gl_pop_attrib( void )
 	    CC.Eval.Map2Vertex4 = enable->Map2Vertex4;
 	    CC.Transform.Normalize = enable->Normalize;
 	    CC.Point.SmoothFlag = enable->PointSmooth;
+	    CC.Polygon.OffsetEnabled = enable->PolygonOffset;
 	    CC.Polygon.SmoothFlag = enable->PolygonSmooth;
 	    CC.Polygon.StippleFlag = enable->PolygonStipple;
 	    CC.Scissor.Enabled = enable->Scissor;
 	    CC.Stencil.Enabled = enable->Stencil;
 	    CC.Texture.Enabled = enable->Texture;
-	    CC.Texture.TexGenEnabledS = enable->TexGenS;
-	    CC.Texture.TexGenEnabledT = enable->TexGenT;
-	    CC.Texture.TexGenEnabledR = enable->TexGenR;
-	    CC.Texture.TexGenEnabledQ = enable->TexGenQ;
+	    CC.Texture.TexGenEnabled = enable->TexGen;
             break;
          case GL_EVAL_BIT:
             MEMCPY( &CC.Eval, attr->data, sizeof(struct gl_eval_attrib) );
@@ -504,7 +513,7 @@ void gl_pop_attrib( void )
       attr = next;
    }
 
-   gl_update_rasterflags();
+   CC.NewState = GL_TRUE;
 }
 
 
