@@ -27,9 +27,17 @@
 
 
 /*
-$Id: blend.c,v 1.12 1995/06/12 15:34:31 brianp Exp $
+$Id: blend.c,v 1.14 1995/10/30 15:30:33 brianp Exp $
 
 $Log: blend.c,v $
+ * Revision 1.14  1995/10/30  15:30:33  brianp
+ * pass mask array to DD.read_color_pixels
+ *
+ * Revision 1.13  1995/08/31  21:30:26  brianp
+ * renamed blend() as dd_blend()
+ * use *DD.read_color_pixels instead of dd_read_color_pixels
+ * introduced CC.NewState convention
+ *
  * Revision 1.12  1995/06/12  15:34:31  brianp
  * changed color arrays to GLubyte
  *
@@ -135,7 +143,7 @@ void glBlendFunc( GLenum sfactor, GLenum dfactor )
       }
    }
 
-   gl_update_pixel_logic();
+   CC.NewState = GL_TRUE;
 }
 
 
@@ -163,7 +171,7 @@ void glBlendEquationEXT( GLenum mode )
 	    gl_error( GL_INVALID_ENUM, "glBlendEquationEXT" );
       }
    }
-   gl_update_pixel_logic();
+   CC.NewState = GL_TRUE;
 }
 
 
@@ -197,11 +205,11 @@ void glBlendColorEXT( GLclampf red, GLclampf green,
  * In/Out:  red, green, blue, alpha - the incoming and modified pixels
  * Input:  rdest, gdest, bdest, adest - the pixels from the dest color buffer
  */
-static void blend( GLuint n, const GLubyte mask[],
-		   GLubyte red[], GLubyte green[],
-		   GLubyte blue[], GLubyte alpha[],
-		   const GLubyte rdest[], const GLubyte gdest[],
-		   const GLubyte bdest[], const GLubyte adest[] )
+static void do_blend( GLuint n, const GLubyte mask[],
+                      GLubyte red[], GLubyte green[],
+                      GLubyte blue[], GLubyte alpha[],
+                      const GLubyte rdest[], const GLubyte gdest[],
+                      const GLubyte bdest[], const GLubyte adest[] )
 {
    GLfloat rscale, gscale, bscale, ascale;
    GLuint i;
@@ -635,7 +643,7 @@ void gl_blend_span( GLuint n, GLint x, GLint y,
    /* Read span of current frame buffer pixels */
    gl_read_color_span( n, x, y, rdest, gdest, bdest, adest );
 
-   blend( n, mask, red, green, blue, alpha, rdest, gdest, bdest, adest );
+   do_blend( n, mask, red, green, blue, alpha, rdest, gdest, bdest, adest );
 }
 
 
@@ -662,7 +670,7 @@ void gl_blend_pixels( GLuint n, const GLint x[], const GLint y[],
    }
 
    /* Read pixels from current color buffer */
-   dd_read_color_pixels( n, x, y, rdest, gdest, bdest, adest );
+   (*DD.read_color_pixels)( n, x, y, rdest, gdest, bdest, adest, mask );
 
-   blend( n, mask, red, green, blue, alpha, rdest, gdest, bdest, adest );
+   do_blend( n, mask, red, green, blue, alpha, rdest, gdest, bdest, adest );
 }
