@@ -28,9 +28,15 @@
 
 
 /*
-$Id: clip.c,v 1.12 1995/07/28 21:32:12 brianp Exp $
+$Id: clip.c,v 1.14 1995/10/14 17:41:29 brianp Exp $
 
 $Log: clip.c,v $
+ * Revision 1.14  1995/10/14  17:41:29  brianp
+ * made glClipPlane display list-able
+ *
+ * Revision 1.13  1995/09/15  18:54:09  brianp
+ * changed INSIDE, OUTSIDE macros to fix Oleg Krivosheev's clipping problems
+ *
  * Revision 1.12  1995/07/28  21:32:12  brianp
  * fixed aux data interpolation bug in gl_viewclip_line()
  *
@@ -74,6 +80,7 @@ $Log: clip.c,v $
 #include "GL/gl.h"
 #include "clip.h"
 #include "context.h"
+#include "list.h"
 #include "macros.h"
 #include "vb.h"
 #include "xform.h"
@@ -130,8 +137,8 @@ static void interpolate_aux( GLuint dst, GLfloat t, GLuint in, GLuint out )
 	 /* also interpolate eye Z component */
 	 VB.Eye[dst][2] = LINTERP( t, VB.Eye[in][2], VB.Eye[out][2] );
       }
-      VB.Vs[dst] = LINTERP( t, VB.Vs[in], VB.Vs[out] );
-      VB.Vt[dst] = LINTERP( t, VB.Vt[in], VB.Vt[out] );
+      VB.TexCoord[dst][0] = LINTERP(t,VB.TexCoord[in][0],VB.TexCoord[out][0]);
+      VB.TexCoord[dst][1] = LINTERP(t,VB.TexCoord[in][1],VB.TexCoord[out][1]);
    }
 
 }
@@ -139,10 +146,9 @@ static void interpolate_aux( GLuint dst, GLfloat t, GLuint in, GLuint out )
 
 
 
-void glClipPlane( GLenum plane, const GLdouble *equation )
+void gl_clipplane( GLenum plane, const float *equation )
 {
    GLint p;
-   GLfloat fequation[4];
 
    p = (GLint) (plane - GL_CLIP_PLANE0);
    if (p<0 || p>=MAX_CLIP_PLANES) {
@@ -154,16 +160,28 @@ void glClipPlane( GLenum plane, const GLdouble *equation )
     * The Equation is transformed by the transpose of the inverse of the
     * current modelview matrix and stored in the resulting eye coordinates.
     */
+   if (!CC.ModelViewInvValid) {
+      gl_compute_modelview_inverse();
+   }
+   gl_transform_vector( CC.Transform.ClipEquation[p], equation,
+		        CC.ModelViewInv );
+}
+
+
+
+void glClipPlane( GLenum plane, const GLdouble *equation )
+{
+   GLfloat fequation[4];
    fequation[0] = (GLfloat) equation[0];
    fequation[1] = (GLfloat) equation[1];
    fequation[2] = (GLfloat) equation[2];
    fequation[3] = (GLfloat) equation[3];
-
-   if (!CC.ModelViewInvValid) {
-      gl_compute_modelview_inverse();
+   if (CC.CompileFlag) {
+      gl_save_clipplane( plane, fequation );
+   }
+   if (CC.ExecuteFlag) {
+      gl_clipplane( plane, fequation );
    }
-   gl_transform_vector( CC.Transform.ClipEquation[p], fequation,
-		        CC.ModelViewInv );
 }
 
 
@@ -699,17 +717,19 @@ GLuint gl_userclip_point( const GLfloat v[] )
 }
 
 
+#define MAGIC_NUMBER -0.8e-03F
+
 
 /* Test if VB.Eye[J] is inside the clipping plane defined by A,B,C,D */
-#define INSIDE( J, A, B, C, D )   					\
-   ( (VB.Eye[J][0] * A + VB.Eye[J][1] * B		\
-    + VB.Eye[J][2] * C + VB.Eye[J][3] * D) >=0.0F )
+#define INSIDE( J, A, B, C, D )   				\
+   ( (VB.Eye[J][0] * A + VB.Eye[J][1] * B			\
+    + VB.Eye[J][2] * C + VB.Eye[J][3] * D) >= MAGIC_NUMBER )
 
 
 /* Test if VB.Eye[J] is outside the clipping plane defined by A,B,C,D */
-#define OUTSIDE( J, A, B, C, D )   					\
-   ( (VB.Eye[J][0] * A + VB.Eye[J][1] * B		\
-    + VB.Eye[J][2] * C + VB.Eye[J][3] * D) < 0.0F )
+#define OUTSIDE( J, A, B, C, D )   				\
+   ( (VB.Eye[J][0] * A + VB.Eye[J][1] * B			\
+    + VB.Eye[J][2] * C + VB.Eye[J][3] * D) < MAGIC_NUMBER )
 
 
 /*
@@ -755,6 +775,7 @@ GLuint gl_userclip_line( GLuint *i, GLuint *j )
 	       else {
 		  t = -(VB.Eye[jj][0]*a+VB.Eye[jj][1]*b
 		       +VB.Eye[jj][2]*c+VB.Eye[jj][3]*d) / denom;
+                  if (t>1.0F)  t = 1.0F;
 	       }
 	       VB.Eye[VB.Free][0] = VB.Eye[jj][0] + t * dx;
 	       VB.Eye[VB.Free][1] = VB.Eye[jj][1] + t * dy;
@@ -784,6 +805,7 @@ GLuint gl_userclip_line( GLuint *i, GLuint *j )
 	       else {
 		  t = -(VB.Eye[ii][0]*a+VB.Eye[ii][1]*b
 		       +VB.Eye[ii][2]*c+VB.Eye[ii][3]*d) / denom;
+                  if (t>1.0F)  t = 1.0F;
 	       }
 	       VB.Eye[VB.Free][0] = VB.Eye[ii][0] + t * dx;
 	       VB.Eye[VB.Free][1] = VB.Eye[ii][1] + t * dy;
@@ -873,6 +895,10 @@ GLuint gl_userclip_polygon( GLuint n, GLuint vlist[] )
 		  else {
 		     t = -(VB.Eye[currj][0]*a+VB.Eye[currj][1]*b
 		       +VB.Eye[currj][2]*c+VB.Eye[currj][3]*d) / denom;
+                     if (t>1.0F) {
+                        /*printf("t1=%g\n", t);*/
+                        t = 1.0F;
+                     }
 		  }
 		  /* interpolate new vertex position */
 		  VB.Eye[VB.Free][0] = VB.Eye[currj][0] + t*dx;
@@ -908,6 +934,10 @@ GLuint gl_userclip_polygon( GLuint n, GLuint vlist[] )
 		  else {
 		     t = -(VB.Eye[prevj][0]*a+VB.Eye[prevj][1]*b
 		       +VB.Eye[prevj][2]*c+VB.Eye[prevj][3]*d) / denom;
+                     if (t>1.0F) {
+                        /*printf("t2=%g\n", t);*/
+                        t = 1.0F;
+                     }
 		  }
 		  /* interpolate new vertex position */
 		  VB.Eye[VB.Free][0] = VB.Eye[prevj][0] + t*dx;
