@@ -22,9 +22,39 @@
 
 
 /*
-$Id: context.c,v 1.23 1995/07/25 16:41:54 brianp Exp $
+$Id: context.c,v 1.32 1995/11/22 13:35:00 brianp Exp $
 
 $Log: context.c,v $
+ * Revision 1.32  1995/11/22  13:35:00  brianp
+ * added MutableColors flag, test for RGBA mode before setting GAMMA_BIT
+ *
+ * Revision 1.31  1995/11/08  22:09:05  brianp
+ * changed GL<type> assertions from == to >=
+ *
+ * Revision 1.30  1995/11/03  17:41:05  brianp
+ * removed unused variables
+ *
+ * Revision 1.29  1995/11/01  21:44:15  brianp
+ * added CC.Light.LastEnabled
+ *
+ * Revision 1.28  1995/10/27  20:28:27  brianp
+ * added glPolygonOffsetEXT() support
+ *
+ * Revision 1.27  1995/10/19  15:45:20  brianp
+ * added gamma support
+ * new arguments to gl_new_context()
+ *
+ * Revision 1.26  1995/10/14  16:26:13  brianp
+ * enable dithering by default
+ * added SWmasking code
+ *
+ * Revision 1.25  1995/09/25  16:31:54  brianp
+ * reorganized front and back material indexing
+ *
+ * Revision 1.24  1995/09/15  18:47:32  brianp
+ * introduced CC.NewState convention
+ * use bitmask flag for CC.Texture.TexGenEnabled
+ *
  * Revision 1.23  1995/07/25  16:41:54  brianp
  * made changes for using CC.VertexFunc pointer
  *
@@ -117,6 +147,7 @@ $Log: context.c,v $
 
 
 #include <assert.h>
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -124,14 +155,21 @@ $Log: context.c,v $
 #include "dd.h"
 #include "draw.h"
 #include "eval.h"
+#include "lines.h"
 #include "list.h"
 #include "macros.h"
 #include "pb.h"
+#include "points.h"
+#include "polygons.h"
+#include "vb.h"
 
 
-struct gl_context CC;              /* Copy of Current Context (PUBLIC) */
 
-static struct gl_context *CCptr;   /* Pointer to Current Context (PRIVATE) */
+/* Copy of Current Context (PUBLIC) */
+struct gl_context CC;
+
+/* Pointer to Current Context (PRIVATE) */
+static struct gl_context *CCptr = NULL;
 
 
 
@@ -171,6 +209,7 @@ static void init_light( struct gl_light *l, GLuint n )
       ASSIGN_4V( l->Specular, 0.0, 0.0, 0.0, 1.0 );
    }
    ASSIGN_4V( l->Position, 0.0, 0.0, 1.0, 0.0 );
+   ASSIGN_3V( l->NormPosition, 0.0, 0.0, 1.0 );
    ASSIGN_3V( l->Direction, 0.0, 0.0, -1.0 );
    l->SpotExponent = 0.0;
    l->SpotCutoff = 180.0;
@@ -192,23 +231,69 @@ static void init_lightmodel( struct gl_lightmodel *lm )
 
 static void init_material( struct gl_material *m )
 {
-   GLint i;
-
-   /* loop over front and back face parameters */
-   for (i=0;i<2;i++) {
-      ASSIGN_4V( m->Ambient[i],  0.2, 0.2, 0.2, 1.0 );
-      ASSIGN_4V( m->Diffuse[i],  0.8, 0.8, 0.8, 1.0 );
-      ASSIGN_4V( m->Specular[i], 0.0, 0.0, 0.0, 1.0 );
-      ASSIGN_4V( m->Emission[i], 0.0, 0.0, 0.0, 1.0 );
-      m->Shininess[i] = 0.0;
-      m->AmbientIndex[i] = 0;
-      m->DiffuseIndex[i] = 1;
-      m->SpecularIndex[i] = 1;
-   }
+   ASSIGN_4V( m->Ambient,  0.2, 0.2, 0.2, 1.0 );
+   ASSIGN_4V( m->Diffuse,  0.8, 0.8, 0.8, 1.0 );
+   ASSIGN_4V( m->Specular, 0.0, 0.0, 0.0, 1.0 );
+   ASSIGN_4V( m->Emission, 0.0, 0.0, 0.0, 1.0 );
+   m->Shininess = 0.0;
+   m->AmbientIndex = 0;
+   m->DiffuseIndex = 1;
+   m->SpecularIndex = 1;
 }
 
 
 
+/*
+ * Setup the gamma lookup tables for the given context by examining the
+ * MESA_GAMMA environment variable.
+ */
+static void init_gamma_tables( struct gl_context *c )
+{
+   char *gamma;
+   GLint i, n;
+   GLdouble g;
+
+   if (!c->RGBAflag) {
+      return;
+   }
+
+   gamma = getenv("MESA_GAMMA");
+   if (gamma) {
+      sscanf( gamma, "%f %f %f", &c->RedGamma, &c->GreenGamma, &c->BlueGamma );
+      if (c->RedGamma==0.0)    c->RedGamma = 1.0;
+      if (c->GreenGamma==0.0)  c->GreenGamma = c->RedGamma;
+      if (c->BlueGamma==0.0)   c->BlueGamma = c->RedGamma;
+   }
+   else {
+      c->RedGamma = c->GreenGamma = c->BlueGamma = 1.0;
+   }
+
+   /* red */
+   n = (GLint) c->RedScale;
+   g = 1.0 / c->RedGamma;
+   for (i=0;i<=n;i++) {
+      double x = (double) i / c->RedScale;
+      c->RedRamp[i] = c->RedScale * pow(x,g) + 0.5;
+   }
+
+   /* green */
+   n = (GLint) c->GreenScale;
+   g = 1.0 / c->GreenGamma;
+   for (i=0;i<=n;i++) {
+      double x = (double) i / c->GreenScale;
+      c->GreenRamp[i] = c->GreenScale * pow(x,g) + 0.5;
+   }
+
+   /* blue */
+   n = (GLint) c->BlueScale;
+   g = 1.0 / c->BlueGamma;
+   for (i=0;i<=n;i++) {
+      double x = (double) i / c->BlueScale;
+      c->BlueRamp[i] = c->BlueScale * pow(x,g) + 0.5;
+   }
+}
+
+
 
 /*
  * Initialize a gl_context structure to default values.
@@ -228,6 +313,7 @@ void gl_initialize_context( struct gl_context *c )
 
       c->Color.IndexMask = 0xffffffff;
       c->Color.ColorMask = 0xf;
+      c->Color.SWmasking = GL_FALSE;
       c->Color.ClearIndex = 0;
       ASSIGN_4V( c->Color.ClearColor, 0.0, 0.0, 0.0, 0.0 );
       c->Color.DrawBuffer = GL_FRONT;
@@ -242,7 +328,7 @@ void gl_initialize_context( struct gl_context *c )
       c->Color.LogicOpEnabled = GL_FALSE;
       c->Color.SWLogicOpEnabled = GL_FALSE;
       c->Color.LogicOp = GL_COPY;
-      c->Color.DitherFlag = GL_FALSE;  /* TODO: this should be GL_TRUE */
+      c->Color.DitherFlag = GL_TRUE;
 
       c->Current.Index = 1;
       ASSIGN_3V( c->Current.Normal, 0.0, 0.0, 1.0 );
@@ -259,6 +345,35 @@ void gl_initialize_context( struct gl_context *c )
       c->Depth.Func = GL_LESS;
       c->Depth.Mask = GL_TRUE;
 
+      c->Eval.Map1Color4 = GL_FALSE;
+      c->Eval.Map1Index = GL_FALSE;
+      c->Eval.Map1Normal = GL_FALSE;
+      c->Eval.Map1TextureCoord1 = GL_FALSE;
+      c->Eval.Map1TextureCoord2 = GL_FALSE;
+      c->Eval.Map1TextureCoord3 = GL_FALSE;
+      c->Eval.Map1TextureCoord4 = GL_FALSE;
+      c->Eval.Map1Vertex3 = GL_FALSE;
+      c->Eval.Map1Vertex4 = GL_FALSE;
+      c->Eval.Map2Color4 = GL_FALSE;
+      c->Eval.Map2Index = GL_FALSE;
+      c->Eval.Map2Normal = GL_FALSE;
+      c->Eval.Map2TextureCoord1 = GL_FALSE;
+      c->Eval.Map2TextureCoord2 = GL_FALSE;
+      c->Eval.Map2TextureCoord3 = GL_FALSE;
+      c->Eval.Map2TextureCoord4 = GL_FALSE;
+      c->Eval.Map2Vertex3 = GL_FALSE;
+      c->Eval.Map2Vertex4 = GL_FALSE;
+      c->Eval.AutoNormal = GL_FALSE;
+      c->Eval.MapGrid1un = 1;
+      c->Eval.MapGrid1u1 = 0.0;
+      c->Eval.MapGrid1u2 = 1.0;
+      c->Eval.MapGrid2un = 1;
+      c->Eval.MapGrid2vn = 1;
+      c->Eval.MapGrid2u1 = 0.0;
+      c->Eval.MapGrid2u2 = 1.0;
+      c->Eval.MapGrid2v1 = 0.0;
+      c->Eval.MapGrid2v2 = 1.0;
+
       c->Fog.Enabled = GL_FALSE;
       c->Fog.Mode = GL_EXP;
       ASSIGN_4V( c->Fog.Color, 0.0, 0.0, 0.0, 0.0 );
@@ -277,12 +392,14 @@ void gl_initialize_context( struct gl_context *c )
 	 init_light( &c->Light.Light[i], i );
       }
       init_lightmodel( &c->Light.Model );
-      init_material( &c->Light.Material );
+      init_material( &c->Light.Material[0] );
+      init_material( &c->Light.Material[1] );
       c->Light.ShadeModel = GL_SMOOTH;
       c->Light.Enabled = GL_FALSE;
       c->Light.ColorMaterialFace = GL_FRONT_AND_BACK;
       c->Light.ColorMaterialMode = GL_AMBIENT_AND_DIFFUSE;
       c->Light.ColorMaterialEnabled = GL_FALSE;
+      c->Light.LastEnabled = -1;
 
       c->Line.SmoothFlag = GL_FALSE;
       c->Line.StippleFlag = GL_FALSE;
@@ -329,7 +446,6 @@ void gl_initialize_context( struct gl_context *c )
       c->Pixel.MapBtoB[0] = 0.0;
       c->Pixel.MapAtoA[0] = 0.0;
 
-
       c->Point.SmoothFlag = GL_FALSE;
       c->Point.Size = 1.0;
 
@@ -341,6 +457,9 @@ void gl_initialize_context( struct gl_context *c )
       c->Polygon.Unfilled = GL_FALSE;
       c->Polygon.SmoothFlag = GL_FALSE;
       c->Polygon.StippleFlag = GL_FALSE;
+      c->Polygon.OffsetFactor = 0.0;
+      c->Polygon.OffsetBias = 0.0;
+      c->Polygon.OffsetEnabled = GL_FALSE;
 
       MEMSET( c->PolygonStipple, 0xff, 32*sizeof(GLuint) );
 
@@ -364,10 +483,7 @@ void gl_initialize_context( struct gl_context *c )
       c->Texture.EnvMode = GL_MODULATE;
       ASSIGN_4V( c->Texture.EnvColor, 0.0, 0.0, 0.0, 0.0 );
       ASSIGN_4V( c->Texture.BorderColor, 0.0, 0.0, 0.0, 0.0 );
-      c->Texture.TexGenEnabledS = GL_FALSE;
-      c->Texture.TexGenEnabledT = GL_FALSE;
-      c->Texture.TexGenEnabledR = GL_FALSE;
-      c->Texture.TexGenEnabledQ = GL_FALSE;
+      c->Texture.TexGenEnabled = 0;
       c->Texture.GenModeS = GL_EYE_LINEAR;
       c->Texture.GenModeT = GL_EYE_LINEAR;
       c->Texture.GenModeR = GL_EYE_LINEAR;
@@ -436,8 +552,13 @@ void gl_initialize_context( struct gl_context *c )
       c->NameStackDepth = 0;
 
       c->AttribStackDepth = 0;
+
+      c->NewState = GL_TRUE;
       c->Mode = GL_BITMAP;
 
+      c->NeedNormals = GL_FALSE;
+      c->FastDrawPixels = GL_TRUE;
+
       c->ModelViewStackDepth = 0;
       MEMCPY( c->ModelViewMatrix, identity, 16*sizeof(GLfloat) );
       MEMCPY( c->ModelViewInv, identity, 16*sizeof(GLfloat) );
@@ -448,6 +569,7 @@ void gl_initialize_context( struct gl_context *c )
 
       c->TextureStackDepth = 0;
       MEMCPY( c->TextureMatrix, identity, 16*sizeof(GLfloat) );
+      c->IdentityTexMat = GL_TRUE;
 
       for (i=0;i<MAX_TEXTURE_LEVELS;i++) {
 	 c->TextureComponents1D[i] = 0;
@@ -464,7 +586,7 @@ void gl_initialize_context( struct gl_context *c )
       }
 
       c->VertexFunc = gl_execute_vertex;
-      c->PointFunc = NULL;
+      c->PointsFunc = NULL;
       c->LineFunc = NULL;
       c->PolygonFunc = NULL;
       c->AuxPolygonFunc = NULL;
@@ -480,10 +602,6 @@ void gl_initialize_context( struct gl_context *c )
       c->StencilBuffer = NULL;
 
       c->ErrorValue = GL_NO_ERROR;
-
-      gl_update_rasterflags();
-
-      /* etc. */
    }
 }
 
@@ -491,25 +609,55 @@ void gl_initialize_context( struct gl_context *c )
 
 /*
  * Allocate and initialize a gl_context structure.
+ * Input:  rgb_flag - GL_TRUE or GL_FALSE to indicate if using RGB mode
+ *         redscale, greenscale, bluescale, alphascale - if in RGB mode
+ *               these values are used to scale floating point color values
+ *               in [0,1] to integers in [0,scale]
+ *         db_flag - GL_TRUE or GL_FALSE to indicate if using double buffering
+ *         sharelist - another context to share display lists with or NULL
  */
-struct gl_context *gl_new_context( struct gl_context *shareList )
+struct gl_context *gl_new_context( GLboolean rgb_flag,
+                                   GLfloat redscale,
+                                   GLfloat greenscale,
+                                   GLfloat bluescale,
+                                   GLfloat alphascale,
+                                   GLboolean db_flag,
+                                   struct gl_context *shareList )
 {
    struct gl_context *c;
 
    /* do some implementation tests */
-   assert( sizeof(GLbyte)==1 );
-   assert( sizeof(GLshort)==2 );
-   assert( sizeof(GLint)==4 );
-   assert( sizeof(GLubyte)==1 );
-   assert( sizeof(GLushort)==2 );
-   assert( sizeof(GLuint)==4 );
+   assert( sizeof(GLbyte) >= 1 );
+   assert( sizeof(GLshort) >= 2 );
+   assert( sizeof(GLint) >= 4 );
+   assert( sizeof(GLubyte) >= 1 );
+   assert( sizeof(GLushort) >= 2 );
+   assert( sizeof(GLuint) >= 4 );
 
    gl_init_lists();
    gl_init_eval();
+   gl_init_vb();
 
    c = (struct gl_context *) malloc( sizeof(struct gl_context) );
    if (c) {
       gl_initialize_context( c );
+      c->RGBAflag = rgb_flag;
+      c->RedScale = redscale;
+      c->GreenScale = greenscale;
+      c->BlueScale = bluescale;
+      c->AlphaScale = alphascale;
+      if (db_flag) {
+         c->DBflag = GL_TRUE;
+         c->Color.DrawBuffer = GL_BACK;
+         c->Pixel.ReadBuffer = GL_BACK;
+      }
+      else {
+         c->DBflag = GL_FALSE;
+         c->Color.DrawBuffer = GL_FRONT;
+         c->Pixel.ReadBuffer = GL_FRONT;
+      }
+      init_gamma_tables( c );
+
       if (shareList) {
 	 /* share the group of display lists of another context */
 	 c->ListGroup = shareList->ListGroup;
@@ -563,8 +711,8 @@ void gl_set_context( struct gl_context *c )
       CCptr = c;
       MEMCPY( &CC, c, sizeof(struct gl_context) );
    }
-   gl_update_rasterflags();
    gl_init_pb( GL_BITMAP );
+   CC.NewState = GL_TRUE;
 }
 
 
@@ -711,6 +859,26 @@ void gl_error( GLenum error, char *s )
 
 
 
+GLenum glGetError( void )
+{
+   GLenum e;
+
+   if (!CCptr) {
+      /* No current context */
+      return GL_NO_ERROR;
+   }
+
+   if (INSIDE_BEGIN_END) {
+      gl_error( GL_INVALID_OPERATION, "glGetError" );
+      return GL_INVALID_OPERATION;
+   }
+
+   e = CC.ErrorValue;
+   CC.ErrorValue = GL_NO_ERROR;
+   return e;
+}
+
+
 
 /*
  * Since the device driver may or may not support pixel logic ops we
@@ -718,12 +886,12 @@ void gl_error( GLenum error, char *s )
  * glBlendEquation, glEn/Disable( GL_LOGIC_OP ), glEn/Disable( GL_BLEND ),
  * or glPopAttrib is called.
  */
-void gl_update_pixel_logic( void )
+static void update_pixel_logic( void )
 {
    if (CC.RGBAflag) {
       /* RGBA mode blending w/ Logic Op */
       if (CC.Color.BlendEnabled && CC.Color.BlendEquation==GL_LOGIC_OP) {
-	 if (dd_logicop( CC.Color.LogicOp )) {
+	 if ((*DD.logicop)( CC.Color.LogicOp )) {
 	    /* Device driver can do logic, don't have to do it in software */
 	    CC.Color.SWLogicOpEnabled = GL_FALSE;
 	 }
@@ -734,14 +902,14 @@ void gl_update_pixel_logic( void )
       }
       else {
 	 /* no logic op */
-	 (void) dd_logicop( GL_COPY );
+	 (void) (*DD.logicop)( GL_COPY );
 	 CC.Color.SWLogicOpEnabled = GL_FALSE;
       }
    }
    else {
       /* CI mode Logic Op */
       if (CC.Color.LogicOpEnabled) {
-	 if (dd_logicop( CC.Color.LogicOp )) {
+	 if ((*DD.logicop)( CC.Color.LogicOp )) {
 	    /* Device driver can do logic, don't have to do it in software */
 	    CC.Color.SWLogicOpEnabled = GL_FALSE;
 	 }
@@ -752,7 +920,7 @@ void gl_update_pixel_logic( void )
       }
       else {
 	 /* no logic op */
-	 (void) dd_logicop( GL_COPY );
+	 (void) (*DD.logicop)( GL_COPY );
 	 CC.Color.SWLogicOpEnabled = GL_FALSE;
       }
    }
@@ -760,31 +928,71 @@ void gl_update_pixel_logic( void )
 
 
 
+/*
+ * Check if software implemented RGBA or Color Index masking is needed.
+ */
+static void update_pixel_masking( void )
+{
+   if (CC.RGBAflag) {
+      if (CC.Color.ColorMask==0xf) {
+         /* disable masking */
+         (void) (*DD.color_mask)( GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE );
+         CC.Color.SWmasking = GL_FALSE;
+      }
+      else {
+         /* Ask DD to do color masking, if it can't we'll do it in software */
+         GLboolean red   = (CC.Color.ColorMask & 8) ? GL_TRUE : GL_FALSE;
+         GLboolean green = (CC.Color.ColorMask & 4) ? GL_TRUE : GL_FALSE;
+         GLboolean blue  = (CC.Color.ColorMask & 2) ? GL_TRUE : GL_FALSE;
+         GLboolean alpha = (CC.Color.ColorMask & 1) ? GL_TRUE : GL_FALSE;
+         if ((*DD.color_mask)( red, green, blue, alpha )) {
+            CC.Color.SWmasking = GL_FALSE;
+         }
+         else {
+            CC.Color.SWmasking = GL_TRUE;
+         }
+      }
+   }
+   else {
+      if (CC.Color.IndexMask==0xffffffff) {
+         /* disable masking */
+         (void) (*DD.index_mask)( 0xffffffff );
+         CC.Color.SWmasking = GL_FALSE;
+      }
+      else {
+         /* Ask DD to do index masking, if it can't we'll do it in software */
+         if ((*DD.index_mask)( CC.Color.IndexMask )) {
+            CC.Color.SWmasking = GL_FALSE;
+         }
+         else {
+            CC.Color.SWmasking = GL_TRUE;
+         }
+      }
+   }
+}
+
+
+
 /*
  * Recompute the value of CC.RasterMask, CC.ClipMask, etc. according to
  * the current context.
  */
-void gl_update_rasterflags( void )
+static void update_rasterflags( void )
 {
-   GLuint old_flags;
-
-
-   old_flags = CC.RasterMask;
    CC.RasterMask = 0;
 
    if (CC.Color.AlphaEnabled)		CC.RasterMask |= ALPHA_BIT;
    if (CC.Color.BlendEnabled)		CC.RasterMask |= BLEND_BIT;
    if (CC.Depth.Test)			CC.RasterMask |= DEPTH_BIT;
-   if (CC.Color.DitherFlag)		CC.RasterMask |= DITHER_BIT;
    if (CC.Fog.Enabled)			CC.RasterMask |= FOG_BIT;
    if (CC.Color.SWLogicOpEnabled)	CC.RasterMask |= LOGIC_OP_BIT;
    if (CC.Scissor.Enabled)		CC.RasterMask |= SCISSOR_BIT;
    if (CC.Stencil.Enabled)		CC.RasterMask |= STENCIL_BIT;
-
-   CC.PointFunc = NULL;
-   CC.LineFunc = NULL;
-   CC.PolygonFunc = NULL;
-
+   if (CC.Color.SWmasking)		CC.RasterMask |= MASKING_BIT;
+   if (CC.RGBAflag &&
+       (CC.RedGamma!=1.0 ||
+        CC.GreenGamma!=1.0 ||
+        CC.BlueGamma!=1.0))		CC.RasterMask |= GAMMA_BIT;
 
    /* Recompute ClipMask (what has to be interpolated when clipping) */
    CC.ClipMask = 0;
@@ -808,8 +1016,49 @@ void gl_update_rasterflags( void )
 
 
    /* Check if the equation of the plane for polygons has to be computed. */
-   CC.ComputePlane = CC.Depth.Test
-                  || CC.Polygon.CullFlag || CC.Light.Model.TwoSide
-                  || CC.Texture.Enabled;
+   CC.ComputePlane = CC.Depth.Test || CC.Polygon.CullFlag
+                  || CC.Light.Model.TwoSide || CC.Texture.Enabled
+                  || CC.Polygon.OffsetEnabled;
+}
+
+
+
+/*
+ * If CC.NewState==GL_TRUE then this function MUST be called before
+ * rendering any primitive.  Basically, function pointers and miscellaneous
+ * flags are updated to reflect the current state of the state machine.
+ */
+void gl_update_state( void )
+{
+   update_pixel_logic();
+   update_pixel_masking();
+   update_rasterflags();
+
+   /* Check if normal vectors are needed */
+   if (CC.Light.Enabled
+       || (CC.Texture.GenModeS==GL_SPHERE_MAP
+           && (CC.Texture.TexGenEnabled & S_BIT))
+       || (CC.Texture.GenModeT==GL_SPHERE_MAP
+           && (CC.Texture.TexGenEnabled & T_BIT))) {
+      CC.NeedNormals = GL_TRUE;
+   }
+   else {
+      CC.NeedNormals = GL_FALSE;
+   }
+
+   /* Check if incoming colors can be modified by rasterization */
+   if ( CC.Color.BlendEnabled || (CC.RasterMask & GAMMA_BIT)
+       || CC.Color.SWmasking) {
+      CC.MutableColors = GL_TRUE;
+   }
+   else {
+      CC.MutableColors = GL_FALSE;
+   }
+
+   gl_set_point_function();
+   gl_set_line_function();
+   gl_set_polygon_function();
+
+   CC.NewState = GL_FALSE;
 }
 
