@@ -22,9 +22,42 @@
 
 
 /*
-$Id: context.h,v 1.27 1995/08/01 22:36:12 brianp Exp $
+$Id: context.h,v 1.37 1995/11/22 13:34:49 brianp Exp $
 
 $Log: context.h,v $
+ * Revision 1.37  1995/11/22  13:34:49  brianp
+ * added MutableColors flag
+ *
+ * Revision 1.36  1995/11/13  21:28:24  brianp
+ * added comment about TextureWidth/Height
+ *
+ * Revision 1.35  1995/11/02  14:57:14  brianp
+ * changed LastEnabled to GLint type
+ *
+ * Revision 1.34  1995/11/01  21:44:15  brianp
+ * added CC.Light.LastEnabled
+ *
+ * Revision 1.33  1995/10/27  20:28:27  brianp
+ * added glPolygonOffsetEXT() support
+ *
+ * Revision 1.32  1995/10/19  15:45:38  brianp
+ * added gamma support
+ * new arguments to gl_new_context()
+ *
+ * Revision 1.31  1995/10/17  21:34:03  brianp
+ * added typedefs for points_func, line_func, and polygon_func
+ *
+ * Revision 1.30  1995/10/14  16:26:13  brianp
+ * enable dithering by default
+ * added SWmasking code
+ *
+ * Revision 1.29  1995/09/25  16:31:29  brianp
+ * reorganized front and back material indexing
+ *
+ * Revision 1.28  1995/09/15  18:48:52  brianp
+ * introduced CC.NewState convention
+ * cleaned up comments, removed dead code
+ *
  * Revision 1.27  1995/08/01  22:36:12  brianp
  * fixed an extension-compatibility symbol problem
  *
@@ -123,6 +156,7 @@ struct gl_light {
 	GLfloat Diffuse[4];		/* diffuse color */
 	GLfloat Specular[4];		/* specular color */
 	GLfloat Position[4];		/* position in eye coordinates */
+	GLfloat NormPosition[3];	/* unit vector from origin to pos */
 	GLfloat Direction[4];		/* spotlight dir in eye coordinates */
 	GLfloat SpotExponent;
 	GLfloat SpotCutoff;		/* in degress */
@@ -141,15 +175,14 @@ struct gl_lightmodel {
 
 
 struct gl_material {
-	/* [0] = front, [1] = back */
-	GLfloat Ambient[2][4];
-	GLfloat Diffuse[2][4];
-	GLfloat Specular[2][4];
-	GLfloat Emission[2][4];
-	GLfloat Shininess[2];
-	GLfloat AmbientIndex[2];	/* for color index lighting */
-	GLfloat DiffuseIndex[2];	/* for color index lighting */
-	GLfloat SpecularIndex[2];	/* for color index lighting */
+	GLfloat Ambient[4];
+	GLfloat Diffuse[4];
+	GLfloat Specular[4];
+	GLfloat Emission[4];
+	GLfloat Shininess;
+	GLfloat AmbientIndex;	/* for color index lighting */
+	GLfloat DiffuseIndex;	/* for color index lighting */
+	GLfloat SpecularIndex;	/* for color index lighting */
 };
 
 
@@ -168,10 +201,11 @@ struct gl_accum_attrib {
 
 struct gl_colorbuffer_attrib {
 	GLuint ClearIndex;		/* Index to use for glClear */
-	GLuint IndexMask;		/* Color index write mask */
-
 	GLfloat ClearColor[4];		/* Color to use for glClear */
+
+	GLuint IndexMask;		/* Color index write mask */
 	GLuint ColorMask;		/* bit 3=red,2=green,1=blue,0=alpha*/
+        GLboolean SWmasking;		/* Do color/CI masking in software? */
 
 	GLenum DrawBuffer;		/* Which buffer to draw into */
 
@@ -190,7 +224,7 @@ struct gl_colorbuffer_attrib {
 
 	/* logic op */
 	GLboolean LogicOpEnabled;	/* Logic op enabled flag */
-	GLboolean SWLogicOpEnabled;	/* Software logic op enabled flag */
+	GLboolean SWLogicOpEnabled;	/* Do logic ops in software? */
 	GLenum LogicOp;			/* Logic operator */
 
 	GLboolean DitherFlag;		/* Dither enable flag */
@@ -255,15 +289,13 @@ struct gl_enable_attrib {
 	GLboolean Map2Vertex4;
 	GLboolean Normalize;
 	GLboolean PointSmooth;
+	GLboolean PolygonOffset;
 	GLboolean PolygonSmooth;
 	GLboolean PolygonStipple;
 	GLboolean Scissor;
 	GLboolean Stencil;
 	GLuint Texture;
-	GLboolean TexGenS;
-	GLboolean TexGenT;
-	GLboolean TexGenR;
-	GLboolean TexGenQ;
+	GLuint TexGen;
 };
 
 
@@ -321,12 +353,13 @@ struct gl_hint_attrib {
 struct gl_light_attrib {
 	struct gl_light Light[MAX_LIGHTS];	/* Array of lights */
 	struct gl_lightmodel Model;		/* Lighting model */
-	struct gl_material Material;		/* Material */
+	struct gl_material Material[2];		/* Material 0=front, 1=back */
 	GLboolean Enabled;			/* Lighting enabled flag */
 	GLenum ShadeModel;			/* GL_FLAT or GL_SMOOTH */
 	GLenum ColorMaterialFace;		/* see glColorMaterial */
 	GLenum ColorMaterialMode;		/* see glColorMaterial */
 	GLboolean ColorMaterialEnabled;		/* see glColorMaterial */
+        GLint LastEnabled;			/* a lighting optimization */
 };
 
 
@@ -396,6 +429,9 @@ struct gl_polygon_attrib {
 	GLboolean CullFlag;	/* Culling on/off flag */
 	GLboolean SmoothFlag;	/* True if GL_POLYGON_SMOOTH is enabled */
 	GLboolean StippleFlag;	/* True if GL_POLYGON_STIPPLE is enabled */
+        GLfloat OffsetFactor;   /* Polygon offset factor */
+        GLfloat OffsetBias;     /* Polygon offset bias */
+        GLboolean OffsetEnabled;/* Polygon offset enable */
 };
 
 
@@ -403,7 +439,7 @@ struct gl_scissor_attrib {
 	GLboolean	Enabled;		/* Scissor test enabled? */
 	GLint		X, Y;			/* Lower left corner of box */
 	GLsizei		Width, Height;		/* Size of box */
-	GLint		Xmin, Xmax, Ymin, Ymax;	/* Box in device coordinates */
+	GLint		Xmin, Xmax, Ymin, Ymax;	/* Bounds in window coords */
 };
 
 
@@ -420,15 +456,18 @@ struct gl_stencil_attrib {
 };
 
 
+#define Q_BIT 1
+#define R_BIT 2
+#define S_BIT 4
+#define T_BIT 8
+
+
 struct gl_texture_attrib {
 	GLuint Enabled;			/* Bitwise-OR of 1 and 2 */
 	GLenum EnvMode;			/* GL_MODULATE, GL_DECAL, GL_BLEND */
 	GLfloat EnvColor[4];
 	GLfloat BorderColor[4];		/* TODO: border color 1-D and 2-D */
-	GLboolean TexGenEnabledS;	/* Tex coord generation on/off */
-	GLboolean TexGenEnabledT;
-	GLboolean TexGenEnabledR;
-	GLboolean TexGenEnabledQ;
+	GLuint TexGenEnabled;		/* Bitwise-OR of [QRST]_BIT values */
 	GLenum GenModeS;		/* Tex coord generation mode, either */
 	GLenum GenModeT;		/*	GL_OBJECT_LINEAR, or */
 	GLenum GenModeR;		/*	GL_EYE_LINEAR, or    */
@@ -507,15 +546,20 @@ struct gl_list_group {
 #define ALPHA_BIT		0x01
 #define BLEND_BIT		0x02
 #define DEPTH_BIT		0x04
-#define DITHER_BIT		0x08
-#define FOG_BIT			0x10
-#define LOGIC_OP_BIT		0x20
-#define SCISSOR_BIT		0x40
-#define STENCIL_BIT		0x80
+#define FOG_BIT			0x08
+#define LOGIC_OP_BIT		0x10
+#define SCISSOR_BIT		0x20
+#define STENCIL_BIT		0x40
+#define MASKING_BIT		0x80
+#define GAMMA_BIT               0x100
 
-#define RASTER_BITS		0xff
+#define RASTER_BITS		0x1ff
 
 
+typedef void (*points_func)( GLuint first, GLuint last );
+typedef void (*line_func)( GLuint v1, GLuint v2, GLuint pv );
+typedef void (*polygon_func)( GLuint n, GLuint vlist[], GLuint pv );
+
 
 
 /*
@@ -537,6 +581,7 @@ struct gl_context {
 
 	/* Texture matrix and stack */
 	GLfloat TextureMatrix[16];
+        GLboolean IdentityTexMat;	/* Is TextureMatrix==Identity? */
 	GLuint TextureStackDepth;
 	GLfloat TextureStack[MAX_TEXTURE_STACK_DEPTH][16];
 
@@ -604,25 +649,26 @@ struct gl_context {
 
 	/* Current 1-D texture map */
 	GLuint TextureComponents1D[MAX_TEXTURE_LEVELS];
-	GLuint TextureWidth1D[MAX_TEXTURE_LEVELS];
+	GLuint TextureWidth1D[MAX_TEXTURE_LEVELS];  /* includes border */
 	GLuint TextureBorder1D[MAX_TEXTURE_LEVELS];
 	GLubyte *TextureImage1D[MAX_TEXTURE_LEVELS];
 	GLboolean TextureImage1DDeleteFlag[MAX_TEXTURE_LEVELS];
 
 	/* Current 2-D texture map */
 	GLuint TextureComponents2D[MAX_TEXTURE_LEVELS];
-	GLuint TextureWidth2D[MAX_TEXTURE_LEVELS];
-	GLuint TextureHeight2D[MAX_TEXTURE_LEVELS];
+	GLuint TextureWidth2D[MAX_TEXTURE_LEVELS];  /* includes border */
+	GLuint TextureHeight2D[MAX_TEXTURE_LEVELS]; /* includes border */
 	GLuint TextureBorder2D[MAX_TEXTURE_LEVELS];
 	GLubyte *TextureImage2D[MAX_TEXTURE_LEVELS];
 	GLboolean TextureImage2DDeleteFlag[MAX_TEXTURE_LEVELS];
 
 	/* Miscellaneous */
+        GLboolean NewState;     /* If GL_TRUE, must call gl_update_state() */
 	GLenum RenderMode;	/* either GL_RENDER, GL_SELECT, GL_FEEDBACK */
 	GLenum Mode;		/* glBegin primitive or GL_BITMAP */
 
 	GLuint StippleCounter;	/* Line stipple counter */
-	GLuint ClipMask;	/* OR of CLIP_* values */
+	GLuint ClipMask;	/* OR of CLIP_* values from above */
 	GLuint RasterMask;	/* OR of rasterization flags */
 	GLuint LightTwoSide;	/* Compute two-sided lighting? */
 	GLboolean ComputePlane;	/* Compute plane eq. for polygons? */
@@ -630,9 +676,12 @@ struct gl_context {
 	GLfloat PlaneC, PlaneD;
 
 	GLboolean ClipSpans;	/* Clip pixel spans to window? */
+	GLboolean NeedNormals;	/* Are vertex normal vectors needed? */
+        GLboolean FastDrawPixels;/* Use optimized glDrawPixels? */
+        GLboolean MutableColors;/* Can rasterization effect colors? */
 
-	GLboolean RGBAflag;	/* TRUE=Buffer is color, FALSE=color index */
-	GLboolean DBflag;	/* Is this context double buffered? */
+	GLboolean RGBAflag;	/* Is frame buffer in RGBA mode, not CI? */
+	GLboolean DBflag;	/* Is frame buffer double buffered? */
 	GLint BufferWidth;	/* Width of output buffer in pixels */
 	GLint BufferHeight;	/* Height of output buffer in pixels */
 
@@ -643,6 +692,14 @@ struct gl_context {
 				/* the number of bits for the component */
 				/* in the frame buffer. */
 
+        /* Gamma Correction */
+        GLfloat RedGamma;       /* The gamma values, 1=linear */
+        GLfloat GreenGamma;
+        GLfloat BlueGamma;
+        GLubyte RedRamp[256];   /* Gamma lookup tables */
+        GLubyte GreenRamp[256];
+        GLubyte BlueRamp[256];
+
 	/* Special buffers (BS = BufferWidth * BufferHeight): */
 	GLint *DepthBuffer;	/* Zbuffer: array [BS] of ints */
 	GLubyte *StencilBuffer;	/* Stencil buffer: array [BS] of ubytes */
@@ -651,19 +708,11 @@ struct gl_context {
 	GLenum ErrorValue;
 
 	/* Current Primitive functions */
-#ifdef ultrix
-	void (*VertexFunc)();
-	void (*PointFunc)();
-	void (*LineFunc)();
-	void (*PolygonFunc)();
-	void (*AuxPolygonFunc)();
-#else
 	void (*VertexFunc)( GLfloat x, GLfloat y, GLfloat z, GLfloat w );
-	void (*PointFunc)( GLuint p );
-	void (*LineFunc)( GLuint i, GLuint j, GLuint pv );
-	void (*PolygonFunc)( GLuint n, GLuint vlist[], GLuint pv );
-	void (*AuxPolygonFunc)( GLuint n2, GLuint vlist2[], GLuint pv2 );
-#endif
+        points_func PointsFunc;
+        line_func LineFunc;
+        polygon_func PolygonFunc;
+        polygon_func AuxPolygonFunc;
 };
 
 
@@ -679,7 +728,19 @@ extern struct gl_context CC;
 
 extern void gl_initialize_context( struct gl_context *c );
 
+/*
 extern struct gl_context *gl_new_context( struct gl_context *shareList );
+*/
+
+extern struct gl_context *
+gl_new_context( GLboolean rgb_flag,
+                GLfloat redscale,
+                GLfloat greenscale,
+                GLfloat bluescale,
+                GLfloat alphascale,
+                GLboolean db_flag,
+                struct gl_context *shareList );
+
 
 extern void gl_destroy_context( struct gl_context *c );
 
@@ -697,54 +758,7 @@ extern void gl_copy_context( struct gl_context *src, struct gl_context *dst,
 extern void gl_error( GLenum error, char *s );
 
 
-extern void gl_update_pixel_logic( void );
-
-extern void gl_update_rasterflags( void );
-
-
-
-
-/*
- * The following allows Mesa to be compiled with OpenGL header files instead
- * of Mesa's GL/gl.h  Basically, all extension symbols are defined here if
- * not already defined.
- */
-
-
-#ifndef GL_EXT_blend_color
-#  define GL_CONSTANT_COLOR_EXT			((GLenum) 0x8001)
-#  define GL_ONE_MINUS_CONSTANT_COLOR_EXT	((GLenum) 0x8002)
-#  define GL_CONSTANT_ALPHA_EXT			((GLenum) 0x8003)
-#  define GL_ONE_MINUS_CONSTANT_ALPHA_EXT	((GLenum) 0x8004)
-#  define GL_BLEND_COLOR_EXT			((GLenum) 0x8005)
-#endif
-
-#ifndef GL_EXT_blend_minmax
-#  define GL_FUNC_ADD_EXT			((GLenum) 0x8006)
-#  define GL_MIN_EXT				((GLenum) 0x8007)
-#  define GL_MAX_EXT				((GLenum) 0x8008)
-#  define GL_BLEND_EQUATION_EXT			((GLenum) 0x8009)
-#endif
-
-#ifndef GL_EXT_blend_subtract
-#  define GL_FUNC_SUBTRACT_EXT			((GLenum) 0x800A)
-#  define GL_FUNC_REVERSE_SUBTRACT_EXT		((GLenum) 0x800B)
-#endif
-
-
-#ifndef GL_REPLACE_EXT
-#  define GL_REPLACE_EXT			((GLenum) 0x8062)
-#endif
-
-
-#ifndef GLX_SAMPLES_SGIS
-#  define GLX_SAMPLES_SGIS			100000
-#endif
-
-#ifndef GLX_SAMPLE_BUFFER_SGIS
-#  define GLX_SAMPLE_BUFFER_SGIS		100001
-#endif
-
+extern void gl_update_state( void );
 
 
 #endif
