@@ -22,9 +22,30 @@
 
 
 /*
-$Id: drawpix.c,v 1.11 1995/07/24 20:35:20 brianp Exp $
+$Id: drawpix.c,v 1.18 1995/11/09 15:15:29 brianp Exp $
 
 $Log: drawpix.c,v $
+ * Revision 1.18  1995/11/09  15:15:29  brianp
+ * allow glDrawPixels width and height to be zero
+ *
+ * Revision 1.17  1995/11/03  17:41:17  brianp
+ * fixed a few things for C++ compilation
+ *
+ * Revision 1.16  1995/10/19  15:48:51  brianp
+ * replaced a for loop with MEMSET
+ *
+ * Revision 1.15  1995/10/16  15:25:49  brianp
+ * added zooming for stencil drawing/copying
+ *
+ * Revision 1.14  1995/10/14  16:28:07  brianp
+ * added glPixelZoom support
+ *
+ * Revision 1.13  1995/09/28  16:18:48  brianp
+ * replaced a loop in draw_color_pixels with a MEMSET
+ *
+ * Revision 1.12  1995/09/15  18:46:10  brianp
+ * directly call DD.write_color_span under right conditions
+ *
  * Revision 1.11  1995/07/24  20:35:20  brianp
  * replaced memset() with MEMSET() and memcpy() with MEMCPY()
  *
@@ -65,6 +86,7 @@ $Log: drawpix.c,v $
 #include <stdlib.h>
 #include <string.h>
 #include "context.h"
+#include "dd.h"
 #include "feedback.h"
 #include "list.h"
 #include "macros.h"
@@ -74,6 +96,7 @@ $Log: drawpix.c,v $
 
 
 
+
 /* TODO:  apply texture mapping to fragments */
 
 
@@ -81,13 +104,22 @@ $Log: drawpix.c,v $
 static void draw_index_pixels( GLsizei width, GLsizei height,
 			       GLenum type, const GLvoid *pixels )
 {
-   GLint x, y;
+   GLint x, y, desty;
    GLuint i, j;
    GLint zspan[MAX_WIDTH];
+   GLboolean zoom;
+
+   if (CC.Pixel.ZoomX==1.0 && CC.Pixel.ZoomY==1.0) {
+      zoom = GL_FALSE;
+   }
+   else {
+      zoom = GL_TRUE;
+   }
 
    /* Position, depth of pixels */
    x = (GLint) (CC.Current.RasterPos[0] + 0.5F);
    y = (GLint) (CC.Current.RasterPos[1] + 0.5F);
+   desty = y;
    if (CC.Depth.Test) {
       GLint zval = (GLint) (CC.Current.RasterPos[2] * MAX_DEPTH);
       for (i=0;i<width;i++) {
@@ -190,8 +222,14 @@ static void draw_index_pixels( GLsizei width, GLsizei height,
 	    blue[j]  = (GLint) (CC.Pixel.MapItoB[ispan[j]] * CC.BlueScale);
 	    alpha[j] = (GLint) (CC.Pixel.MapItoA[ispan[j]] * CC.AlphaScale);
 	 }
-	 gl_write_color_span( (GLuint) width, x, y, zspan,
-			      red, green, blue, alpha, GL_BITMAP );
+         if (zoom) {
+            gl_write_zoomed_color_span( width, x, y, zspan,
+                                        red, green, blue, alpha, desty );
+         }
+         else {
+            gl_write_color_span( width, x, y, zspan,
+                                 red, green, blue, alpha, GL_BITMAP );
+         }
       }
       else {
 	 /* optionally apply index map then write to frame buffer */
@@ -200,7 +238,12 @@ static void draw_index_pixels( GLsizei width, GLsizei height,
 	       ispan[j] = CC.Pixel.MapItoI[ispan[j]];
 	    }
 	 }
-	 gl_write_index_span( (GLuint) width, x, y, zspan, ispan, GL_BITMAP );
+         if (zoom) {
+            gl_write_zoomed_index_span( width, x, y, zspan, ispan, desty );
+         }
+         else {
+            gl_write_index_span( width, x, y, zspan, ispan, GL_BITMAP );
+         }
       }
    }
 
@@ -211,12 +254,21 @@ static void draw_index_pixels( GLsizei width, GLsizei height,
 static void draw_stencil_pixels( GLsizei width, GLsizei height,
 			         GLenum type, const GLvoid *pixels )
 {
-   GLint x, y;
+   GLint x, y, desty;
    GLuint i, j;
+   GLboolean zoom;
+
+   if (CC.Pixel.ZoomX==1.0 && CC.Pixel.ZoomY==1.0) {
+      zoom = GL_FALSE;
+   }
+   else {
+      zoom = GL_TRUE;
+   }
 
    /* Position, depth of pixels */
    x = (GLint) (CC.Current.RasterPos[0] + 0.5F);
    y = (GLint) (CC.Current.RasterPos[1] + 0.5F);
+   desty = y;
 
    /* process the image row by row */
    for (i=0;i<height;i++,y++) {
@@ -307,7 +359,12 @@ static void draw_stencil_pixels( GLsizei width, GLsizei height,
       }
 
       /* write stencil values to stencil buffer */
-      gl_write_stencil_span( (GLuint) width, x, y, stencil );
+      if (zoom) {
+         gl_write_zoomed_stencil_span( (GLuint) width, x, y, stencil, desty );
+      }
+      else {
+         gl_write_stencil_span( (GLuint) width, x, y, stencil );
+      }
    }
 }
 
@@ -316,16 +373,25 @@ static void draw_stencil_pixels( GLsizei width, GLsizei height,
 static void draw_depth_pixels( GLsizei width, GLsizei height,
 			       GLenum type, const GLvoid *pixels )
 {
-   GLint x, y;
+   GLint x, y, desty;
    GLuint i, j;
    GLfloat depth[MAX_WIDTH];
    GLint zspan[MAX_WIDTH];
    GLubyte red[MAX_WIDTH], green[MAX_WIDTH], blue[MAX_WIDTH], alpha[MAX_WIDTH];
    GLuint ispan[MAX_WIDTH];
+   GLboolean zoom;
+
+   if (CC.Pixel.ZoomX==1.0 && CC.Pixel.ZoomY==1.0) {
+      zoom = GL_FALSE;
+   }
+   else {
+      zoom = GL_TRUE;
+   }
 
    /* Position, depth of pixels */
    x = (GLint) (CC.Current.RasterPos[0] + 0.5F);
    y = (GLint) (CC.Current.RasterPos[1] + 0.5F);
+   desty = y;
 
    /* Color or index */
    if (CC.RGBAflag) {
@@ -334,12 +400,10 @@ static void draw_depth_pixels( GLsizei width, GLsizei height,
       g = (GLint) (CC.Current.RasterColor[1] * CC.GreenScale);
       b = (GLint) (CC.Current.RasterColor[2] * CC.BlueScale);
       a = (GLint) (CC.Current.RasterColor[3] * CC.AlphaScale);
-      for (j=0;j<width;j++) {
-	 red[j]   = r;
-	 green[j] = g;
-	 blue[j]  = b;
-	 alpha[j] = a;
-      }
+      MEMSET( red,   r, width );
+      MEMSET( green, g, width );
+      MEMSET( blue,  b, width );
+      MEMSET( alpha, a, width );
    }
    else {
       for (j=0;j<width;j++) {
@@ -420,11 +484,22 @@ static void draw_depth_pixels( GLsizei width, GLsizei height,
 	 zspan[j] = (GLint) ( CLAMP( depth[j], 0.0, 1.0 ) * MAX_DEPTH);
       }
       if (CC.RGBAflag) {
-	 gl_write_color_span( (GLuint) width, x, y, zspan,
-			      red, green, blue, alpha, GL_BITMAP );
+         if (zoom) {
+            gl_write_zoomed_color_span( width, x, y, zspan,
+                                        red, green, blue, alpha, desty );
+         }
+         else {
+            gl_write_color_span( width, x, y, zspan,
+                                 red, green, blue, alpha, GL_BITMAP );
+         }
       }
       else {
-	 gl_write_index_span( (GLuint) width, x, y, zspan, ispan, GL_BITMAP );
+         if (zoom) {
+            gl_write_zoomed_index_span( width, x, y, zspan, ispan, GL_BITMAP );
+         }
+         else {
+            gl_write_index_span( width, x, y, zspan, ispan, GL_BITMAP );
+         }
       }
 
    }
@@ -437,12 +512,21 @@ static void draw_color_pixels( GLsizei width, GLsizei height, GLenum format,
 			       GLenum type, const GLvoid *pixels )
 {
    GLuint i, j;
-   GLint x, y, zspan[MAX_WIDTH];
-   GLboolean scale_or_bias;
+   GLint x, y, desty, zspan[MAX_WIDTH];
+   GLboolean scale_or_bias, quick_draw;
+   GLboolean zoom;
+
+   if (CC.Pixel.ZoomX==1.0 && CC.Pixel.ZoomY==1.0) {
+      zoom = GL_FALSE;
+   }
+   else {
+      zoom = GL_TRUE;
+   }
 
    /* Position, depth of pixels */
    x = (GLint) (CC.Current.RasterPos[0] + 0.5F);
    y = (GLint) (CC.Current.RasterPos[1] + 0.5F);
+   desty = y;
    if (CC.Depth.Test) {
       /* fill in array of z values */
       GLint z = (GLint) (CC.Current.RasterPos[2] * MAX_DEPTH);
@@ -462,24 +546,49 @@ static void draw_color_pixels( GLsizei width, GLsizei height, GLenum format,
       scale_or_bias = GL_FALSE;
    }
 
+   /* Determine if we can directly call the device driver function */
+   if (CC.RasterMask==0 && !zoom && x>=0 && y>=0
+       && x+width<=CC.BufferWidth && y+height<=CC.BufferHeight) {
+      quick_draw = GL_TRUE;
+   }
+   else {
+      quick_draw = GL_FALSE;
+   }
+
    /* First check for common cases */
    if (type==GL_UNSIGNED_BYTE && format==GL_RGB
        && !CC.Pixel.MapColorFlag && !scale_or_bias
        && CC.RedScale==255.0 && CC.GreenScale==255.0 && CC.BlueScale==255.0) {
       /* 8-bit RGB pixels */
+      DEFARRAY( GLubyte, red, MAX_WIDTH );
+      DEFARRAY( GLubyte, green, MAX_WIDTH );
+      DEFARRAY( GLubyte, blue, MAX_WIDTH );
+      DEFARRAY( GLubyte, alpha, MAX_WIDTH );
       GLubyte *src = (GLubyte *) pixels;
+      /* constant alpha */
+      MEMSET( alpha, (GLint) CC.AlphaScale, width );
       for (i=0;i<height;i++,y++) {
-	 GLubyte red[MAX_WIDTH], green[MAX_WIDTH];
-	 GLubyte blue[MAX_WIDTH], alpha[MAX_WIDTH];
 	 for (j=0;j<width;j++) {
 	    red[j]   = *src++;
 	    green[j] = *src++;
 	    blue[j]  = *src++;
-	    alpha[j] = (GLint) CC.AlphaScale;
 	 }
-	 gl_write_color_span( (GLuint) width, x, y, zspan,
-			      red, green, blue, alpha, GL_BITMAP );
+         if (quick_draw) {
+            (*DD.write_color_span)( width, x,y, red, green, blue, alpha, NULL);
+         }
+         else if (zoom) {
+            gl_write_zoomed_color_span( (GLuint) width, x, y, zspan,
+                                        red, green, blue, alpha, desty );
+         }
+         else {
+            gl_write_color_span( (GLuint) width, x, y, zspan,
+                                 red, green, blue, alpha, GL_BITMAP );
+         }
       }
+      UNDEFARRAY( red );
+      UNDEFARRAY( green );
+      UNDEFARRAY( blue );
+      UNDEFARRAY( alpha );
    }
    else {
       /* General solution */
@@ -689,8 +798,17 @@ static void draw_color_pixels( GLsizei width, GLsizei height, GLenum format,
 	 }
 
 	 /* write to frame buffer */
-	 gl_write_color_span( (GLuint) width, x, y, zspan,
-			      red, green, blue, alpha, GL_BITMAP );
+         if (quick_draw) {
+            (*DD.write_color_span)( width, x,y, red, green, blue, alpha, NULL);
+         }
+         else if (zoom) {
+            gl_write_zoomed_color_span( width, x, y, zspan,
+                                        red, green, blue, alpha, desty );
+         }
+         else {
+            gl_write_color_span( (GLuint) width, x, y, zspan,
+                                 red, green, blue, alpha, GL_BITMAP );
+         }
       }
    }
 
@@ -698,6 +816,54 @@ static void draw_color_pixels( GLsizei width, GLsizei height, GLenum format,
 
 
 
+/*** EXPERIMENTAL:  THIS ISN'T USED YET ***/
+#ifdef LEAVEOUT
+/*
+ * Do a glDrawPixels( w, h, GL_RGB, GL_UNSIGNED_BYTE, pixels ) optimized
+ * for the case of no pixel mapping, no scale, no bias, no zoom, default
+ * storage mode, no raster ops, and no pixel clipping.
+ */
+static void quickdraw_rgb( GLsizei width, GLsizei height, const void *pixels )
+{
+   DEFARRAY( GLubyte, red, MAX_WIDTH );
+   DEFARRAY( GLubyte, green, MAX_WIDTH );
+   DEFARRAY( GLubyte, blue, MAX_WIDTH );
+   DEFARRAY( GLubyte, alpha, MAX_WIDTH );
+   GLint i, j;
+   GLint x, y;
+   GLint bytes_per_row;
+
+   bytes_per_row = width * 3 + (width%4);
+
+   x = (GLint) (CC.Current.RasterPos[0] + 0.5F);
+   y = (GLint) (CC.Current.RasterPos[1] + 0.5F);
+
+   /* constant alpha */
+   for (j=0;j<width;j++) {
+      alpha[j] = (GLint) CC.AlphaScale;
+   }
+
+   /* write directly to device driver */
+   for (i=0;i<height;i++) {
+      /* each row of pixel data starts at 4-byte boundary */
+      GLubyte *src = (GLubyte *) pixels + i * bytes_per_row;
+      for (j=0;j<width;j++) {
+         red[j]   = *src++;
+         green[j] = *src++;
+         blue[j]  = *src++;
+      }
+      (*DD.write_color_span)( width, x, y+i, red, green, blue, alpha, NULL);
+   }
+
+   UNDEFARRAY( red );
+   UNDEFARRAY( green );
+   UNDEFARRAY( blue );
+   UNDEFARRAY( alpha );
+}
+#endif
+
+
+
 void gl_drawpixels( GLsizei width, GLsizei height,
 		    GLenum format, GLenum type, const GLvoid *pixels )
 {
@@ -706,13 +872,17 @@ void gl_drawpixels( GLsizei width, GLsizei height,
       return;
    }
 
+   if (CC.NewState) {
+      gl_update_state();
+   }
+
    if (CC.RenderMode==GL_RENDER) {
       if (!CC.Current.RasterPosValid) {
 	 return;
       }
       switch (format) {
 	 case GL_COLOR_INDEX:
-	    draw_index_pixels( width, height, type, pixels );
+            draw_index_pixels( width, height, type, pixels );
 	    break;
 	 case GL_STENCIL_INDEX:
 	    draw_stencil_pixels( width, height, type, pixels );
@@ -728,7 +898,7 @@ void gl_drawpixels( GLsizei width, GLsizei height,
 	 case GL_LUMINANCE:
 	 case GL_LUMINANCE_ALPHA:
 	 case GL_RGBA:
-	    draw_color_pixels( width, height, format, type, pixels );
+            draw_color_pixels( width, height, format, type, pixels );
 	    break;
 	 default:
 	    gl_error( GL_INVALID_ENUM, "glDrawPixels" );
@@ -762,7 +932,7 @@ void glDrawPixels( GLsizei width, GLsizei height,
 {
    GLvoid *image;
 
-   if (width<=0 || height<=0) {
+   if (width<0 || height<0) {
       gl_error( GL_INVALID_VALUE, "glDrawPixels" );
       return;
    }
