@@ -22,9 +22,18 @@
 
 
 /*
-$Id: get.c,v 1.14 1995/07/12 21:19:02 brianp Exp $
+$Id: get.c,v 1.17 1995/10/27 20:29:02 brianp Exp $
 
 $Log: get.c,v $
+ * Revision 1.17  1995/10/27  20:29:02  brianp
+ * added glPolygonOffsetEXT() support
+ *
+ * Revision 1.16  1995/10/14  16:29:18  brianp
+ * replaced dd_buffer_info with DD.buffer_size
+ *
+ * Revision 1.15  1995/08/31  21:29:33  brianp
+ * new TexGenEnabled bitfield
+ *
  * Revision 1.14  1995/07/12  21:19:02  brianp
  * bug fix:  glGet(GL_DEPTH_BITS) returned bytes, not bits (per Tor Stenvaag)
  *
@@ -345,7 +354,7 @@ void glGetBooleanv( GLenum pname, GLboolean *params )
       case GL_INDEX_BITS:
 	 {
 	    GLuint width, height, depth;
-	    dd_buffer_info( &width, &height, &depth );
+	    (*DD.buffer_size)( &width, &height, &depth );
 	    *params = INT_TO_BOOL(depth);
 	 }
 	 break;
@@ -632,6 +641,12 @@ void glGetBooleanv( GLenum pname, GLboolean *params )
 	 params[0] = ENUM_TO_BOOL(CC.Polygon.FrontMode);
 	 params[1] = ENUM_TO_BOOL(CC.Polygon.BackMode);
 	 break;
+      case GL_POLYGON_OFFSET_BIAS_EXT:
+         *params = FLOAT_TO_BOOL( CC.Polygon.OffsetBias );
+         break;
+      case GL_POLYGON_OFFSET_FACTOR_EXT:
+         *params = FLOAT_TO_BOOL( CC.Polygon.OffsetFactor );
+         break;
       case GL_POLYGON_SMOOTH:
 	 *params = CC.Polygon.SmoothFlag;
 	 break;
@@ -731,16 +746,16 @@ void glGetBooleanv( GLenum pname, GLboolean *params )
 	 *params = ENUM_TO_BOOL(CC.Texture.EnvMode);
 	 break;
       case GL_TEXTURE_GEN_S:
-	 *params = CC.Texture.TexGenEnabledS;
+	 *params = (CC.Texture.TexGenEnabled & S_BIT) ? GL_TRUE : GL_FALSE;
 	 break;
       case GL_TEXTURE_GEN_T:
-	 *params = CC.Texture.TexGenEnabledT;
+	 *params = (CC.Texture.TexGenEnabled & T_BIT) ? GL_TRUE : GL_FALSE;
 	 break;
       case GL_TEXTURE_GEN_R:
-	 *params = CC.Texture.TexGenEnabledR;
+	 *params = (CC.Texture.TexGenEnabled & R_BIT) ? GL_TRUE : GL_FALSE;
 	 break;
       case GL_TEXTURE_GEN_Q:
-	 *params = CC.Texture.TexGenEnabledQ;
+	 *params = (CC.Texture.TexGenEnabled & Q_BIT) ? GL_TRUE : GL_FALSE;
 	 break;
       case GL_TEXTURE_MATRIX:
 	 for (i=0;i<16;i++) {
@@ -1026,7 +1041,7 @@ void glGetDoublev( GLenum pname, GLdouble *params )
       case GL_INDEX_BITS:
 	 {
 	    GLuint width, height, depth;
-	    dd_buffer_info( &width, &height, &depth );
+	    (*DD.buffer_size)( &width, &height, &depth );
 	    *params = (GLdouble) depth;
 	 }
 	 break;
@@ -1313,6 +1328,12 @@ void glGetDoublev( GLenum pname, GLdouble *params )
 	 params[0] = (GLdouble) CC.Polygon.FrontMode;
 	 params[1] = (GLdouble) CC.Polygon.BackMode;
 	 break;
+      case GL_POLYGON_OFFSET_BIAS_EXT:
+         *params = (GLdouble) CC.Polygon.OffsetBias;
+         break;
+      case GL_POLYGON_OFFSET_FACTOR_EXT:
+         *params = (GLdouble) CC.Polygon.OffsetFactor;
+         break;
       case GL_POLYGON_SMOOTH:
 	 *params = (GLdouble) CC.Polygon.SmoothFlag;
 	 break;
@@ -1414,16 +1435,16 @@ void glGetDoublev( GLenum pname, GLdouble *params )
 	 *params = (GLdouble) CC.Texture.EnvMode;
 	 break;
       case GL_TEXTURE_GEN_S:
-	 *params = (GLdouble) CC.Texture.TexGenEnabledS;
+	 *params = (CC.Texture.TexGenEnabled & S_BIT) ? 1.0 : 0.0;
 	 break;
       case GL_TEXTURE_GEN_T:
-	 *params = (GLdouble) CC.Texture.TexGenEnabledT;
+	 *params = (CC.Texture.TexGenEnabled & T_BIT) ? 1.0 : 0.0;
 	 break;
       case GL_TEXTURE_GEN_R:
-	 *params = (GLdouble) CC.Texture.TexGenEnabledR;
+	 *params = (CC.Texture.TexGenEnabled & R_BIT) ? 1.0 : 0.0;
 	 break;
       case GL_TEXTURE_GEN_Q:
-	 *params = (GLdouble) CC.Texture.TexGenEnabledQ;
+	 *params = (CC.Texture.TexGenEnabled & Q_BIT) ? 1.0 : 0.0;
 	 break;
       case GL_TEXTURE_MATRIX:
          for (i=0;i<16;i++) {
@@ -1707,7 +1728,7 @@ void glGetFloatv( GLenum pname, GLfloat *params )
       case GL_INDEX_BITS:
 	 {
 	    GLuint width, height, depth;
-	    dd_buffer_info( &width, &height, &depth );
+	    (*DD.buffer_size)( &width, &height, &depth );
 	    *params = (GLfloat) depth;
 	 }
 	 break;
@@ -1994,6 +2015,12 @@ void glGetFloatv( GLenum pname, GLfloat *params )
 	 params[0] = (GLfloat) CC.Polygon.FrontMode;
 	 params[1] = (GLfloat) CC.Polygon.BackMode;
 	 break;
+      case GL_POLYGON_OFFSET_BIAS_EXT:
+         *params = CC.Polygon.OffsetBias;
+         break;
+      case GL_POLYGON_OFFSET_FACTOR_EXT:
+         *params = CC.Polygon.OffsetFactor;
+         break;
       case GL_POLYGON_SMOOTH:
 	 *params = (GLfloat) CC.Polygon.SmoothFlag;
 	 break;
@@ -2095,16 +2122,16 @@ void glGetFloatv( GLenum pname, GLfloat *params )
 	 *params = (GLfloat) CC.Texture.EnvMode;
 	 break;
       case GL_TEXTURE_GEN_S:
-	 *params = (GLfloat) CC.Texture.TexGenEnabledS;
+	 *params = (CC.Texture.TexGenEnabled & S_BIT) ? 1.0 : 0.0;
 	 break;
       case GL_TEXTURE_GEN_T:
-	 *params = (GLfloat) CC.Texture.TexGenEnabledT;
+	 *params = (CC.Texture.TexGenEnabled & T_BIT) ? 1.0 : 0.0;
 	 break;
       case GL_TEXTURE_GEN_R:
-	 *params = (GLfloat) CC.Texture.TexGenEnabledR;
+	 *params = (CC.Texture.TexGenEnabled & R_BIT) ? 1.0 : 0.0;
 	 break;
       case GL_TEXTURE_GEN_Q:
-	 *params = (GLfloat) CC.Texture.TexGenEnabledQ;
+	 *params = (CC.Texture.TexGenEnabled & Q_BIT) ? 1.0 : 0.0;
 	 break;
       case GL_TEXTURE_MATRIX:
          for (i=0;i<16;i++) {
@@ -2389,7 +2416,7 @@ void glGetIntegerv( GLenum pname, GLint *params )
       case GL_INDEX_BITS:
 	 {
 	    GLuint width, height, depth;
-	    dd_buffer_info( &width, &height, &depth );
+	    (*DD.buffer_size)( &width, &height, &depth );
 	    *params = (GLint) depth;
 	 }
          break;
@@ -2676,6 +2703,12 @@ void glGetIntegerv( GLenum pname, GLint *params )
 	 params[0] = (GLint) CC.Polygon.FrontMode;
 	 params[1] = (GLint) CC.Polygon.BackMode;
 	 break;
+      case GL_POLYGON_OFFSET_BIAS_EXT:
+         *params = (GLint) CC.Polygon.OffsetBias;
+         break;
+      case GL_POLYGON_OFFSET_FACTOR_EXT:
+         *params = (GLint) CC.Polygon.OffsetFactor;
+         break;
       case GL_POLYGON_SMOOTH:
 	 *params = (GLint) CC.Polygon.SmoothFlag;
 	 break;
@@ -2777,16 +2810,16 @@ void glGetIntegerv( GLenum pname, GLint *params )
 	 *params = (GLint) CC.Texture.EnvMode;
 	 break;
       case GL_TEXTURE_GEN_S:
-	 *params = (GLint) CC.Texture.TexGenEnabledS;
+	 *params = (CC.Texture.TexGenEnabled & S_BIT) ? 1 : 0;
 	 break;
       case GL_TEXTURE_GEN_T:
-	 *params = (GLint) CC.Texture.TexGenEnabledT;
+	 *params = (CC.Texture.TexGenEnabled & T_BIT) ? 1 : 0;
 	 break;
       case GL_TEXTURE_GEN_R:
-	 *params = (GLint) CC.Texture.TexGenEnabledR;
+	 *params = (CC.Texture.TexGenEnabled & R_BIT) ? 1 : 0;
 	 break;
       case GL_TEXTURE_GEN_Q:
-	 *params = (GLint) CC.Texture.TexGenEnabledQ;
+	 *params = (CC.Texture.TexGenEnabled & Q_BIT) ? 1 : 0;
 	 break;
       case GL_TEXTURE_MATRIX:
          for (i=0;i<16;i++) {
