@@ -22,9 +22,42 @@
 
 
 /*
-$Id: glx.c,v 1.28 1995/08/01 20:56:39 brianp Exp $
+$Id: glx.c,v 1.38 1995/11/30 00:20:20 brianp Exp $
 
 $Log: glx.c,v $
+ * Revision 1.38  1995/11/30  00:20:20  brianp
+ * changed version strings to 1.2.5
+ *
+ * Revision 1.37  1995/11/20  20:49:47  brianp
+ * check for HP color recovery atom before choosing 8-bit Pseudo over TrueColor
+ *
+ * Revision 1.36  1995/11/03  17:39:08  brianp
+ * changed visual->class to visual->c_class for C++
+ *
+ * Revision 1.35  1995/11/01  15:13:40  brianp
+ * renamed all class variables per Steven Spitz
+ *
+ * Revision 1.34  1995/10/14  16:31:52  brianp
+ * don't make copy of XVisualInfo anymore
+ * cleaned up some code, bumped version string to 1.2.4
+ *
+ * Revision 1.33  1995/09/20  18:21:15  brianp
+ * support for 4-bit visuals added
+ *
+ * Revision 1.32  1995/09/15  18:52:32  brianp
+ * updated version strings to 1.2.3
+ *
+ * Revision 1.31  1995/09/05  19:59:17  brianp
+ * glXCreateContext() accepts non-glXChooseVisual() visuals per Wolfram Gloger
+ *
+ * Revision 1.30  1995/09/05  15:35:48  brianp
+ * bitcount() was incorrect
+ * changed find_glx_visual() to match IDs per Armin Liebchen
+ *
+ * Revision 1.29  1995/08/24  14:30:08  brianp
+ * try to match root visual in get_visual(), per Asif Khan
+ * pick 8-bit PseudoColor before 8-bit TrueColor in choose_x_visual()
+ *
  * Revision 1.28  1995/08/01  20:56:39  brianp
  * lots of new GL/X visual handling code
  *
@@ -118,12 +151,14 @@ $Log: glx.c,v $
 
 
 /*
- * A pseudo-GLX implementation to allow OpenGL/GLX programs to work with
- * Mesa.  Initial version contributed by Philip Brown
- * (philb@CSUA.Berkeley.EDU).  Thanks Philip!
+ * A pseudo-GLX implementation to allow OpenGL/GLX programs to work with Mesa.
  *
- * Better glXGetConfig() support contributed by Armin Liebchen
- * (liebchen@asylum.cs.utah.edu).  Thanks Armin!
+ * Thanks to the contributors:
+ *
+ * Initial version:  Philip Brown (philb@CSUA.Berkeley.EDU)
+ * Better glXGetConfig() support: Armin Liebchen (liebchen@asylum.cs.utah.edu)
+ * Further visual-handling refinements: Wolfram Gloger
+ *    (wmglo@Dent.MED.Uni-Muenchen.DE).
  */
 
 
@@ -140,7 +175,6 @@ $Log: glx.c,v $
 #include "config.h"
 #include "context.h"
 #include "dd.h"
-#include "macros.h"
 
 
 
@@ -148,11 +182,11 @@ $Log: glx.c,v $
  * Since we don't have a real GLX extension, the XVisualInfo doesn't
  * carry all the info we need such as double buffer mode, depth buffer, etc.
  * We use a table to associate this extra information with each XVisualInfo
- * structure we return from glXChooseVisual.
+ * structure we encounter in glXChooseVisual() or glXGetConfig().
  */
 struct glx_visual {
 	Display *dpy;
-	XVisualInfo *visinfo;		/* Returned from glXChooseVisual */
+	XVisualInfo *visinfo;		/* The X information */
 	GLboolean rgba_buffer_flag;	/* RGBA mode? */
 	GLboolean double_buffer_flag;	/* double buffered? */
 	GLboolean depth_buffer_flag;	/* depth buffer? */
@@ -181,43 +215,29 @@ static int NumPixmaps = 0;
 
 /*
  * Given an XVisualInfo and RGB, Double, and Depth buffer flags, save the
- * configuration in our list and return a new XVisualInfo corresponding to
- * the config.
+ * configuration in our list of GLX visuals.
  */
-static XVisualInfo *save_glx_visual( Display *dpy, XVisualInfo *vinfo,
-				     GLboolean rgba,
-				     GLboolean dbl,
-				     GLboolean depth )
+static void save_glx_visual( Display *dpy, XVisualInfo *vinfo,
+                             GLboolean rgba, GLboolean dbl, GLboolean depth )
 {
-   XVisualInfo *newvis;
-   int i;
-
-   /* Make a copy of the XVisualInfo so it becomes a unique GLX Visual Info */
-   newvis = (XVisualInfo *) malloc( sizeof(XVisualInfo) );
-   if (!newvis) {
-      return NULL;
-   }
-   MEMCPY( newvis, vinfo, sizeof(XVisualInfo) );
-
    if (NumVisuals+1<MAX_VISUALS) {
       VisualTable[NumVisuals].dpy = dpy;
-      VisualTable[NumVisuals].visinfo = newvis;
+      VisualTable[NumVisuals].visinfo = vinfo;
       VisualTable[NumVisuals].rgba_buffer_flag = rgba;
       VisualTable[NumVisuals].double_buffer_flag = dbl;
       VisualTable[NumVisuals].depth_buffer_flag = depth;
       NumVisuals++;
-      return newvis;
    }
    else {
       fprintf( stderr, "GLX Error: maximum number of visuals exceeded\n");
-      return NULL;
    }
 }
 
 
 
 /*
- * Find the GLX visual associated with an XVisualInfo.
+ * Find the GLX visual associated with an XVisualInfo then return the
+ * RGBA, double-buffering, and depth-buffer flags.
  */
 static GLboolean find_glx_visual( Display *dpy, XVisualInfo *vinfo,
 				  GLboolean *rgba,
@@ -226,8 +246,19 @@ static GLboolean find_glx_visual( Display *dpy, XVisualInfo *vinfo,
 {
    int i;
 
+   /* First try to match pointers */
    for (i=0;i<NumVisuals;i++) {
-      if (VisualTable[i].dpy==dpy && VisualTable[i].visinfo == vinfo) {
+      if (VisualTable[i].dpy==dpy && VisualTable[i].visinfo==vinfo) {
+	 *rgba  = VisualTable[i].rgba_buffer_flag;
+	 *dbl   = VisualTable[i].double_buffer_flag;
+	 *depth = VisualTable[i].depth_buffer_flag;
+	 return GL_TRUE;
+      }
+   }
+   /* try to match visual id */
+   for (i=0;i<NumVisuals;i++) {
+      if (VisualTable[i].dpy==dpy
+          && VisualTable[i].visinfo->visualid == vinfo->visualid) {
 	 *rgba  = VisualTable[i].rgba_buffer_flag;
 	 *dbl   = VisualTable[i].double_buffer_flag;
 	 *depth = VisualTable[i].depth_buffer_flag;
@@ -240,19 +271,23 @@ static GLboolean find_glx_visual( Display *dpy, XVisualInfo *vinfo,
 
 
 /*
- * Test if the given XVisualInfo is usable for GL rendering.
+ * Test if the given XVisualInfo is usable for Mesa rendering.
  */
 static GLboolean is_usable_visual( XVisualInfo *vinfo )
 {
+#if defined(__cplusplus) || defined(c_plusplus)
+   switch (vinfo->c_class) {
+#else
    switch (vinfo->class) {
+#endif
       case StaticGray:
       case GrayScale:
          /* Any StaticGray/GrayScale visual works in RGB or CI mode */
          return GL_TRUE;
       case StaticColor:
       case PseudoColor:
-	 /* Any StaticColor/PseudoColor visual of at least 8 bits */
-	 if (vinfo->depth>=8) {
+	 /* Any StaticColor/PseudoColor visual of at least 4 bits */
+	 if (vinfo->depth>=4) {
 	    return GL_TRUE;
 	 }
 	 else {
@@ -275,18 +310,37 @@ static GLboolean is_usable_visual( XVisualInfo *vinfo )
  * Try to get an X visual which matches the given arguments.
  */
 static XVisualInfo *get_visual( Display *dpy, int scr,
-			        unsigned int depth, int class )
+			        unsigned int depth, int xclass )
 {
-   XVisualInfo template;
+   XVisualInfo temp;
    long mask;
    int n;
+   int default_depth;
+   int default_class;
 
    mask = VisualScreenMask | VisualDepthMask | VisualClassMask;
-   template.screen = scr;
-   template.depth = depth;
-   template.class = class;
+   temp.screen = scr;
+   temp.depth = depth;
+#if defined(__cplusplus) || defined(c_plusplus)
+   temp.c_class = xclass;
+#else
+   temp.class = xclass;
+#endif
 
-   return XGetVisualInfo( dpy, mask, &template, &n );
+   default_depth = DefaultDepth(dpy,scr);
+#if defined(__cplusplus) || defined(c_plusplus)
+   default_class = DefaultVisual(dpy,scr)->c_class;
+#else
+   default_class = DefaultVisual(dpy,scr)->class;
+#endif
+
+   if (depth==default_depth && xclass==default_class) {
+      /* try to get root window's visual */
+      temp.visualid = DefaultVisual(dpy,scr)->visualid;
+      mask |= VisualIDMask;
+   }
+
+   return XGetVisualInfo( dpy, mask, &temp, &n );
 }
 
 
@@ -303,7 +357,7 @@ static XVisualInfo *get_env_visual( Display *dpy, int scr, char *varname )
 {
    char *value;
    char type[100];
-   int depth, class = -1;
+   int depth, xclass = -1;
    XVisualInfo *vis;
 
    value = getenv( varname );
@@ -313,15 +367,15 @@ static XVisualInfo *get_env_visual( Display *dpy, int scr, char *varname )
 
    sscanf( value, "%s %d", type, &depth );
 
-   if (strcmp(type,"TrueColor")==0)          class = TrueColor;
-   else if (strcmp(type,"DirectColor")==0)   class = DirectColor;
-   else if (strcmp(type,"PseudoColor")==0)   class = PseudoColor;
-   else if (strcmp(type,"StaticColor")==0)   class = StaticColor;
-   else if (strcmp(type,"GrayScale")==0)     class = GrayScale;
-   else if (strcmp(type,"StaticGray")==0)    class = StaticGray;
+   if (strcmp(type,"TrueColor")==0)          xclass = TrueColor;
+   else if (strcmp(type,"DirectColor")==0)   xclass = DirectColor;
+   else if (strcmp(type,"PseudoColor")==0)   xclass = PseudoColor;
+   else if (strcmp(type,"StaticColor")==0)   xclass = StaticColor;
+   else if (strcmp(type,"GrayScale")==0)     xclass = GrayScale;
+   else if (strcmp(type,"StaticGray")==0)    xclass = StaticGray;
 
-   if (class>-1 && depth>0) {
-      vis = get_visual( dpy, scr, depth, class );
+   if (xclass>-1 && depth>0) {
+      vis = get_visual( dpy, scr, depth, xclass );
       if (vis) {
 	 return vis;
       }
@@ -345,18 +399,19 @@ static XVisualInfo *choose_x_visual( Display *dpy, int screen,
 				     GLboolean rgba, int min_depth )
 {
    XVisualInfo *vis;
-   int class, visclass;
+   int xclass, visclass;
    int depth;
 
    if (rgba) {
+      Atom hp_cr_maps = XInternAtom(dpy, "_HP_RGB_SMOOTH_MAP_LIST", True);
       /* First see if the MESA_RGB_VISUAL env var is defined */
       vis = get_env_visual( dpy, screen, "MESA_RGB_VISUAL" );
       if (vis) {
 	 return vis;
       }
       /* Otherwise, search for a suitable visual */
-      for (class=0;class<6;class++) {
-	 switch (class) {
+      for (xclass=0;xclass<6;xclass++) {
+	 switch (xclass) {
 	    case 0:  visclass = TrueColor;    break;
 	    case 1:  visclass = DirectColor;  break;
 	    case 2:  visclass = PseudoColor;  break;
@@ -367,6 +422,14 @@ static XVisualInfo *choose_x_visual( Display *dpy, int screen,
 	 if (min_depth==0) {
 	    /* start with shallowest */
 	    for (depth=0;depth<=24;depth++) {
+               if (visclass==TrueColor && depth==8 && !hp_cr_maps) {
+                  /* Special case:  try to get 8-bit PseudoColor before */
+                  /* 8-bit TrueColor */
+                  vis = get_visual( dpy, screen, 8, PseudoColor );
+                  if (vis) {
+                     return vis;
+                  }
+               }
 	       vis = get_visual( dpy, screen, depth, visclass );
 	       if (vis) {
 		  return vis;
@@ -376,6 +439,14 @@ static XVisualInfo *choose_x_visual( Display *dpy, int screen,
 	 else {
 	    /* start with deepest */
 	    for (depth=24;depth>=min_depth;depth--) {
+               if (visclass==TrueColor && depth==8 && !hp_cr_maps) {
+                  /* Special case:  try to get 8-bit PseudoColor before */
+                  /* 8-bit TrueColor */
+                  vis = get_visual( dpy, screen, 8, PseudoColor );
+                  if (vis) {
+                     return vis;
+                  }
+               }
 	       vis = get_visual( dpy, screen, depth, visclass );
 	       if (vis) {
 		  return vis;
@@ -391,8 +462,8 @@ static XVisualInfo *choose_x_visual( Display *dpy, int screen,
 	 return vis;
       }
       /* Otherwise, search for a suitable visual, starting with shallowest */
-      for (class=0;class<4;class++) {
-	 switch (class) {
+      for (xclass=0;xclass<4;xclass++) {
+	 switch (xclass) {
 	    case 0:  visclass = PseudoColor;  break;
 	    case 1:  visclass = StaticColor;  break;
 	    case 2:  visclass = GrayScale;    break;
@@ -426,15 +497,17 @@ static XVisualInfo *choose_x_visual( Display *dpy, int screen,
 static int bitcount( unsigned long n )
 {
    int bits;
-   for (bits=0; n>0; bits++) {
-      n = n >> 1;
+   for (bits=0; n>0; n=n>>1) {
+      if (n&1) {
+         bits++;
+      }
    }
    return bits;
 }
 
 
 
-XVisualInfo *glXChooseVisual(Display *dpy, int screen, int *list)
+XVisualInfo *glXChooseVisual( Display *dpy, int screen, int *list )
 {
    int *parselist;
    XVisualInfo *vis;
@@ -447,9 +520,9 @@ XVisualInfo *glXChooseVisual(Display *dpy, int screen, int *list)
 
    parselist = list;
 
-   while(*parselist) {
+   while (*parselist) {
 
-      switch(*parselist) {
+      switch (*parselist) {
 	 case GLX_USE_GL:
 	    /* ignore */
 	    parselist++;
@@ -559,7 +632,7 @@ XVisualInfo *glXChooseVisual(Display *dpy, int screen, int *list)
    }
 
    if (vis) {
-      vis = save_glx_visual( dpy, vis, rgba_flag, double_flag, depth_flag );
+      save_glx_visual( dpy, vis, rgba_flag, double_flag, depth_flag );
    }
 
    return vis;
@@ -571,35 +644,47 @@ XVisualInfo *glXChooseVisual(Display *dpy, int screen, int *list)
 GLXContext glXCreateContext( Display *dpy, XVisualInfo *visinfo,
 			     GLXContext share_list, Bool direct )
 {
+   XMesaContext ctx;
+   GLboolean ximage_flag = GL_TRUE;
    GLboolean rgba_flag, double_flag, depth_flag;
 
-   if (find_glx_visual(dpy, visinfo, &rgba_flag, &double_flag, &depth_flag)) {
-      XMesaContext ctx;
-      GLboolean ximage_flag = GL_TRUE;
-      if (double_flag) {
-	 /* Check if the MESA_BACK_BUFFER env var is set */
-	 char *backbuffer = getenv("MESA_BACK_BUFFER");
-	 if (backbuffer) {
-	    if (backbuffer[0]=='p' || backbuffer[0]=='P') {
-	       ximage_flag = GL_FALSE;
-	    }
-	    else if (backbuffer[0]=='x' || backbuffer[0]=='X') {
-	       ximage_flag = GL_TRUE;
-	    }
-	    else {
-	       fprintf(stderr, "Mesa: invalid value for MESA_BACK_BUFFER ");
-	       fprintf(stderr, "environment variable, using an XImage.\n");
-	    }
-	 }
+   if (!find_glx_visual(dpy, visinfo, &rgba_flag, &double_flag, &depth_flag)) {
+      /* This visual wasn't found with glXChooseVisual() */
+      if (is_usable_visual( visinfo )) {
+         /* Configure this visual as RGB, double-buffered, depth-buffered. */
+         /* This is surely wrong for some people's needs but what else */
+         /* can be done?  They should use glXChooseVisual(). */
+         rgba_flag = GL_TRUE;
+         double_flag = GL_TRUE;
+         depth_flag = GL_TRUE;
+         save_glx_visual( dpy, visinfo, rgba_flag, double_flag, depth_flag );
       }
-      ctx = XMesaCreateContext( dpy, visinfo, rgba_flag, double_flag,
+      else {
+         fprintf(stderr,"Mesa: error in glXCreateContext: bad visual\n");
+         return NULL;
+      }
+   }
+
+   if (double_flag) {
+      /* Check if the MESA_BACK_BUFFER env var is set */
+      char *backbuffer = getenv("MESA_BACK_BUFFER");
+      if (backbuffer) {
+         if (backbuffer[0]=='p' || backbuffer[0]=='P') {
+            ximage_flag = GL_FALSE;
+         }
+         else if (backbuffer[0]=='x' || backbuffer[0]=='X') {
+            ximage_flag = GL_TRUE;
+         }
+         else {
+            fprintf(stderr, "Mesa: invalid value for MESA_BACK_BUFFER ");
+            fprintf(stderr, "environment variable, using an XImage.\n");
+         }
+      }
+   }
+
+   ctx = XMesaCreateContext( dpy, visinfo, rgba_flag, double_flag,
 			        ximage_flag, (XMesaContext) share_list );
-      return (GLXContext) ctx;
-   }
-   else {
-      fprintf(stderr,"Mesa: error in glXCreateContext: bad visual\n");
-      return NULL;
-   }
+   return (GLXContext) ctx;
 }
 
 
@@ -726,6 +811,7 @@ void glXSwapBuffers( Display *dpy, GLXDrawable drawable )
 
 Bool glXQueryVersion( Display *dpy, int *maj, int *min )
 {
+   /* Return GLX version, not Mesa version */
    *maj = 1;
    *min = 1;
    return True;
@@ -740,15 +826,6 @@ int glXGetConfig( Display *dpy, XVisualInfo *visual,
 		  int attrib, int *value )
 {
    GLboolean rgba_flag, double_flag, depth_flag;
-#ifdef LEAVEOUT
-   static GLboolean load_visuals = GL_TRUE;
-
-   if (load_visuals) {
-      /* load the visual list once. */
-      load_all_glx_visuals( dpy );
-      load_visuals = GL_FALSE;
-   }
-#endif
 
    if (!find_glx_visual(dpy, visual, &rgba_flag, &double_flag, &depth_flag)) {
       /* this visual wasn't obtained with glXChooseVisual */
@@ -757,6 +834,7 @@ int glXGetConfig( Display *dpy, XVisualInfo *visual,
 	 rgba_flag = GL_TRUE;
 	 double_flag = GL_TRUE;
 	 depth_flag = GL_TRUE;
+         save_glx_visual( dpy, visual, rgba_flag, double_flag, depth_flag );
       }
       else {
 	 /* this visual can't be used for GL rendering */
@@ -870,14 +948,14 @@ GLXDrawable glXGetCurrentDrawable( void )
 
 void glXWaitGL( void )
 {
-   dd_flush();
+   (*DD.flush)();
 }
 
 
 
 void glXWaitX( void )
 {
-   dd_flush();
+   (*DD.flush)();
 }
 
 
@@ -900,7 +978,7 @@ const char *glXQueryServerString( Display *dpy, int screen, int name )
 {
    static char *extensions = "";
    static char *vendor = "Brian Paul";
-   static char *version = "1.2.2 Mesa";
+   static char *version = "1.2.5 Mesa";
 
    switch (name) {
       case GLX_EXTENSIONS:
@@ -920,7 +998,7 @@ const char *glXGetClientString( Display *dpy, int name )
 {
    static char *extensions = "";
    static char *vendor = "Brian Paul";
-   static char *version = "1.2.2 Mesa";
+   static char *version = "1.2.5 Mesa";
 
    switch (name) {
       case GLX_EXTENSIONS:
