@@ -22,9 +22,25 @@
 
 
 /*
-$Id: lines.c,v 1.22 1995/07/25 18:36:32 brianp Exp $
+$Id: lines.c,v 1.27 1995/10/23 21:28:30 brianp Exp $
 
 $Log: lines.c,v $
+ * Revision 1.27  1995/10/23  21:28:30  brianp
+ * use new gl_interpolate_4ub function
+ *
+ * Revision 1.26  1995/10/17  21:40:58  brianp
+ * removed simple_ci/rgba_line() functions because of new device driver
+ *
+ * Revision 1.25  1995/09/21  14:07:29  brianp
+ * more new DD prototyping
+ *
+ * Revision 1.24  1995/09/20  18:20:39  brianp
+ * prototype device driver changes described
+ *
+ * Revision 1.23  1995/09/13  14:49:34  brianp
+ * use CC.NewState convention
+ * replaced VB.Vs and VB.Vt with VB.TexCoord
+ *
  * Revision 1.22  1995/07/25  18:36:32  brianp
  * convert window coords from floats to ints by rounding, not truncating
  *
@@ -128,9 +144,7 @@ void glLineWidth( GLfloat width )
       }
 
       CC.Line.Width = width;
-      CC.LineFunc = ( void (*)() ) 0;
-
-      gl_update_rasterflags();
+      CC.NewState = GL_TRUE;
    }
 }
 
@@ -233,37 +247,6 @@ static void select_line( GLuint v1, GLuint v2, GLuint pv )
 
 
 
-/*
- * Simple color index line.
- */
-static void simple_ci_line( GLuint v1, GLuint v2, GLuint pv )
-{
-   GLint x1 = (GLint) (VB.Win[v1][0] + 0.5F);
-   GLint y1 = (GLint) (VB.Win[v1][1] + 0.5F);
-   GLint x2 = (GLint) (VB.Win[v2][0] + 0.5F);
-   GLint y2 = (GLint) (VB.Win[v2][1] + 0.5F);
-
-   dd_index( VB.Index[pv] );
-   (*DD.draw_line)( x1, y1, x2, y2 );
-}
-
-
-/*
- * Simple RGBA line.
- */
-static void simple_rgba_line( GLuint v1, GLuint v2, GLuint pv )
-{
-   GLint x1 = (GLint) (VB.Win[v1][0] + 0.5F);
-   GLint y1 = (GLint) (VB.Win[v1][1] + 0.5F);
-   GLint x2 = (GLint) (VB.Win[v2][0] + 0.5F);
-   GLint y2 = (GLint) (VB.Win[v2][1] + 0.5F);
-
-   dd_color( VB.Color[pv] );
-   (*DD.draw_line)( x1, y1, x2, y2 );
-}
-
-
-
 #if MAX_WIDTH > MAX_HEIGHT
 #  define MAXPOINTS MAX_WIDTH
 #else
@@ -381,19 +364,16 @@ static void smooth_rgba_line( GLuint v1, GLuint v2, GLuint pv )
    }
 
    /* interpolate color */
-   gl_interpolate_ub( n,
+   gl_interpolate_4ub( n,
 		      (GLint) (VB.Color[v1][0] * CC.RedScale),
 		      (GLint) (VB.Color[v2][0] * CC.RedScale ),
-		      PB.r+PB.count );
-   gl_interpolate_ub( n,
+		      PB.r+PB.count,
 		      (GLint) (VB.Color[v1][1] * CC.GreenScale),
 		      (GLint) (VB.Color[v2][1] * CC.GreenScale ),
-		      PB.g+PB.count );
-   gl_interpolate_ub( n,
+		      PB.g+PB.count,
 		      (GLint) (VB.Color[v1][2] * CC.BlueScale),
 		      (GLint) (VB.Color[v2][2] * CC.BlueScale ),
-		      PB.b+PB.count );
-   gl_interpolate_ub( n,
+		      PB.b+PB.count,
 		      (GLint) (VB.Color[v1][3] * CC.AlphaScale),
 		      (GLint) (VB.Color[v2][3] * CC.AlphaScale ),
 		      PB.a+PB.count );
@@ -575,18 +555,19 @@ static void general_rgba_line( GLuint v1, GLuint v2, GLuint pv )
    }
    else {
       /* interpolate color */
-      GLint r0 = (GLint) (VB.Color[v1][0] * CC.RedScale);
-      GLint r1 = (GLint) (VB.Color[v2][0] * CC.RedScale);
-      GLint g0 = (GLint) (VB.Color[v1][1] * CC.GreenScale);
-      GLint g1 = (GLint) (VB.Color[v2][1] * CC.GreenScale);
-      GLint b0 = (GLint) (VB.Color[v1][2] * CC.BlueScale);
-      GLint b1 = (GLint) (VB.Color[v2][2] * CC.BlueScale);
-      GLint a0 = (GLint) (VB.Color[v1][3] * CC.AlphaScale);
-      GLint a1 = (GLint) (VB.Color[v2][3] * CC.AlphaScale);
-      GL_INTERPOLATE_UB( n, r0, r1, red );
-      GL_INTERPOLATE_UB( n, g0, g1, green );
-      GL_INTERPOLATE_UB( n, b0, b1, blue );
-      GL_INTERPOLATE_UB( n, a0, a1, alpha );
+      gl_interpolate_4ub( n,
+                          (GLint) (VB.Color[v1][0] * CC.RedScale),
+                          (GLint) (VB.Color[v2][0] * CC.RedScale),
+                          red,
+                          (GLint) (VB.Color[v1][1] * CC.GreenScale),
+                          (GLint) (VB.Color[v2][1] * CC.GreenScale),
+                          green,
+                          (GLint) (VB.Color[v1][2] * CC.BlueScale),
+                          (GLint) (VB.Color[v2][2] * CC.BlueScale),
+                          blue,
+                          (GLint) (VB.Color[v1][3] * CC.AlphaScale),
+                          (GLint) (VB.Color[v2][3] * CC.AlphaScale),
+                          alpha );
    }
 
    /* compute delta x and delta y */
@@ -722,24 +703,26 @@ static void textured_rgba_line( GLuint v1, GLuint v2, GLuint pv )
    }
    else {
       /* interpolate color */
-      GLint r0 = (GLint) (VB.Color[v1][0] * CC.RedScale);
-      GLint r1 = (GLint) (VB.Color[v2][0] * CC.RedScale);
-      GLint g0 = (GLint) (VB.Color[v1][1] * CC.GreenScale);
-      GLint g1 = (GLint) (VB.Color[v2][1] * CC.GreenScale);
-      GLint b0 = (GLint) (VB.Color[v1][2] * CC.BlueScale);
-      GLint b1 = (GLint) (VB.Color[v2][2] * CC.BlueScale);
-      GLint a0 = (GLint) (VB.Color[v1][3] * CC.AlphaScale);
-      GLint a1 = (GLint) (VB.Color[v2][3] * CC.AlphaScale);
-      GL_INTERPOLATE_UB( n, r0, r1, red );
-      GL_INTERPOLATE_UB( n, g0, g1, green );
-      GL_INTERPOLATE_UB( n, b0, b1, blue );
-      GL_INTERPOLATE_UB( n, a0, a1, alpha );
+      gl_interpolate_4ub( n,
+                          (GLint) (VB.Color[v1][0] * CC.RedScale),
+                          (GLint) (VB.Color[v2][0] * CC.RedScale),
+                          red,
+                          (GLint) (VB.Color[v1][1] * CC.GreenScale),
+                          (GLint) (VB.Color[v2][1] * CC.GreenScale),
+                          green,
+                          (GLint) (VB.Color[v1][2] * CC.BlueScale),
+                          (GLint) (VB.Color[v2][2] * CC.BlueScale),
+                          blue,
+                          (GLint) (VB.Color[v1][3] * CC.AlphaScale),
+                          (GLint) (VB.Color[v2][3] * CC.AlphaScale),
+                          alpha );
    }
 
    /* interpolate texture coordinates */
    gl_interp_texcoords( n, VB.Eye[v1][2], VB.Eye[v2][2],
 		        VB.Win[v1][2] * MAX_DEPTH, VB.Win[v2][2] * MAX_DEPTH,
-		        VB.Vs[v1], VB.Vs[v2], VB.Vt[v1], VB.Vt[v2],
+		        VB.TexCoord[v1][0], VB.TexCoord[v2][0],
+                        VB.TexCoord[v1][1], VB.TexCoord[v2][1],
 		        s, t, NULL );
 
 
@@ -835,7 +818,11 @@ void gl_set_line_function( void )
    /* TODO: antialiased lines */
 
    if (CC.RenderMode==GL_RENDER) {
-      if (CC.Texture.Enabled) {
+      CC.LineFunc = (*DD.get_line_func)();
+      if (CC.LineFunc) {
+         /* Device driver will draw lines. */
+      }
+      else if (CC.Texture.Enabled) {
 	 CC.LineFunc = textured_rgba_line;
       }
       else if (CC.Line.Width!=1.0 || CC.Line.StippleFlag || CC.Line.SmoothFlag
@@ -847,13 +834,9 @@ void gl_set_line_function( void )
 	    /* Width==1, non-stippled, smooth-shaded, any raster ops */
 	    CC.LineFunc = CC.RGBAflag ? smooth_rgba_line : smooth_ci_line;
 	 }
-	 else if (CC.RasterMask & RASTER_BITS) {
+         else {
 	    /* Width==1, non-stippled, flat-shaded, any raster ops */
 	    CC.LineFunc = CC.RGBAflag ? flat_rgba_line : flat_ci_line;
-	 }
-	 else {
-	    /* Simplest case */
-	    CC.LineFunc = CC.RGBAflag ? simple_rgba_line : simple_ci_line;
          }
       }
    }
