@@ -22,9 +22,27 @@
 
 
 /*
-$Id: list.c,v 1.23 1995/08/01 21:51:40 brianp Exp $
+$Id: list.c,v 1.29 1995/11/03 17:38:16 brianp Exp $
 
 $Log: list.c,v $
+ * Revision 1.29  1995/11/03  17:38:16  brianp
+ * removed unused variables, added casts for C++ compilation
+ *
+ * Revision 1.28  1995/11/02  14:56:31  brianp
+ * added several (Node *) casts per Steven Spitz
+ *
+ * Revision 1.27  1995/10/29  19:14:19  brianp
+ * added glPolygonOffsetEXT display list support
+ *
+ * Revision 1.26  1995/10/14  17:42:12  brianp
+ * added glClipPlane, glLightModel, and texture functions
+ *
+ * Revision 1.25  1995/10/04  19:35:07  brianp
+ * replaced gl_save_normal with gl_save_normal3fv and gl_save_normal3f
+ *
+ * Revision 1.24  1995/08/31  18:34:16  brianp
+ * added display list / glScissor support
+ *
  * Revision 1.23  1995/08/01  21:51:40  brianp
  * renamed gl_save_pixelZoom as gl_save_pixelzoom
  *
@@ -108,6 +126,7 @@ $Log: list.c,v $
 #include "accum.h"
 #include "attrib.h"
 #include "bitmap.h"
+#include "clip.h"
 #include "context.h"
 #include "copypix.h"
 #include "draw.h"
@@ -141,6 +160,7 @@ glClearColor	gl_save_clearcolor	<same>		NODE_CLEAR_COLOR
 glClearDepth	gl_save_cleardepth	<same>		NODE_CLEAR_DEPTH
 glClearIndex	gl_save_clearindex	<same>		NODE_CLEAR_INDEX
 glClearStencil	gl_save_clearstencil	<same>		NODE_CLEAR_STENCIL
+glClipPlane     gl_save_clipplane       gl_clipplane    NODE_CLIP_PLANE
 glColor*	gl_save_color		gl_color	NODE_COLOR
 glColorMask	gl_save_colormask	<same>		NODE_COLOR_MASK
 glCopyPixels	gl_save_copypixels	gl_copypixels	NODE_COPY_PIXELS
@@ -169,6 +189,7 @@ glIndex*	gl_save_index		gl_index	NODE_INDEX
 glIndexMask	gl_save_indexmask	<same>		NODE_INDEX_MASK
 glInitNames	gl_save_initnames	<same>		NODE_INIT_NAMES
 glLight*	gl_save_light		gl_light	NODE_LIGHT
+glLightModel*	gl_save_lightmodel	gl_lightmodel	NODE_LIGHT_MODEL
 glLineStipple	gl_save_linestipple	gl_line_stipple	NODE_LINE_STIPPLE
 glLineWidth	gl_save_linewidth	gl_line_width	NODE_LINE_WIDTH
 glListBase	gl_save_listbase	<same>		NODE_LIST_BASE
@@ -189,8 +210,9 @@ glNormal*	gl_save_normal		<none>		NODE_NORMAL
 glPassThrough	gl_save_passthrough	gl_passthrough	NODE_PASSTHROUGH
 glPixelMap	gl_save_pixelmap	gl_pixel_map	NODE_PIXEL_MAP
 glPixelZoom	gl_save_pixelzoom	<same>		NODE_PIXEL_ZOOM
-glPointSize	gl_save_pointsize	gl_pointsize	NODE_POINTSIZE
-glPolygonMode	gl_save_polygonmode	gl_polygon_mode	NODE_POLYGON_MODE
+glPointSize	gl_save_pointsize	<same>   	NODE_POINTSIZE
+glPolygonMode	gl_save_polygonmode	<same>    	NODE_POLYGON_MODE
+glPolygonOffset gl_save_polygonoffset   <same>          NODE_POLYGON_OFFSET
 glPopAttrib	gl_save_popmatrix	gl_pop_attrib	NODE_POP_ATTRIB
 glPopMatrix	gl_save_popmatrix	gl_pop_matrix	NODE_POP_MATRIX
 glPopName	gl_save_popname		<same>		NODE_POP_NAME
@@ -199,11 +221,15 @@ glPushMatrix	gl_save_pushmatrix	gl_push_matrix	NODE_PUSH_MATRIX
 glPushName	gl_save_pushname	<same>		NODE_PUSH_NAME
 glRasterPos*	gl_save_rasterpos	gl_rasterpos	NODE_RASTER_POS
 glReadBuffer	gl_save_readbuffer	<same>		NODE_READ_BUFFER
+glScissor       gl_save_scissor         <same>          NODE_SCISSOR
 glShadeModel	gl_save_shademodel	<same>		NODE_SHADE_MODEL
 glStencilFunc	gl_save_stencilfunc	<same>		NODE_STENCIL_FUNC
 glStencilMask	gl_save_stencilmask	<same>		NODE_STENCIL_MASK
 glStencilOp	gl_save_stencilop	<same>		NODE_STENCIL_OP
 glTexCoord*	gl_save_texcoord	<none>		NODE_TEXCOORD
+glTexEnv*       gl_save_texenv          gl_texinv       NODE_TEXENV
+glTexGen*       gl_save_texgen          gl_texgen       NODE_TEXGEN
+glTexParam*     gl_save_texparameter    gl_texparameter NODE_TEXPARAMETER
 glTexImage1D	gl_save_teximage1d	gl_teximage1d	NODE_TEXIMAGE1D
 glTexImage2D	gl_save_teximage2d	gl_teximage2d	NODE_TEXIMAGE2D
 glVertex*	gl_save_vertex		gl_vertex	NODE_VERTEX
@@ -255,6 +281,7 @@ typedef enum {
 	NODE_CLEAR_DEPTH,
 	NODE_CLEAR_INDEX,
 	NODE_CLEAR_STENCIL,
+        NODE_CLIP_PLANE,
 	NODE_COLOR,
 	NODE_COLOR_MASK,
 	NODE_COPY_PIXELS,
@@ -280,6 +307,7 @@ typedef enum {
 	NODE_INDEX_MASK,
 	NODE_INIT_NAMES,
 	NODE_LIGHT,
+	NODE_LIGHT_MODEL,
 	NODE_LINE_STIPPLE,
 	NODE_LINE_WIDTH,
 	NODE_LIST_BASE,
@@ -298,6 +326,7 @@ typedef enum {
 	NODE_PIXEL_ZOOM,
 	NODE_POINTSIZE,
 	NODE_POLYGON_MODE,
+	NODE_POLYGON_OFFSET,
 	NODE_POP_ATTRIB,
 	NODE_POP_MATRIX,
 	NODE_POP_NAME,
@@ -306,6 +335,7 @@ typedef enum {
 	NODE_PUSH_NAME,
 	NODE_RASTER_POS,
 	NODE_READ_BUFFER,
+	NODE_SCISSOR,
 	NODE_SET_BOOLEAN,
 	NODE_SET_ENUM,
 	NODE_SET_FLOAT,
@@ -316,6 +346,9 @@ typedef enum {
 	NODE_STENCIL_MASK,
 	NODE_STENCIL_OP,
 	NODE_TEXCOORD,
+        NODE_TEXENV,
+        NODE_TEXGEN,
+        NODE_TEXPARAMETER,
 	NODE_TEXIMAGE1D,
 	NODE_TEXIMAGE2D,
         NODE_VERTEX,
@@ -377,7 +410,7 @@ static Node *alloc_nodes( GLuint count )
       n[0].kind = NODE_CONTINUE;
       n[1].next = (Node *) malloc( sizeof(Node) * BLOCK_SIZE );
       /* TODO: check for out of memory */
-      CurrentBlock = n[1].next;
+      CurrentBlock = (Node *) n[1].next;
       CurrentPos = 0;
    }
 
@@ -449,7 +482,7 @@ void destroy_list( GLuint list )
 	    }
 	    break;
 	 case NODE_CONTINUE:
-	    n = n[1].next;
+	    n = (Node *) n[1].next;
 	    free( block );
 	    block = n;
 	    break;
@@ -533,7 +566,6 @@ static GLuint translate_id( GLsizei n, GLenum type, const GLvoid *list )
 void gl_init_lists( void )
 {
    static int init_flag = 0;
-   int i;
 
    if (init_flag==0) {
       CurrentListPtr = CurrentBlock = NULL;
@@ -551,6 +583,7 @@ void gl_init_lists( void )
       NodeSize[NODE_CLEAR_COLOR] = 5;
       NodeSize[NODE_CLEAR_DEPTH] = 2;
       NodeSize[NODE_CLEAR_INDEX] = 2;
+      NodeSize[NODE_CLIP_PLANE] = 6;
       NodeSize[NODE_COLOR] = 5;
       NodeSize[NODE_COLOR_MASK] = 5;
       NodeSize[NODE_COPY_PIXELS] = 6;
@@ -576,6 +609,7 @@ void gl_init_lists( void )
       NodeSize[NODE_INDEX_MASK] = 2;
       NodeSize[NODE_INIT_NAMES] = 1;
       NodeSize[NODE_LIGHT] = 7;
+      NodeSize[NODE_LIGHT_MODEL] = 6;
       NodeSize[NODE_LINE_STIPPLE] = 3;
       NodeSize[NODE_LINE_WIDTH] = 2;
       NodeSize[NODE_LIST_BASE] = 2;
@@ -594,6 +628,7 @@ void gl_init_lists( void )
       NodeSize[NODE_PIXEL_ZOOM] = 3;
       NodeSize[NODE_POINTSIZE] = 2;
       NodeSize[NODE_POLYGON_MODE] = 3;
+      NodeSize[NODE_POLYGON_OFFSET] = 3;
       NodeSize[NODE_POP_ATTRIB] = 1;
       NodeSize[NODE_POP_MATRIX] = 1;
       NodeSize[NODE_POP_NAME] = 1;
@@ -602,6 +637,7 @@ void gl_init_lists( void )
       NodeSize[NODE_PUSH_NAME] = 2;
       NodeSize[NODE_RASTER_POS] = 5;
       NodeSize[NODE_READ_BUFFER] = 2;
+      NodeSize[NODE_SCISSOR] = 5;
       NodeSize[NODE_SET_BOOLEAN] = 3;
       NodeSize[NODE_SET_ENUM] = 3;
       NodeSize[NODE_SET_FLOAT] = 3;
@@ -612,6 +648,9 @@ void gl_init_lists( void )
       NodeSize[NODE_STENCIL_OP] = 4;
       NodeSize[NODE_SHADE_MODEL] = 2;
       NodeSize[NODE_TEXCOORD] = 5;
+      NodeSize[NODE_TEXENV] = 7;
+      NodeSize[NODE_TEXGEN] = 7;
+      NodeSize[NODE_TEXPARAMETER] = 7;
       NodeSize[NODE_TEXIMAGE1D] = 6;
       NodeSize[NODE_TEXIMAGE2D] = 7;
       NodeSize[NODE_VERTEX] = 5;
@@ -680,7 +719,6 @@ void gl_save_bitmap( GLsizei width, GLsizei height,
 {
    Node *n = alloc_nodes(8);
    if (n) {
-      GLuint bytes = (width+7)/8 * height;
       n[0].kind = NODE_BITMAP;
       n[1].i = (GLint) width;
       n[2].i = (GLint) height;
@@ -772,6 +810,21 @@ void gl_save_clearstencil( GLint s )
 }
 
 
+void gl_save_clipplane( GLenum plane, const GLfloat *equ )
+{
+   Node *n = alloc_nodes(6);
+   if (n) {
+      n[0].kind = NODE_CLIP_PLANE;
+      n[1].e = plane;
+      n[2].f = equ[0];
+      n[3].f = equ[1];
+      n[4].f = equ[2];
+      n[5].f = equ[3];
+   }
+}
+
+
+
 /*
  * Add a color to current display list.
  */
@@ -793,7 +846,7 @@ void gl_save_colormask( GLboolean red, GLboolean green,
 {
    Node *n = alloc_nodes(5);
    if (n) {
-      n[0].kind = NODE_COLOR;
+      n[0].kind = NODE_COLOR_MASK;
       n[1].b = red;
       n[2].b = green;
       n[3].b = blue;
@@ -993,7 +1046,6 @@ void gl_save_fog( GLenum pname, const GLfloat *params )
 {
    Node *n = alloc_nodes(6);
    if (n) {
-      GLuint i;
       n[0].kind = NODE_FOG;
       n[1].e = pname;
       n[2].f = params[0];
@@ -1077,6 +1129,21 @@ void gl_save_light( GLenum light, GLenum pname, const GLfloat *params,
 }
 
 
+void gl_save_lightmodel( GLenum pname, const GLfloat *params )
+{
+   Node *n = alloc_nodes(6);
+   if (n) {
+      n[0].kind = NODE_LIGHT_MODEL;
+      n[1].e = pname;
+      n[2].f = params[0];
+      n[3].f = params[1];
+      n[4].f = params[2];
+      n[5].f = params[3];
+   }
+}
+
+
+
 /*
  * Save a glLineStipple call.
  */
@@ -1140,7 +1207,6 @@ void gl_save_loadname( GLuint name )
 {
    Node *n = alloc_nodes(2);
    if (n) {
-      GLuint i;
       n[0].kind = NODE_LOAD_NAME;
       n[1].ui = name;
    }
@@ -1269,7 +1335,7 @@ void gl_save_multmatrix( const GLfloat *m )
 /*
  * Add a normal to current display list.
  */
-void gl_save_normal( const GLfloat norm[3] )
+void gl_save_normal3fv( const GLfloat norm[3] )
 {
    Node *n = alloc_nodes(4);
    if (n) {
@@ -1282,6 +1348,22 @@ void gl_save_normal( const GLfloat norm[3] )
 
 
 
+/*
+ * Add a normal to current display list.
+ */
+void gl_save_normal3f( GLfloat nx, GLfloat ny, GLfloat nz )
+{
+   Node *n = alloc_nodes(4);
+   if (n) {
+      n[0].kind = NODE_NORMAL;
+      n[1].f = nx;
+      n[2].f = ny;
+      n[3].f = nz;
+   }
+}
+
+
+
 /*
  * Save a glPixelMap call.
  */
@@ -1343,6 +1425,17 @@ void gl_save_polygonmode( GLenum face, GLenum mode )
 
 
 
+void gl_save_polygonoffset( GLfloat factor, GLfloat bias )
+{
+   Node *n = alloc_nodes(3);
+   if (n) {
+      n[0].kind = NODE_POLYGON_OFFSET;
+      n[1].f = factor;
+      n[2].f = bias;
+   }
+}
+
+
 /*
  * Add a pop attributies operation to current display list.
  */
@@ -1374,7 +1467,6 @@ void gl_save_popname( void )
 {
    Node *n = alloc_nodes(1);
    if (n) {
-      GLuint i;
       n[0].kind = NODE_POP_NAME;
    }
 }
@@ -1414,7 +1506,6 @@ void gl_save_pushname( GLuint name )
 {
    Node *n = alloc_nodes(2);
    if (n) {
-      GLuint i;
       n[0].kind = NODE_PUSH_NAME;
       n[1].ui = name;
    }
@@ -1456,6 +1547,20 @@ void gl_save_readbuffer( GLenum mode )
 }
 
 
+void gl_save_scissor( GLint x, GLint y, GLsizei width, GLsizei height )
+{
+   Node *n = alloc_nodes(5);
+   if (n) {
+      n[0].kind = NODE_SCISSOR;
+      n[1].i = x;
+      n[2].i = y;
+      n[3].i = width;
+      n[4].i = height;
+   }
+}
+
+
+
 /*
  * Save a "set boolean value" operation.
  */
@@ -1600,6 +1705,52 @@ void gl_save_texcoord( const GLfloat tc[4] )
 
 
 
+void gl_save_texenv( GLenum target, GLenum pname, const GLfloat *params )
+{
+   Node *n = alloc_nodes(7);
+   if (n) {
+      n[0].kind = NODE_TEXENV;
+      n[1].e = target;
+      n[2].e = pname;
+      n[3].f = params[0];
+      n[4].f = params[1];
+      n[5].f = params[2];
+      n[6].f = params[3];
+   }
+}
+
+
+void gl_save_texgen( GLenum coord, GLenum pname, const GLfloat *params )
+{
+   Node *n = alloc_nodes(7);
+   if (n) {
+      n[0].kind = NODE_TEXGEN;
+      n[1].e = coord;
+      n[2].e = pname;
+      n[3].f = params[0];
+      n[4].f = params[1];
+      n[5].f = params[2];
+      n[6].f = params[3];
+   }
+}
+
+
+void gl_save_texparameter( GLenum target, GLenum pname, const GLfloat *params )
+{
+   Node *n = alloc_nodes(7);
+   if (n) {
+      n[0].kind = NODE_TEXPARAMETER;
+      n[1].e = target;
+      n[2].e = pname;
+      n[3].f = params[0];
+      n[4].f = params[1];
+      n[5].f = params[2];
+      n[6].f = params[3];
+   }
+}
+
+
+
 /*
  * Add a glTexImage1D call to current display list.
  */
@@ -1818,6 +1969,16 @@ static void execute_list( GLuint list )
 	 case NODE_CLEAR_STENCIL:
 	    glClearStencil( n[1].i );
 	    break;
+         case NODE_CLIP_PLANE:
+            {
+               GLfloat equ[4];
+               equ[0] = n[2].f;
+               equ[1] = n[3].f;
+               equ[2] = n[4].f;
+               equ[3] = n[5].f;
+               gl_clipplane( n[1].e, equ );
+            }
+            break;
 	 case NODE_COLOR_MASK:
 	    glColorMask( n[1].b, n[2].b, n[3].b, n[4].b );
 	    break;
@@ -1900,6 +2061,16 @@ static void execute_list( GLuint list )
 	       gl_light( n[1].e, n[2].e, p );
 	    }
 	    break;
+         case NODE_LIGHT_MODEL:
+	    {
+	       GLfloat p[4];
+	       p[0] = n[2].f;
+	       p[1] = n[3].f;
+	       p[2] = n[4].f;
+	       p[3] = n[5].f;
+	       gl_lightmodel( n[1].e, p );
+	    }
+	    break;
 	 case NODE_LINE_STIPPLE:
 	    glLineStipple( n[1].i, n[2].us );
 	    break;
@@ -1984,6 +2155,9 @@ static void execute_list( GLuint list )
 	 case NODE_POLYGON_MODE:
 	    glPolygonMode( n[1].e, n[2].e );
 	    break;
+	 case NODE_POLYGON_OFFSET:
+	    glPolygonOffsetEXT( n[1].f, n[2].f );
+	    break;
 	 case NODE_POP_ATTRIB:
 	    gl_pop_attrib();
 	    break;
@@ -2015,6 +2189,9 @@ static void execute_list( GLuint list )
 	 case NODE_READ_BUFFER:
 	    glReadBuffer( n[1].e );
 	    break;
+	 case NODE_SCISSOR:
+	    glScissor( n[1].i, n[2].i, n[3].i, n[4].i );
+	    break;
 	 case NODE_SET_BOOLEAN:
 	    if (INSIDE_BEGIN_END) {
 	       gl_error( GL_INVALID_OPERATION, "in display list" );
@@ -2077,17 +2254,49 @@ static void execute_list( GLuint list )
 	 case NODE_STENCIL_OP:
 	    glStencilOp( n[1].e, n[2].e, n[3].e );
 	    break;
+         case NODE_TEXENV:
+            {
+               GLfloat params[4];
+               params[0] = n[3].f;
+               params[1] = n[4].f;
+               params[2] = n[5].f;
+               params[3] = n[6].f;
+               gl_texenv( n[1].e, n[2].e, params );
+            }
+            break;
+         case NODE_TEXGEN:
+            {
+               GLfloat params[4];
+               params[0] = n[3].f;
+               params[1] = n[4].f;
+               params[2] = n[5].f;
+               params[3] = n[6].f;
+               gl_texgen( n[1].e, n[2].e, params );
+            }
+            break;
+         case NODE_TEXPARAMETER:
+            {
+               GLfloat params[4];
+               params[0] = n[3].f;
+               params[1] = n[4].f;
+               params[2] = n[5].f;
+               params[3] = n[6].f;
+               gl_texparameter( n[1].e, n[2].e, params );
+            }
+            break;
 	 case NODE_TEXIMAGE1D:
-	    gl_teximage1d( n[1].i, n[2].i, n[3].i, n[4].i, n[5].data );
+	    gl_teximage1d( n[1].i, n[2].i, n[3].i, n[4].i,
+                           (GLubyte *) n[5].data );
 	    break;
 	 case NODE_TEXIMAGE2D:
-	    gl_teximage2d( n[1].i, n[2].i, n[3].i, n[4].i, n[5].i, n[6].data );
+	    gl_teximage2d( n[1].i, n[2].i, n[3].i, n[4].i, n[5].i,
+                           (GLubyte *) n[6].data );
 	    break;
 	 case NODE_VIEWPORT:
 	    gl_viewport( n[1].i, n[2].i, (GLsizei) n[3].i, (GLsizei) n[4].i );
 	    break;
 	 case NODE_CONTINUE:
-	    n = n[1].next;
+	    n = (Node *) n[1].next;
 	    break;
 	 case NODE_END_OF_LIST:
 	    done = GL_TRUE;
