@@ -22,9 +22,24 @@
 
 
 /*
-$Id: pixel.c,v 1.7 1995/08/01 20:53:50 brianp Exp $
+$Id: pixel.c,v 1.12 1995/11/30 00:19:31 brianp Exp $
 
 $Log: pixel.c,v $
+ * Revision 1.12  1995/11/30  00:19:31  brianp
+ * include stdio.h
+ *
+ * Revision 1.11  1995/11/17  14:30:41  brianp
+ * in gl_write_zoomed_*() limit spans to MAX_WIDTH, removed mallocs/frees
+ *
+ * Revision 1.10  1995/10/16  15:26:34  brianp
+ * added gl_write_zoomed_stencil_span
+ *
+ * Revision 1.9  1995/10/14  16:28:56  brianp
+ * added glPixelZoom support
+ *
+ * Revision 1.8  1995/09/15  18:39:07  brianp
+ * added update_drawpixels_state()
+ *
  * Revision 1.7  1995/08/01  20:53:50  brianp
  * added gl_save_pixelzoom()
  *
@@ -55,12 +70,16 @@ $Log: pixel.c,v $
 
 
 
+#include <assert.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "context.h"
 #include "list.h"
 #include "macros.h"
+#include "pixel.h"
 #include "span.h"
+#include "stencil.h"
 
 
 
@@ -119,11 +138,296 @@ static void swap2( GLushort *p, GLuint n )
 
 
 
+/*
+ * Determine if we can use the optimized glDrawPixels function.
+ */
+static void update_drawpixels_state( void )
+{
+   if (CC.RGBAflag==GL_TRUE &&
+       CC.Pixel.RedBias==0.0   && CC.Pixel.RedScale==1.0 &&
+       CC.Pixel.GreenBias==0.0 && CC.Pixel.GreenScale==1.0 &&
+       CC.Pixel.BlueBias==0.0  && CC.Pixel.BlueScale==1.0 &&
+       CC.Pixel.AlphaBias==0.0 && CC.Pixel.AlphaScale==1.0 &&
+       CC.Pixel.MapColorFlag==GL_FALSE &&
+       CC.Pixel.ZoomX==1.0 && CC.Pixel.ZoomY==1.0 &&
+       CC.UnpackAlignment==4 &&
+       CC.UnpackRowLength==0 &&
+       CC.UnpackSkipPixels==0 &&
+       CC.UnpackSkipRows==0 &&
+       CC.UnpackSwapBytes==0 &&
+       CC.UnpackLSBFirst==0) {
+      CC.FastDrawPixels = GL_TRUE;
+   }
+   else {
+      CC.FastDrawPixels = GL_FALSE;
+   }
+}
+
+
+
+
 /**********************************************************************/
 /*****                    glPixelZoom                             *****/
 /**********************************************************************/
 
 
+/*
+ * Write a span of pixels to the frame buffer while applying a pixel zoom.
+ * This is only used by glDrawPixels and glCopyPixels.
+ * Input:  n - number of pixels in input row
+ *         x, y - destination of the span
+ *         z - depth values for the span
+ *         red, green, blue, alpha - array of colors
+ *         y0 - location of first row in the image we're drawing.
+ */
+void
+gl_write_zoomed_color_span( GLuint n, GLint x, GLint y, const GLint z[],
+                            const GLubyte red[], const GLubyte green[],
+                            const GLubyte blue[], const GLubyte alpha[],
+                            GLint y0 )
+{
+   GLint m;
+   GLint r0, r1, row, r;
+   GLint i, j, skipcol;
+   GLubyte zred[MAX_WIDTH], zgreen[MAX_WIDTH];  /* zoomed pixel colors */
+   GLubyte zblue[MAX_WIDTH], zalpha[MAX_WIDTH];
+   GLint zdepth[MAX_WIDTH];  /* zoomed depth values */
+
+   /* compute width of output row */
+   m = (GLint) ABS( n * CC.Pixel.ZoomX );
+   if (m==0) {
+      return;
+   }
+   if (CC.Pixel.ZoomX<0.0) {
+      /* adjust x coordinate for left/right mirroring */
+      x = x - m;
+   }
+
+   /* compute which rows to draw */
+   row = y-y0;
+   r0 = y0 + (GLint) (row * CC.Pixel.ZoomY);
+   r1 = y0 + (GLint) ((row+1) * CC.Pixel.ZoomY);
+   if (r0==r1) {
+      return;
+   }
+   else if (r1<r0) {
+      GLint rtmp = r1;
+      r1 = r0;
+      r0 = rtmp;
+   }
+
+   /* make sure span isn't too long */
+   skipcol = 0;
+   if (m>MAX_WIDTH) {
+      /* shorten the span */
+      if (x<0) {
+         skipcol = -x;
+         m += x;
+      }
+      if (m>MAX_WIDTH) {
+         m = MAX_WIDTH - skipcol;
+      }
+      if (m<=0) {
+         return;
+      }
+   }
+
+   assert( m <= MAX_WIDTH );
+
+   /* zoom the span horizontally */
+   if (CC.Pixel.ZoomX==-1.0F) {
+      /* n==m */
+      for (j=0;j<m;j++) {
+         i = n - (j+skipcol) - 1;
+         zred[j]   = red[i];
+         zgreen[j] = green[i];
+         zblue[j]  = blue[i];
+         zalpha[j] = alpha[i];
+         zdepth[j] = z[i];
+      }
+   }
+   else {
+      GLfloat xscale = 1.0F / CC.Pixel.ZoomX;
+      for (j=0;j<m;j++) {
+         i = (j+skipcol) * xscale;
+         if (i<0)  i = n + i - 1;
+         zred[j]   = red[i];
+         zgreen[j] = green[i];
+         zblue[j]  = blue[i];
+         zalpha[j] = alpha[i];
+         zdepth[j] = z[i];
+      }
+   }
+
+   /* write the span */
+   for (r=r0; r<r1; r++) {
+      gl_write_color_span( m, x+skipcol, r, zdepth,
+                           zred, zgreen, zblue, zalpha, GL_BITMAP );
+   }
+}
+
+
+
+/*
+ * As above, but write CI pixels.
+ */
+void
+gl_write_zoomed_index_span( GLuint n, GLint x, GLint y, const GLint z[],
+                            const GLuint indexes[], GLint y0 )
+{
+   GLint m;
+   GLint r0, r1, row, r;
+   GLint i, j, skipcol;
+   GLuint zindexes[MAX_WIDTH];  /* zoomed color indexes */
+   GLint zdepth[MAX_WIDTH];  /* zoomed depth values */
+
+   /* compute width of output row */
+   m = (GLint) ABS( n * CC.Pixel.ZoomX );
+   if (m==0) {
+      return;
+   }
+   if (CC.Pixel.ZoomX<0.0) {
+      /* adjust x coordinate for left/right mirroring */
+      x = x - m;
+   }
+
+   /* compute which rows to draw */
+   row = y-y0;
+   r0 = y0 + (GLint) (row * CC.Pixel.ZoomY);
+   r1 = y0 + (GLint) ((row+1) * CC.Pixel.ZoomY);
+   if (r0==r1) {
+      return;
+   }
+   else if (r1<r0) {
+      GLint rtmp = r1;
+      r1 = r0;
+      r0 = rtmp;
+   }
+
+   /* make sure span isn't too long */
+   skipcol = 0;
+   if (m>MAX_WIDTH) {
+      /* shorten the span */
+      if (x<0) {
+         skipcol = -x;
+         m += x;
+      }
+      if (m>MAX_WIDTH) {
+         m = MAX_WIDTH - skipcol;
+      }
+      if (m<=0) {
+         return;
+      }
+   }
+
+   assert( m <= MAX_WIDTH );
+
+   /* zoom the span horizontally */
+   if (CC.Pixel.ZoomX==-1.0F) {
+      /* n==m */
+      for (j=0;j<m;j++) {
+         i = n - (j+skipcol) - 1;
+         zindexes[j] = indexes[i];
+         zdepth[j]   = z[i];
+      }
+   }
+   else {
+      GLfloat xscale = 1.0F / CC.Pixel.ZoomX;
+      for (j=0;j<m;j++) {
+         i = (j+skipcol) * xscale;
+         if (i<0)  i = n + i - 1;
+         zindexes[j] = indexes[i];
+         zdepth[j] = z[i];
+      }
+   }
+
+   /* write the span */
+   for (r=r0; r<r1; r++) {
+      gl_write_index_span( m, x+skipcol, r, zdepth, zindexes, GL_BITMAP );
+   }
+}
+
+
+
+/*
+ * As above, but write stencil values.
+ */
+void
+gl_write_zoomed_stencil_span( GLuint n, GLint x, GLint y,
+                              const GLubyte stencil[], GLint y0 )
+{
+   GLint m;
+   GLint r0, r1, row, r;
+   GLint i, j, skipcol;
+   GLubyte zstencil[MAX_WIDTH];  /* zoomed stencil values */
+
+   /* compute width of output row */
+   m = (GLint) ABS( n * CC.Pixel.ZoomX );
+   if (m==0) {
+      return;
+   }
+   if (CC.Pixel.ZoomX<0.0) {
+      /* adjust x coordinate for left/right mirroring */
+      x = x - m;
+   }
+
+   /* compute which rows to draw */
+   row = y-y0;
+   r0 = y0 + (GLint) (row * CC.Pixel.ZoomY);
+   r1 = y0 + (GLint) ((row+1) * CC.Pixel.ZoomY);
+   if (r0==r1) {
+      return;
+   }
+   else if (r1<r0) {
+      GLint rtmp = r1;
+      r1 = r0;
+      r0 = rtmp;
+   }
+
+   /* make sure span isn't too long */
+   skipcol = 0;
+   if (m>MAX_WIDTH) {
+      /* shorten the span */
+      if (x<0) {
+         skipcol = -x;
+         m += x;
+      }
+      if (m>MAX_WIDTH) {
+         m = MAX_WIDTH - skipcol;
+      }
+      if (m<=0) {
+         return;
+      }
+   }
+
+   assert( m <= MAX_WIDTH );
+
+   /* zoom the span horizontally */
+   if (CC.Pixel.ZoomX==-1.0F) {
+      /* n==m */
+      for (j=0;j<m;j++) {
+         i = n - (j+skipcol) - 1;
+         zstencil[j] = stencil[i];
+      }
+   }
+   else {
+      GLfloat xscale = 1.0F / CC.Pixel.ZoomX;
+      for (j=0;j<m;j++) {
+         i = (j+skipcol) * xscale;
+         if (i<0)  i = n + i - 1;
+         zstencil[j] = stencil[i];
+      }
+   }
+
+   /* write the span */
+   for (r=r0; r<r1; r++) {
+      gl_write_stencil_span( m, x+skipcol, r, zstencil );
+   }
+}
+
+
+
+
 void glPixelZoom( GLfloat xfactor, GLfloat yfactor )
 {
    if (CC.CompileFlag) {
@@ -136,6 +440,7 @@ void glPixelZoom( GLfloat xfactor, GLfloat yfactor )
       }
       CC.Pixel.ZoomX = xfactor;
       CC.Pixel.ZoomY = yfactor;
+      update_drawpixels_state();
    }
 }
 
@@ -235,6 +540,7 @@ void glPixelStorei( GLenum pname, GLint param )
       default:
 	 gl_error( GL_INVALID_ENUM, "glPixelStore" );
    }
+   update_drawpixels_state();
 }
 
 
@@ -251,7 +557,6 @@ void glPixelStoref( GLenum pname, GLfloat param )
 /**********************************************************************/
 
 
-/* TODO: display list compile */
 
 void gl_pixel_map( GLenum map, GLint mapsize, const GLfloat *values )
 {
@@ -336,7 +641,6 @@ void gl_pixel_map( GLenum map, GLint mapsize, const GLfloat *values )
       default:
          gl_error( GL_INVALID_ENUM, "glPixelMapfv(map)" );
    }
-
 }
 
 
@@ -700,6 +1004,7 @@ void glPixelTransferf( GLenum pname, GLfloat param )
       default:
          gl_error( GL_INVALID_ENUM, "glPixelTransfer(pname)" );
    }
+   update_drawpixels_state();
 }
 
 
