@@ -22,9 +22,18 @@
 
 
 /*
-$Id: readpix.c,v 1.7 1995/07/24 20:34:16 brianp Exp $
+$Id: readpix.c,v 1.10 1995/10/14 16:27:03 brianp Exp $
 
 $Log: readpix.c,v $
+ * Revision 1.10  1995/10/14  16:27:03  brianp
+ * use DD.set_buffer
+ *
+ * Revision 1.9  1995/10/04  22:08:52  brianp
+ * replaced i with j in byte swap function calls
+ *
+ * Revision 1.8  1995/08/31  21:26:39  brianp
+ * use DD.read_*_span instead of dd_read_*_span
+ *
  * Revision 1.7  1995/07/24  20:34:16  brianp
  * replaced memset() with MEMSET() and memcpy() with MEMCPY()
  *
@@ -141,8 +150,6 @@ static void read_index_pixels( GLint x, GLint y,
 	 return;
    }
 
-   dd_read_buffer( CC.Pixel.ReadBuffer );
-
    /* Compute packing parameters */
    a = CC.PackAlignment;
    if (CC.PackRowLength>0) {
@@ -165,7 +172,7 @@ static void read_index_pixels( GLint x, GLint y,
    /* process image row by row */
    for (j=0;j<height;j++,y++) {
       GLuint index[MAX_WIDTH];
-      dd_read_index_span( width, x, y, index );
+      (*DD.read_index_span)( width, x, y, index );
 
       if (CC.Pixel.IndexShift!=0 || CC.Pixel.IndexOffset!=0) {
 	 GLuint s;
@@ -266,8 +273,6 @@ static void read_index_pixels( GLint x, GLint y,
       }
 
    }
-
-   dd_read_buffer( CC.Color.DrawBuffer );
 }
 
 
@@ -712,7 +717,7 @@ static void read_color_pixels( GLint x, GLint y,
 	 GLfloat ascale = 1.0F / CC.AlphaScale;
 
 	 /* read colors and convert to floats */
-	 dd_read_color_span( width, x, y, r, g, b, a );
+	 (*DD.read_color_span)( width, x, y, r, g, b, a );
 	 for (i=0;i<width;i++) {
 	    red[i]   = r[i] * rscale;
 	    green[i] = g[i] * gscale;
@@ -730,7 +735,7 @@ static void read_color_pixels( GLint x, GLint y,
       else {
 	 /* convert CI values to RGBA */
 	 GLuint index[MAX_WIDTH];
-	 dd_read_index_span( width, x, y, index );
+	 (*DD.read_index_span)( width, x, y, index );
 
 	 if (CC.Pixel.IndexShift!=0 || CC.Pixel.IndexOffset!=0) {
 	    GLuint s;
@@ -799,7 +804,7 @@ static void read_color_pixels( GLint x, GLint y,
 	       }
 	    }
 	    if (CC.PackSwapBytes) {
-	       swap2( (GLushort *) pixels + start + i * k, width*n );
+	       swap2( (GLushort *) pixels + start + j * k, width*n );
 	    }
 	    break;
 	 case GL_SHORT:
@@ -813,7 +818,7 @@ static void read_color_pixels( GLint x, GLint y,
 		  if (a_flag)  *dst++ = FLOAT_TO_SHORT( alpha[i] );
 	       }
 	       if (CC.PackSwapBytes) {
-		  swap2( (GLushort *) pixels + start + i * k, width*n );
+		  swap2( (GLushort *) pixels + start + j * k, width*n );
 	       }
 	    }
 	    break;
@@ -828,7 +833,7 @@ static void read_color_pixels( GLint x, GLint y,
 		  if (a_flag)  *dst++ = FLOAT_TO_UINT( alpha[i] );
 	       }
 	       if (CC.PackSwapBytes) {
-		  swap4( (GLuint *) pixels + start + i * k, width*n );
+		  swap4( (GLuint *) pixels + start + j * k, width*n );
 	       }
 	    }
 	    break;
@@ -843,7 +848,7 @@ static void read_color_pixels( GLint x, GLint y,
 		  if (a_flag)  *dst++ = FLOAT_TO_INT( alpha[i] );
 	       }
 	       if (CC.PackSwapBytes) {
-		  swap4( (GLuint *) pixels + start + i * k, width*n );
+		  swap4( (GLuint *) pixels + start + j * k, width*n );
 	       }
 	    }
 	    break;
@@ -858,7 +863,7 @@ static void read_color_pixels( GLint x, GLint y,
 		  if (a_flag)  *dst++ = alpha[i];
 	       }
 	       if (CC.PackSwapBytes) {
-		  swap4( (GLuint *) pixels + start + i * k, width*n );
+		  swap4( (GLuint *) pixels + start + j * k, width*n );
 	       }
 	    }
 	    break;
@@ -872,6 +877,8 @@ static void read_color_pixels( GLint x, GLint y,
 void glReadPixels( GLint x, GLint y, GLsizei width, GLsizei height,
 		   GLenum format, GLenum type, GLvoid *pixels )
 {
+   (void) (*DD.set_buffer)( CC.Pixel.ReadBuffer );
+
    switch (format) {
       case GL_COLOR_INDEX:
          read_index_pixels( x, y, width, height, type, pixels );
@@ -895,7 +902,6 @@ void glReadPixels( GLint x, GLint y, GLsizei width, GLsizei height,
       default:
 	 gl_error( GL_INVALID_ENUM, "glReadPixels(format)" );
    }
+
+   (void) (*DD.set_buffer)( CC.Color.DrawBuffer );
 }
-
-
-
