@@ -22,9 +22,24 @@
 
 
 /*
-$Id: span.c,v 1.18 1995/07/26 15:03:48 brianp Exp $
+$Id: span.c,v 1.23 1995/11/30 00:19:52 brianp Exp $
 
 $Log: span.c,v $
+ * Revision 1.23  1995/11/30  00:19:52  brianp
+ * make copy of span colors if they may be modified and reused
+ *
+ * Revision 1.22  1995/10/19  15:49:30  brianp
+ * added gamma support
+ *
+ * Revision 1.21  1995/10/13  22:41:16  brianp
+ * removed dithering code, added color/index masking code
+ *
+ * Revision 1.20  1995/09/18  14:21:29  brianp
+ * use NULL mask if writing all pixels in a color span
+ *
+ * Revision 1.19  1995/08/31  21:33:29  brianp
+ * use *DD.read_*_span instead of dd_read_*_span
+ *
  * Revision 1.18  1995/07/26  15:03:48  brianp
  * replaced some literals with variables for SunOS 4.x per Asif Khan
  *
@@ -90,21 +105,27 @@ $Log: span.c,v $
  */
 
 
+#include <string.h>
 #include "alpha.h"
 #include "blend.h"
 #include "context.h"
 #include "depth.h"
-#include "dither.h"
 #include "dd.h"
 #include "fog.h"
+#include "gamma.h"
 #include "logic.h"
 #include "macros.h"
+#include "masking.h"
 #include "scissor.h"
 #include "span.h"
 #include "stencil.h"
 #include "texture.h"
 
 
+#ifndef NULL
+#  define NULL 0
+#endif
+
 
 /*
  * Apply the current polygon stipple pattern to a span of pixels.
@@ -227,16 +248,15 @@ void gl_write_index_span( GLuint n, GLint x, GLint y, GLint z[],
       return;
    }
 
-   /* dithering */
-   if (CC.Color.DitherFlag) {
-      gl_dither_index_span( n, x, y, index, mask );
-   }
-
    /* logic op */
    if (CC.Color.SWLogicOpEnabled) {
       gl_logic_span( n, x, y, index, mask );
    }
 
+   if (CC.Color.SWmasking) {
+      gl_mask_index_span( n, x, y, index );
+   }
+
    /* write pixels */
    (*DD.write_index_span)( n, x, y, index, mask );
 }
@@ -291,7 +311,7 @@ void gl_write_monoindex_span( GLuint n, GLint x, GLint y, GLint z[],
    }
 
    if ((CC.Fog.Enabled && (CC.Hint.Fog==GL_NICEST || primitive==GL_BITMAP))
-        || CC.Color.DitherFlag || CC.Color.SWLogicOpEnabled) {
+        || CC.Color.SWLogicOpEnabled || CC.Color.SWmasking) {
       GLuint ispan[MAX_WIDTH];
       for (i=0;i<n;i++) {
 	 ispan[i] = index;
@@ -301,14 +321,14 @@ void gl_write_monoindex_span( GLuint n, GLint x, GLint y, GLint z[],
 	 gl_fog_index_pixels( n, z, ispan );
       }
 
-      if (CC.Color.DitherFlag) {
-	 gl_dither_index_span( n, x, y, ispan, mask );
-      }
-
       if (CC.Color.SWLogicOpEnabled) {
 	 gl_logic_span( n, x, y, ispan, mask );
       }
 
+      if (CC.Color.SWmasking) {
+         gl_mask_index_span( n, x, y, ispan );
+      }
+
       (*DD.write_index_span)( n, x, y, ispan, mask );
    }
    else {
@@ -318,14 +338,16 @@ void gl_write_monoindex_span( GLuint n, GLint x, GLint y, GLint z[],
 
 
 
-
 void gl_write_color_span( GLuint n, GLint x, GLint y, GLint z[],
-			  GLubyte red[], GLubyte green[],
-			  GLubyte blue[], GLubyte alpha[],
+			  GLubyte r[], GLubyte g[],
+			  GLubyte b[], GLubyte a[],
 			  GLenum primitive )
 {
    GLuint i;
    GLubyte mask[MAX_WIDTH];
+   GLboolean write_all = GL_TRUE;
+   GLubyte rtmp[MAX_WIDTH], gtmp[MAX_WIDTH], btmp[MAX_WIDTH], atmp[MAX_WIDTH];
+   GLubyte *red, *green, *blue, *alpha;
 
    /* init mask to 1's (all pixels are to be written) */
    for (i=0;i<n;i++)
@@ -335,6 +357,25 @@ void gl_write_color_span( GLuint n, GLint x, GLint y, GLint z[],
       if (clip_span(n,x,y,mask)==0) {
 	 return;
       }
+      write_all = GL_FALSE;
+   }
+
+   if (CC.MutableColors && primitive==GL_BITMAP) {
+      /* must make a copy of the colors since they can be modified */
+      MEMCPY( rtmp, r, n * sizeof(GLubyte) );
+      MEMCPY( gtmp, g, n * sizeof(GLubyte) );
+      MEMCPY( btmp, b, n * sizeof(GLubyte) );
+      MEMCPY( atmp, a, n * sizeof(GLubyte) );
+      red = rtmp;
+      green = gtmp;
+      blue = btmp;
+      alpha = atmp;
+   }
+   else {
+      red   = r;
+      green = g;
+      blue  = b;
+      alpha = a;
    }
 
    /* Per-pixel fog */
@@ -347,11 +388,13 @@ void gl_write_color_span( GLuint n, GLint x, GLint y, GLint z[],
       if (gl_scissor_span( n, x, y, mask )==0) {
 	 return;
       }
+      write_all = GL_FALSE;
    }
 
    /* Polygon Stippling */
    if (CC.Polygon.StippleFlag && primitive==GL_POLYGON) {
       stipple_polygon_span( n, x, y, mask );
+      write_all = GL_FALSE;
    }
 
    /* Do the alpha test */
@@ -359,6 +402,7 @@ void gl_write_color_span( GLuint n, GLint x, GLint y, GLint z[],
       if (gl_alpha_test( n, alpha, mask )==0) {
 	 return;
       }
+      write_all = GL_FALSE;
    }
 
    if (CC.Stencil.Enabled) {
@@ -368,10 +412,17 @@ void gl_write_color_span( GLuint n, GLint x, GLint y, GLint z[],
       }
       /* depth buffering w/ stencil */
       gl_depth_stencil_span( n, x, y, z, mask );
+      write_all = GL_FALSE;
    }
    else if (CC.Depth.Test) {
       /* regular depth testing */
-      if (gl_depth_test_span( n, x, y, z, mask )==0)  return;
+      GLuint m = gl_depth_test_span( n, x, y, z, mask );
+      if (m==0) {
+         return;
+      }
+      if (m<n) {
+         write_all = GL_FALSE;
+      }
    }
 
    if (CC.Color.ColorMask==0) {
@@ -379,17 +430,23 @@ void gl_write_color_span( GLuint n, GLint x, GLint y, GLint z[],
       return;
    }
 
+   if (CC.RasterMask & GAMMA_BIT) {
+      gl_apply_gamma( n, red, green, blue );
+   }
+
    /* blending */
    if (CC.Color.BlendEnabled) {
       gl_blend_span( n, x, y, red, green, blue, alpha, mask );
    }
 
-   if (CC.Color.DitherFlag) {
-      gl_dither_color_span( n, x, y, red, green, blue, alpha, mask );
+   /* Color component masking */
+   if (CC.Color.SWmasking) {
+      gl_mask_color_span( n, x, y, red, green, blue, alpha );
    }
 
    /* write pixels */
-   (*DD.write_color_span)( n, x, y, red, green, blue, alpha, mask );
+   (*DD.write_color_span)( n, x, y, red, green, blue, alpha,
+                           write_all ? NULL : mask );
 }
 
 
@@ -409,6 +466,7 @@ void gl_write_monocolor_span( GLuint n, GLint x, GLint y, GLint z[],
 {
    GLuint i;
    GLubyte mask[MAX_WIDTH];
+   GLboolean write_all = GL_TRUE;
 
    /* init mask to 1's (all pixels are to be written) */
    for (i=0;i<n;i++)
@@ -418,6 +476,7 @@ void gl_write_monocolor_span( GLuint n, GLint x, GLint y, GLint z[],
       if (clip_span(n,x,y,mask)==0) {
 	 return;
       }
+      write_all = GL_FALSE;
    }
 
    /* Do the scissor test */
@@ -425,11 +484,13 @@ void gl_write_monocolor_span( GLuint n, GLint x, GLint y, GLint z[],
       if (gl_scissor_span( n, x, y, mask )==0) {
 	 return;
       }
+      write_all = GL_FALSE;
    }
 
    /* Polygon Stippling */
    if (CC.Polygon.StippleFlag && primitive==GL_POLYGON) {
       stipple_polygon_span( n, x, y, mask );
+      write_all = GL_FALSE;
    }
 
    /* Do the alpha test */
@@ -441,6 +502,7 @@ void gl_write_monocolor_span( GLuint n, GLint x, GLint y, GLint z[],
       if (gl_alpha_test( n, alpha, mask )==0) {
 	 return;
       }
+      write_all = GL_FALSE;
    }
 
    if (CC.Stencil.Enabled) {
@@ -450,19 +512,26 @@ void gl_write_monocolor_span( GLuint n, GLint x, GLint y, GLint z[],
       }
       /* depth buffering w/ stencil */
       gl_depth_stencil_span( n, x, y, z, mask );
+      write_all = GL_FALSE;
    }
    else if (CC.Depth.Test) {
       /* regular depth testing */
-      if (gl_depth_test_span( n, x, y, z, mask )==0)  return;
+      GLuint m = gl_depth_test_span( n, x, y, z, mask );
+      if (m==0) {
+         return;
+      }
+      if (m<n) {
+         write_all = GL_FALSE;
+      }
    }
 
-
    if (CC.Color.ColorMask==0) {
       /* write no pixels */
       return;
    }
 
-   if (CC.Color.BlendEnabled || CC.Color.DitherFlag) {
+   if (CC.Color.BlendEnabled || CC.Color.SWmasking
+       || (CC.RasterMask & GAMMA_BIT)) {
       /* assign same color to each pixel */
       GLubyte red[MAX_WIDTH], green[MAX_WIDTH];
       GLubyte blue[MAX_WIDTH], alpha[MAX_WIDTH];
@@ -479,15 +548,23 @@ void gl_write_monocolor_span( GLuint n, GLint x, GLint y, GLint z[],
 	    alpha[i] = a;
 	 }
       }
-      if (CC.Color.BlendEnabled) {
-	 gl_blend_span( n, x, y, red, green, blue, alpha, mask );
+
+      if (CC.RasterMask & GAMMA_BIT) {
+         gl_apply_gamma( n, red, green, blue );
       }
-      if (CC.Color.DitherFlag) {
-	 gl_dither_color_span( n, x, y, red, green, blue, alpha, mask );
+
+      if (CC.Color.BlendEnabled) {
+         gl_blend_span( n, x, y, red, green, blue, alpha, mask );
+      }
+
+      /* Color component masking */
+      if (CC.Color.SWmasking) {
+         gl_mask_color_span( n, x, y, red, green, blue, alpha );
       }
 
       /* write pixels */
-      (*DD.write_color_span)( n, x, y, red, green, blue, alpha, mask );
+      (*DD.write_color_span)( n, x, y, red, green, blue, alpha,
+                              write_all ? NULL : mask );
    }
    else {
       (*DD.write_monocolor_span)( n, x, y, mask );
@@ -510,12 +587,15 @@ void gl_write_monocolor_span( GLuint n, GLint x, GLint y, GLint z[],
  */
 void gl_write_texture_span( GLuint n, GLint x, GLint y, GLint z[],
 			    GLfloat s[], GLfloat t[],
-			    GLubyte red[], GLubyte green[],
-			    GLubyte blue[], GLubyte alpha[],
+			    GLubyte r[], GLubyte g[],
+			    GLubyte b[], GLubyte a[],
 			    GLenum primitive )
 {
    GLuint i;
    GLubyte mask[MAX_WIDTH];
+   GLboolean write_all = GL_TRUE;
+   GLubyte rtmp[MAX_WIDTH], gtmp[MAX_WIDTH], btmp[MAX_WIDTH], atmp[MAX_WIDTH];
+   GLubyte *red, *green, *blue, *alpha;
 
    /* init mask to 1's (all pixels are to be written) */
    for (i=0;i<n;i++)
@@ -525,6 +605,26 @@ void gl_write_texture_span( GLuint n, GLint x, GLint y, GLint z[],
       if (clip_span(n,x,y,mask)==0) {
 	 return;
       }
+      write_all = GL_FALSE;
+   }
+
+
+   if (primitive==GL_BITMAP) {
+      /* must make a copy of the colors since they can be modified */
+      MEMCPY( rtmp, r, n * sizeof(GLubyte) );
+      MEMCPY( gtmp, g, n * sizeof(GLubyte) );
+      MEMCPY( btmp, b, n * sizeof(GLubyte) );
+      MEMCPY( atmp, a, n * sizeof(GLubyte) );
+      red = rtmp;
+      green = gtmp;
+      blue = btmp;
+      alpha = atmp;
+   }
+   else {
+      red   = r;
+      green = g;
+      blue  = b;
+      alpha = a;
    }
 
    /* Texture */
@@ -546,11 +646,13 @@ void gl_write_texture_span( GLuint n, GLint x, GLint y, GLint z[],
       if (gl_scissor_span( n, x, y, mask )==0) {
 	 return;
       }
+      write_all = GL_FALSE;
    }
 
    /* Polygon Stippling */
    if (CC.Polygon.StippleFlag && primitive==GL_POLYGON) {
       stipple_polygon_span( n, x, y, mask );
+      write_all = GL_FALSE;
    }
 
    /* Do the alpha test */
@@ -558,6 +660,7 @@ void gl_write_texture_span( GLuint n, GLint x, GLint y, GLint z[],
       if (gl_alpha_test( n, alpha, mask )==0) {
 	 return;
       }
+      write_all = GL_FALSE;
    }
 
    if (CC.Stencil.Enabled) {
@@ -567,10 +670,17 @@ void gl_write_texture_span( GLuint n, GLint x, GLint y, GLint z[],
       }
       /* depth buffering w/ stencil */
       gl_depth_stencil_span( n, x, y, z, mask );
+      write_all = GL_FALSE;
    }
    else if (CC.Depth.Test) {
       /* regular depth testing */
-      if (gl_depth_test_span( n, x, y, z, mask )==0)  return;
+      GLuint m = gl_depth_test_span( n, x, y, z, mask );
+      if (m==0) {
+         return;
+      }
+      if (m<n) {
+         write_all = GL_FALSE;
+      }
    }
 
    if (CC.Color.ColorMask==0) {
@@ -578,18 +688,22 @@ void gl_write_texture_span( GLuint n, GLint x, GLint y, GLint z[],
       return;
    }
 
+   if (CC.RasterMask & GAMMA_BIT) {
+      gl_apply_gamma( n, red, green, blue );
+   }
+
    /* blending */
    if (CC.Color.BlendEnabled) {
       gl_blend_span( n, x, y, red, green, blue, alpha, mask );
    }
 
-   /* dithering */
-   if (CC.Color.DitherFlag) {
-      gl_dither_color_span( n, x, y, red, green, blue, alpha, mask );
+   if (CC.Color.SWmasking) {
+      gl_mask_color_span( n, x, y, red, green, blue, alpha );
    }
 
    /* write pixels */
-   (*DD.write_color_span)( n, x, y, red, green, blue, alpha, mask );
+   (*DD.write_color_span)( n, x, y, red, green, blue, alpha,
+                           write_all ? NULL : mask );
 }
 
 
@@ -613,7 +727,7 @@ void gl_read_color_span( GLuint n, GLint x, GLint y,
    else {
       if (x>=0 && x+n<=CC.BufferWidth) {
 	 /* OK */
-	 dd_read_color_span( n, x, y, red, green, blue, alpha );
+	 (*DD.read_color_span)( n, x, y, red, green, blue, alpha );
       }
       else {
 	 i = 0;
@@ -626,7 +740,7 @@ void gl_read_color_span( GLuint n, GLint x, GLint y,
 	    }
 	 }
 	 n = MIN2( n, CC.BufferWidth - x );
-	 dd_read_color_span( n, x, y, red+i, green+i, blue+i, alpha+i );
+	 (*DD.read_color_span)( n, x, y, red+i, green+i, blue+i, alpha+i );
       }
    }
 }
@@ -651,7 +765,7 @@ void gl_read_index_span( GLuint n, GLint x, GLint y, GLuint indx[] )
    else {
       if (x>=0 && x+n<=CC.BufferWidth) {
 	 /* OK */
-	 dd_read_index_span( n, x, y, indx );
+	 (*DD.read_index_span)( n, x, y, indx );
       }
       else {
 	 i = 0;
@@ -664,7 +778,7 @@ void gl_read_index_span( GLuint n, GLint x, GLint y, GLuint indx[] )
 	    }
 	 }
 	 n = MIN2( n, CC.BufferWidth - x );
-	 dd_read_index_span( n, x, y, indx+i );
+	 (*DD.read_index_span)( n, x, y, indx+i );
       }
    }
 }
