@@ -22,9 +22,12 @@
 
 
 /*
-$Id: stencil.c,v 1.11 1995/07/24 20:35:20 brianp Exp $
+$Id: stencil.c,v 1.12 1995/11/01 15:13:11 brianp Exp $
 
 $Log: stencil.c,v $
+ * Revision 1.12  1995/11/01  15:13:11  brianp
+ * renamed variable operator as oper per Steven Spitz
+ *
  * Revision 1.11  1995/07/24  20:35:20  brianp
  * replaced memset() with MEMSET() and memcpy() with MEMCPY()
  *
@@ -224,11 +227,11 @@ ENDIF
  * mask flag is set.
  * Input:  n - number of pixels in the span
  *         x, y - location of leftmost pixel in the span
- *         operator - the stencil buffer operator
+ *         oper - the stencil buffer operator
  *         mask - array [n] of flag:  1=apply operator, 0=don't apply operator
  */
 static void apply_stencil_op_to_span( GLuint n, GLint x, GLint y,
-				      GLenum operator, GLubyte mask[] )
+				      GLenum oper, GLubyte mask[] )
 {
    register GLuint i, j;
    register GLuint r, s;
@@ -240,7 +243,7 @@ static void apply_stencil_op_to_span( GLuint n, GLint x, GLint y,
    r = CC.Stencil.Ref;
    j = y * CC.BufferWidth + x;
 
-   switch (operator) {
+   switch (oper) {
       case GL_KEEP:
          /* do nothing */
          break;
@@ -712,7 +715,7 @@ void gl_depth_stencil_span( GLuint n, GLint x, GLint y, GLint z[],
  */
 static void apply_stencil_op_to_pixels( GLuint n, const GLint x[],
 				        const GLint y[],
-				        GLenum operator, GLubyte mask[] )
+				        GLenum oper, GLubyte mask[] )
 {
    register GLuint i, j;
    register GLuint r, s;
@@ -723,7 +726,7 @@ static void apply_stencil_op_to_pixels( GLuint n, const GLint x[],
 
    r = CC.Stencil.Ref;
 
-   switch (operator) {
+   switch (oper) {
       case GL_KEEP:
          /* do nothing */
          break;
