@@ -22,9 +22,15 @@
 
 
 /*
-$Id: vertex.c,v 1.4 1995/07/25 16:41:54 brianp Exp $
+$Id: vertex.c,v 1.6 1995/10/19 15:49:41 brianp Exp $
 
 $Log: vertex.c,v $
+ * Revision 1.6  1995/10/19  15:49:41  brianp
+ * check for argument underflow in glNormal3d[v] functions
+ *
+ * Revision 1.5  1995/10/04  19:35:37  brianp
+ * fixed bugs and optimized glNormal calls
+ *
  * Revision 1.4  1995/07/25  16:41:54  brianp
  * made changes for using CC.VertexFunc pointer
  *
@@ -228,38 +234,34 @@ void glVertex4sv( const GLshort *v )
 
 void glNormal3b( GLbyte nx, GLbyte ny, GLbyte nz )
 {
-   GLfloat n[3];
-
-   n[0] = BYTE_TO_FLOAT(nx);
-   n[1] = BYTE_TO_FLOAT(ny);
-   n[2] = BYTE_TO_FLOAT(nz);
-
    if (CC.CompileFlag) {
-      gl_save_normal( n );
+      GLfloat x, y, z;
+      x = BYTE_TO_FLOAT(nx);
+      y = BYTE_TO_FLOAT(ny);
+      z = BYTE_TO_FLOAT(nz);
+      gl_save_normal3f( x, y, z );
    }
    if (CC.ExecuteFlag) {
-      CC.Current.Normal[0] = n[0];
-      CC.Current.Normal[1] = n[1];
-      CC.Current.Normal[2] = n[2];
+      CC.Current.Normal[0] = BYTE_TO_FLOAT(nz);
+      CC.Current.Normal[1] = BYTE_TO_FLOAT(ny);
+      CC.Current.Normal[2] = BYTE_TO_FLOAT(nz);
    }
 }
 
 
 void glNormal3d( GLdouble nx, GLdouble ny, GLdouble nz )
 {
-   GLfloat n[3];
-
-   n[0] = (GLfloat) nx;
-   n[1] = (GLfloat) ny;
-   n[2] = (GLfloat) nz;
-
+   GLfloat fx, fy, fz;
+   if (ABS(nx)<0.00001)   fx = 0.0F;   else  fx = nx;
+   if (ABS(ny)<0.00001)   fy = 0.0F;   else  fy = ny;
+   if (ABS(nz)<0.00001)   fz = 0.0F;   else  fz = nz;
    if (CC.CompileFlag) {
-      gl_save_normal( n );
+      gl_save_normal3f( fx, fy, fz );
    }
    if (CC.ExecuteFlag) {
-      CC.Current.Normal[0] = n[0];
-      CC.Current.Normal[1] = n[1];
-      CC.Current.Normal[2] = n[2];
+      CC.Current.Normal[0] = fx;
+      CC.Current.Normal[1] = fy;
+      CC.Current.Normal[2] = fz;
    }
 }
 
@@ -267,11 +269,7 @@ void glNormal3d( GLdouble nx, GLdouble ny, GLdouble nz )
 void glNormal3f( GLfloat nx, GLfloat ny, GLfloat nz )
 {
    if (CC.CompileFlag) {
-      GLfloat n[3];
-      n[0] = nx;
-      n[1] = ny;
-      n[2] = nz;
-      gl_save_normal( n );
+      gl_save_normal3f( nx, ny, nz );
    }
    if (CC.ExecuteFlag) {
       CC.Current.Normal[0] = nx;
@@ -283,38 +281,34 @@ void glNormal3f( GLfloat nx, GLfloat ny, GLfloat nz )
 
 void glNormal3i( GLint nx, GLint ny, GLint nz )
 {
-   GLfloat n[3];
-
-   n[0] = INT_TO_FLOAT(nx);
-   n[1] = INT_TO_FLOAT(ny);
-   n[2] = INT_TO_FLOAT(nz);
-
    if (CC.CompileFlag) {
-      gl_save_normal( n );
+      GLfloat x, y, z;
+      x = INT_TO_FLOAT(nx);
+      y = INT_TO_FLOAT(ny);
+      z = INT_TO_FLOAT(nz);
+      gl_save_normal3f( x, y, z );
    }
    if (CC.ExecuteFlag) {
-      CC.Current.Normal[0] = n[0];
-      CC.Current.Normal[1] = n[1];
-      CC.Current.Normal[2] = n[2];
+      CC.Current.Normal[0] = INT_TO_FLOAT(nx);
+      CC.Current.Normal[1] = INT_TO_FLOAT(ny);
+      CC.Current.Normal[2] = INT_TO_FLOAT(nz);
    }
 }
 
 
 void glNormal3s( GLshort nx, GLshort ny, GLshort nz )
 {
-   GLfloat n[3];
-
-   n[0] = SHORT_TO_FLOAT(nx);
-   n[1] = SHORT_TO_FLOAT(ny);
-   n[2] = SHORT_TO_FLOAT(nz);
-
    if (CC.CompileFlag) {
-      gl_save_normal( n );
+      GLfloat x, y, z;
+      x = SHORT_TO_FLOAT(nx);
+      y = SHORT_TO_FLOAT(ny);
+      z = SHORT_TO_FLOAT(nz);
+      gl_save_normal3f( x, y, z );
    }
    if (CC.ExecuteFlag) {
-      CC.Current.Normal[0] = n[0];
-      CC.Current.Normal[1] = n[1];
-      CC.Current.Normal[2] = n[2];
+      CC.Current.Normal[0] = SHORT_TO_FLOAT(nx);
+      CC.Current.Normal[1] = SHORT_TO_FLOAT(ny);
+      CC.Current.Normal[2] = SHORT_TO_FLOAT(nz);
    }
 }
 
@@ -323,38 +317,34 @@ void glNormal3s( GLshort nx, GLshort ny, GLshort nz )
 
 void glNormal3bv( const GLbyte *v )
 {
-   GLfloat n[3];
-
-   n[0] = BYTE_TO_FLOAT(v[0]);
-   n[1] = BYTE_TO_FLOAT(v[0]);
-   n[2] = BYTE_TO_FLOAT(v[0]);
-
    if (CC.CompileFlag) {
-      gl_save_normal( n );
+      GLfloat x, y, z;
+      x = BYTE_TO_FLOAT(v[0]);
+      y = BYTE_TO_FLOAT(v[1]);
+      z = BYTE_TO_FLOAT(v[2]);
+      gl_save_normal3f( x, y, z );
    }
    if (CC.ExecuteFlag) {
-      CC.Current.Normal[0] = n[0];
-      CC.Current.Normal[1] = n[1];
-      CC.Current.Normal[2] = n[2];
+      CC.Current.Normal[0] = BYTE_TO_FLOAT(v[0]);
+      CC.Current.Normal[1] = BYTE_TO_FLOAT(v[1]);
+      CC.Current.Normal[2] = BYTE_TO_FLOAT(v[2]);
    }
 }
 
 
 void glNormal3dv( const GLdouble *v )
 {
-   GLfloat n[3];
-
-   n[0] = (GLfloat) v[0];
-   n[1] = (GLfloat) v[1];
-   n[2] = (GLfloat) v[2];
-
+   GLfloat fx, fy, fz;
+   if (ABS(v[0])<0.00001)   fx = 0.0F;   else  fx = v[0];
+   if (ABS(v[1])<0.00001)   fy = 0.0F;   else  fy = v[1];
+   if (ABS(v[2])<0.00001)   fz = 0.0F;   else  fz = v[2];
    if (CC.CompileFlag) {
-      gl_save_normal( n );
+      gl_save_normal3f( fx, fy, fz );
    }
    if (CC.ExecuteFlag) {
-      CC.Current.Normal[0] = n[0];
-      CC.Current.Normal[1] = n[1];
-      CC.Current.Normal[2] = n[2];
+      CC.Current.Normal[0] = fx;
+      CC.Current.Normal[1] = fy;
+      CC.Current.Normal[2] = fz;
    }
 }
 
@@ -362,7 +352,7 @@ void glNormal3dv( const GLdouble *v )
 void glNormal3fv( const GLfloat *v )
 {
    if (CC.CompileFlag) {
-      gl_save_normal( v );
+      gl_save_normal3fv( v );
    }
    if (CC.ExecuteFlag) {
       CC.Current.Normal[0] = v[0];
@@ -374,38 +364,34 @@ void glNormal3fv( const GLfloat *v )
 
 void glNormal3iv( const GLint *v )
 {
-   GLfloat n[3];
-
-   n[0] = INT_TO_FLOAT(v[0]);
-   n[1] = INT_TO_FLOAT(v[0]);
-   n[2] = INT_TO_FLOAT(v[0]);
-
    if (CC.CompileFlag) {
-      gl_save_normal( n );
+      GLfloat x, y, z;
+      x = INT_TO_FLOAT(v[0]);
+      y = INT_TO_FLOAT(v[1]);
+      z = INT_TO_FLOAT(v[2]);
+      gl_save_normal3f( x, y, z );
    }
    if (CC.ExecuteFlag) {
-      CC.Current.Normal[0] = n[0];
-      CC.Current.Normal[1] = n[1];
-      CC.Current.Normal[2] = n[2];
+      CC.Current.Normal[0] = INT_TO_FLOAT(v[0]);
+      CC.Current.Normal[1] = INT_TO_FLOAT(v[1]);
+      CC.Current.Normal[2] = INT_TO_FLOAT(v[2]);
    }
 }
 
 
 void glNormal3sv( const GLshort *v )
 {
-   GLfloat n[3];
-
-   n[0] = SHORT_TO_FLOAT(v[0]);
-   n[1] = SHORT_TO_FLOAT(v[1]);
-   n[2] = SHORT_TO_FLOAT(v[2]);
-
    if (CC.CompileFlag) {
-      gl_save_normal( n );
+      GLfloat x, y, z;
+      x = SHORT_TO_FLOAT(v[0]);
+      y = SHORT_TO_FLOAT(v[1]);
+      z = SHORT_TO_FLOAT(v[2]);
+      gl_save_normal3f( x, y, z );
    }
    if (CC.ExecuteFlag) {
-      CC.Current.Normal[0] = n[0];
-      CC.Current.Normal[1] = n[1];
-      CC.Current.Normal[2] = n[2];
+      CC.Current.Normal[0] = SHORT_TO_FLOAT(v[0]);
+      CC.Current.Normal[1] = SHORT_TO_FLOAT(v[1]);
+      CC.Current.Normal[2] = SHORT_TO_FLOAT(v[2]);
    }
 }
 
