@@ -2,7 +2,7 @@
 
 /*
  * Mesa 3-D graphics library
- * Version:  1.2 beta
+ * Version:  1.2
  * Copyright (C) 1995  Brian Paul  (brianp@ssec.wisc.edu)
  *
  * This library is free software; you can redistribute it and/or
@@ -23,9 +23,25 @@
  */
 
 /*
-$Id: wmesa.c,v 1.5 1995/07/24 18:56:00 brianp Exp $
+$Id: wmesa.c,v 1.10 1995/11/20 20:11:08 brianp Exp $
 
 $Log: wmesa.c,v $
+ * Revision 1.10  1995/11/20  20:11:08  brianp
+ * incorporated Mark's November 17th updates
+ *
+ * Revision 1.9  1995/10/30  15:30:00  brianp
+ * pass mask array to read_[index|color]_pixels()
+ *
+ * Revision 1.8  1995/10/19  15:44:42  brianp
+ * updated arguments to set_color, clear_color and gl_new_context
+ *
+ * Revision 1.7  1995/10/17  21:28:01  brianp
+ * updated for new device driver structure in Mesa 1.2.4
+ *
+ * Revision 1.6  1995/09/15  18:52:55  brianp
+ * cleaned up some code
+ * modified write_color_span to take NULL mask
+ *
  * Revision 1.5  1995/07/24  18:56:00  brianp
  * added dd_finish()
  *
@@ -48,11 +64,10 @@ $Log: wmesa.c,v $
 #include <stdlib.h>
 #include <windows.h>
 #include "gl\wmesa.h"
-#include "bresenhm.h"
 #include "context.h"
 #include "dd.h"
-#include "quikpoly.h"
 #include "xform.h"
+#include "vb.h"
 #include "wing.h"
 
 
@@ -73,8 +88,6 @@ struct wmesa_context
   GLuint depth;		/* bits per pixel (1, 8, 24, etc) */
   unsigned long pixel;	/* current color index or RGBA pixel value */
   unsigned long clearpixel; /* pixel for clearing the color buffers */
-  GLint rmult, gmult, bmult, amult;	/* Multiplier */
-	GLint rshift, gshift, bshift, ashift;	/* Bit shifts */
   char *ScreenMem; // WinG memory
   BITMAPINFO *IndexFormat;
   HPALETTE hPal; // Current Palette
@@ -98,144 +111,53 @@ struct wmesa_context
 static WMesaContext Current = NULL;
 #ifdef __SYMANTEC_BUGS
   struct dd_function_table DD;
-  #include "vb.h"
-  struct vertex_buffer VB;
 #endif
 #define FLIP(Y)  (Current->height-(Y)-1)
+
+
+
+
+
 /* Finish all pending operations and synchronize. */
-void dd_finish(void)
+static void finish(void)
 {
+   /* no op */
 }
-void dd_flush(void)
+
+
+
+static void flush(void)
 {
+   /* no op */
 }
-/* Return characteristics of the output buffer. */
-void dd_buffer_info( GLuint *width, GLuint *height,GLuint *depth)
-{
-  int New_Size;
-  RECT CR;
-  GetClientRect(Current->Window,&CR);
-  *width=CR.right;
-  *height=CR.bottom;
-  *depth = Current->depth;
-  New_Size=((*width)!=Current->width) || ((*height)!=Current->height);
-  if (New_Size)
-  {
-    Current->width=*width;
-    Current->ScanWidth=Current->width;
-    if ((Current->ScanWidth%sizeof(long))!=0)
-      Current->ScanWidth+=(sizeof(long)-(Current->ScanWidth%sizeof(long)));
-    Current->height=*height;
-    if (Current->db_flag)
-    {
-      if (Current->rgb_flag==GL_TRUE)
-        Current->Compat_BM=CreateCompatibleBitmap(Current->Compat_DC,Current->width,Current->height);
-      else
-      {
-        Current->IndexFormat->bmiHeader.biWidth=Current->width;
-        if (Current->IndexFormat->bmiHeader.biHeight<0)
-          Current->IndexFormat->bmiHeader.biHeight=-Current->height;
-        else
-          Current->IndexFormat->bmiHeader.biHeight=Current->height;
-        Current->Compat_BM=WinGCreateBitmap(Current->Compat_DC,Current->IndexFormat,&((void *) Current->ScreenMem));
-      }
-      DeleteObject(SelectObject(Current->Compat_DC,Current->Compat_BM));
-    }
-  }
-}
-/*
- * Specify which buffer to read from.
- * Input:  mode - as passed to glReadBuffer
- * Return:  new mode or GL_FALSE if error.
- */
-GLenum dd_read_buffer(GLenum mode)
-{
-  GLenum m;
-  switch (mode)
-  {
-    case GL_FRONT:
-    case GL_FRONT_LEFT:
-    case GL_LEFT:
-      m = mode;
-	    //Current->Readable = Current->Window;
-    break;
-    case GL_BACK:
-    case GL_BACK_LEFT:
-	    m = mode;
-      //if (Current->backpixmap)
-	    //  Current->readable = Current->backpixmap;
-      //else if (Current->backimage)
-	    //  Current->readable = None;
-    break;
-    case GL_AUX0:
-    case GL_FRONT_RIGHT:
-    case GL_RIGHT:
-    case GL_BACK_RIGHT:
-    default:
-	    return GL_FALSE;
-  }
-  return m;
-}
-/*
- * Specify which buffer(s) to draw into.
- * Input:  mode - as passed to glDrawBuffer
- * Return:  new output mode or GL_FALSE if error.
- */
-GLenum dd_draw_buffer(GLenum mode)
-{
-  GLenum m;
-  switch (mode)
-  {
-    case GL_NONE:
-      m = GL_NONE;
-      //Current->dest = 0;
-    break;
-    case GL_FRONT:
-    case GL_FRONT_LEFT:
-	    m = mode;
-	    //Current->dest = FRONT_PIXMAP;
-	  break;
-    case GL_BACK:
-    case GL_BACK_LEFT:
-	    m = mode;
-      //Current->dest = 0;
-    break;
-    case GL_LEFT:
-    case GL_FRONT_AND_BACK:
-	    m = mode;
-      //Current->dest = FRONT_PIXMAP;
-    break;
-    case GL_RIGHT:
-    case GL_FRONT_RIGHT:
-    case GL_BACK_RIGHT:
-    case GL_AUX0:
-    default:
-	   /* Don't change drawbuffer, return error signal */
-	   return GL_FALSE;
-  }
-  return m;
-}
+
+
+
 /*
  * Set the color index used to clear the color buffer.
- * This implements glClearIndex().
  */
-void dd_clear_index(GLuint index)
+static void clear_index(GLuint index)
 {
   Current->clearpixel = index;
 }
+
+
+
 /*
  * Set the color used to clear the color buffer.
- * This implements glClearColor().
  */
-void dd_clear_color(const GLfloat color[4])
+static void clear_color( GLubyte r, GLubyte g, GLubyte b, GLubyte a )
 {
-  Current->clearpixel=RGB(color[0],color[1],color[2]);
+  Current->clearpixel=RGB(r, g, b );
 }
+
+
+
 /*
  * Clear the specified region of the color buffer using the clear color
  * or index as specified by one of the two functions above.
  */
-void dd_clear(GLboolean all,GLint x, GLint y, GLint width, GLint height )
+static void clear(GLboolean all,GLint x, GLint y, GLint width, GLint height )
 {
   if (all)
   {
@@ -268,119 +190,280 @@ void dd_clear(GLboolean all,GLint x, GLint y, GLint width, GLint height )
     }
   }
 }
+
+
+
+/* Set the current color index. */
+static void set_index(GLuint index)
+{
+  Current->pixel=index;
+}
+
+
+
+/* Set the current RGBA color. */
+static void set_color( GLubyte r, GLubyte g, GLubyte b, GLubyte a )
+{
+  Current->pixel = RGB( r, g, b );
+}
+
+
+
+/* Set the index mode bitplane mask. */
+static GLboolean index_mask(GLuint mask)
+{
+   /* can't implement */
+   return GL_FALSE;
+}
+
+
+
+/* Set the RGBA drawing mask. */
+static GLboolean color_mask( GLboolean rmask, GLboolean gmask,
+                             GLboolean bmask, GLboolean amask)
+{
+   /* can't implement */
+   return GL_FALSE;
+}
+
+
+
 /*
  * Set the pixel logic operation.  Return GL_TRUE if the device driver
  * can perform the operation, otherwise return GL_FALSE.  If GL_FALSE
  * is returned, the logic op will be done in software by Mesa.
  */
-GLboolean dd_logicop( GLenum op )
+GLboolean logicop( GLenum op )
 {
-  return GL_FALSE;
+   /* can't implement */
+   return GL_FALSE;
 }
-/**********************************************************************/
-/*****                Simple rendering functions                  *****/
-/**********************************************************************/
-/* Set the current color index. */
-void dd_index(GLuint index)
+
+
+static void dither( GLboolean enable )
 {
-  Current->pixel=index;
+   /* No op */
 }
-/* Set the index mode bitplane mask. */
-void dd_index_mask(GLuint mask)
+
+
+
+static GLboolean set_buffer( GLenum mode )
 {
-  assert(0);
+   /* TODO: this could be better */
+   if (mode==GL_FRONT || mode==GL_BACK) {
+      return GL_TRUE;
+   }
+   else {
+      return GL_FALSE;
+   }
 }
-/* Set the current RGBA color. */
-void dd_color( const GLfloat color[4] )
+
+
+
+/* Return characteristics of the output buffer. */
+static void buffer_size( GLuint *width, GLuint *height, GLuint *depth )
 {
-  Current->pixel=RGB(color[0]*Current->rmult,
-                     color[1]*Current->gmult,
-                     color[2]*Current->bmult);
-}
-/* Set the RGBA drawing mask. */
-void dd_color_mask( GLboolean rmask, GLboolean gmask,GLboolean bmask, GLboolean amask)
-{
-  assert(0);
-}
-/* Write a pixel using current index or color. */
-void dd_draw_pixel( GLint x, GLint y )
-{
-  if (Current->rgb_flag==GL_TRUE)
+  int New_Size;
+  RECT CR;
+  GetClientRect(Current->Window,&CR);
+  *width=CR.right;
+  *height=CR.bottom;
+  *depth = Current->depth;
+  New_Size=((*width)!=Current->width) || ((*height)!=Current->height);
+  if (New_Size)
   {
-    HDC DC=DD_GETDC;
-    y=FLIP(y);
-    SetPixel(DC,x,y,Current->pixel);
-    DD_RELEASEDC;
-  }
-  else
-    Current->ScreenMem[(y*Current->ScanWidth)+x]=Current->pixel;
-}
-#define BRESENHAM_PLOT(x,y) dd_draw_pixel(x,y);
-/* Draw a line using the current index or color. */
-void dd_draw_line( GLint x0, GLint y0, GLint x1, GLint y1 )
-{
-  if (Current->rgb_flag==GL_TRUE)
-  {
-    HDC DC=DD_GETDC;
-    HPEN Pen=CreatePen(PS_SOLID,1,Current->pixel);
-    HPEN Old_Pen=SelectObject(DC,Pen);
-    y0=FLIP(y0);
-    y1=FLIP(y1);
-    MoveToEx(DC,x0,y0,NULL);
-    LineTo(DC,x1,y1);
-    SelectObject(DC,Old_Pen);
-    DeleteObject(Pen);
-    DD_RELEASEDC;
-  }
-  else
-  {
-    if (y0==y1)
+    Current->width=*width;
+    Current->ScanWidth=Current->width;
+    if ((Current->ScanWidth%sizeof(long))!=0)
+      Current->ScanWidth+=(sizeof(long)-(Current->ScanWidth%sizeof(long)));
+    Current->height=*height;
+    if (Current->db_flag)
     {
-      if (x1<x0)
+      if (Current->rgb_flag==GL_TRUE)
+        Current->Compat_BM=CreateCompatibleBitmap(Current->Compat_DC,Current->width,Current->height);
+      else
       {
-        GLint t=x1;
-        x1=x0;
-        x0=t;
+        Current->IndexFormat->bmiHeader.biWidth=Current->width;
+        if (Current->IndexFormat->bmiHeader.biHeight<0)
+          Current->IndexFormat->bmiHeader.biHeight=-Current->height;
+        else
+          Current->IndexFormat->bmiHeader.biHeight=Current->height;
+        Current->Compat_BM=WinGCreateBitmap(Current->Compat_DC,Current->IndexFormat,&((void *) Current->ScreenMem));
       }
-      memset(Current->ScreenMem+y0*Current->ScanWidth+x0,Current->pixel,x1-x0+1);
+      DeleteObject(SelectObject(Current->Compat_DC,Current->Compat_BM));
     }
-    else
-      BRESENHAM(x0,y0,x1,y1);
   }
 }
-/* Draw a convex polygon using the current index or color. */
-void dd_draw_polygon( GLuint n, GLint x[], GLint y[] )
+
+
+
+/**********************************************************************/
+/*****           Accelerated point, line, polygon rendering       *****/
+/**********************************************************************/
+
+
+static void fast_rgb_points( GLuint first, GLuint last )
 {
-  if (Current->rgb_flag==GL_TRUE)
-  {
-    POINT *Pts=(POINT *) malloc(n*sizeof(POINT));
-    int i;
-    HDC DC=DD_GETDC;
-    HPEN Pen=CreatePen(PS_SOLID,1,Current->pixel);
-    HBRUSH Brush=CreateSolidBrush(Current->pixel);
-    HPEN Old_Pen=SelectObject(DC,Pen);
-    HBRUSH Old_Brush=SelectObject(DC,Brush);
-    for (i=0; i<n; i++)
-    {
-      Pts[i].x=x[i];
-      Pts[i].y=FLIP(y[i]);
-    }
-    Polygon(DC,Pts,n);
-    SelectObject(DC,Old_Pen);
-    SelectObject(DC,Old_Brush);
-    DeleteObject(Pen);
-    DeleteObject(Brush);
-    DD_RELEASEDC;
-    free(Pts);
-  }
-  else
-    gl_quick_polygon(n,x,y);
+   int i;
+   HDC DC=DD_GETDC;
+   if (VB.MonoColor) {
+      /* all drawn with current color */
+      for (i=first;i<=last;i++) {
+         if (VB.Unclipped[i]) {
+            int x, y;
+            x =       (GLint) (VB.Win[i][0] + 0.05F);
+            y = FLIP( (GLint) (VB.Win[i][1] + 0.05F) );
+            SetPixel(DC,x,y,Current->pixel);
+         }
+      }
+   }
+   else {
+      /* draw points of different colors */
+      HPEN Pen=CreatePen(PS_SOLID,1,Current->pixel);
+      HPEN Old_Pen=SelectObject(DC,Pen);
+      for (i=first;i<=last;i++) {
+         if (VB.Unclipped[i]) {
+            int x, y;
+            unsigned long pixel=RGB(VB.Color[i][0]*255.0,
+                                    VB.Color[i][1]*255.0,
+                                    VB.Color[i][2]*255.0);
+            x =       (GLint) (VB.Win[i][0] + 0.05F);
+            y = FLIP( (GLint) (VB.Win[i][1] + 0.05F) );
+            SetPixel(DC,x,y,pixel);
+         }
+      }
+      SelectObject(DC,Old_Pen);
+      DeleteObject(Pen);
+   }
+   DD_RELEASEDC;
 }
+
+
+
+/* Return pointer to accerated points function */
+static points_func choose_points_function( void )
+{
+   if (CC.Point.Size==1.0 && !CC.Point.SmoothFlag && CC.RasterMask==0
+       && !CC.Texture.Enabled  && Current->rgb_flag) {
+      return fast_rgb_points;
+   }
+   else {
+      return NULL;
+   }
+}
+
+
+
+/* Draw a line using the color specified by VB.Color[pv] */
+static void fast_flat_rgb_line( GLuint v0, GLuint v1, GLuint pv )
+{
+   int x0, y0, x1, y1;
+   unsigned long pixel;
+   HDC DC=DD_GETDC;
+   HPEN Pen;
+   HPEN Old_Pen;
+
+   if (VB.MonoColor) {
+      pixel = Current->pixel;  /* use current color */
+   }
+   else {
+      pixel = RGB(VB.Color[pv][0]*255.0, VB.Color[pv][1]*255.0, VB.Color[pv][2]*255.0);
+   }
+
+   x0 =       (int) (VB.Win[v0][0] + 0.05F);
+   y0 = FLIP( (int) (VB.Win[v0][1] + 0.05F) );
+   x1 =       (int) (VB.Win[v1][0] + 0.05F);
+   y1 = FLIP( (int) (VB.Win[v1][1] + 0.05F) );
+
+   Pen=CreatePen(PS_SOLID,1,pixel);
+   Old_Pen=SelectObject(DC,Pen);
+   MoveToEx(DC,x0,y0,NULL);
+   LineTo(DC,x1,y1);
+   SelectObject(DC,Old_Pen);
+   DeleteObject(Pen);
+   DD_RELEASEDC;
+}
+
+
+
+/* Return pointer to accerated line function */
+static line_func choose_line_function( void )
+{
+   if (CC.Line.Width==1.0 && !CC.Line.SmoothFlag && !CC.Line.StippleFlag
+       && CC.Light.ShadeModel==GL_FLAT && CC.RasterMask==0
+       && !CC.Texture.Enabled && Current->rgb_flag) {
+      return fast_flat_rgb_line;
+   }
+   else {
+      return NULL;
+   }
+}
+
+
+/* Draw a convex polygon using color VB.Color[pv] */
+static void fast_flat_rgb_polygon( GLuint n, GLuint vlist[], GLuint pv )
+{
+   POINT *Pts=(POINT *) malloc(n*sizeof(POINT));
+   HDC DC=DD_GETDC;
+   HPEN Pen;
+   HBRUSH Brush;
+   HPEN Old_Pen;
+   HBRUSH Old_Brush;
+   GLint pixel;
+   int i;
+
+   if (VB.MonoColor) {
+      pixel = Current->pixel;  /* use current color */
+   }
+   else {
+      pixel = RGB(VB.Color[pv][0]*255.0, VB.Color[pv][1]*255.0, VB.Color[pv][2]*255.0);
+   }
+
+   Pen=CreatePen(PS_SOLID,1,pixel);
+   Brush=CreateSolidBrush(pixel);
+   Old_Pen=SelectObject(DC,Pen);
+   Old_Brush=SelectObject(DC,Brush);
+
+   for (i=0; i<n; i++) {
+      int j = vlist[i];
+      Pts[i].x =       (int) (VB.Win[j][0] + 0.5F);
+      Pts[i].y = FLIP( (int) (VB.Win[j][1] + 0.5F) );
+   }
+   Polygon(DC,Pts,n);
+   SelectObject(DC,Old_Pen);
+   SelectObject(DC,Old_Brush);
+   DeleteObject(Pen);
+   DeleteObject(Brush);
+   DD_RELEASEDC;
+   free(Pts);
+}
+
+
+
+/* Return pointer to accerated polygon function */
+static polygon_func choose_polygon_function( void )
+{
+   if (!CC.Polygon.SmoothFlag && !CC.Polygon.StippleFlag
+       && CC.Light.ShadeModel==GL_FLAT && CC.RasterMask==0
+       && !CC.Texture.Enabled && Current->rgb_flag==GL_TRUE) {
+      return fast_flat_rgb_polygon;
+   }
+   else {
+      return NULL;
+   }
+}
+
+
+
 /**********************************************************************/
-/*****             Span-based pixel drawing and reading           *****/
+/*****                 Span-based pixel drawing                   *****/
 /**********************************************************************/
+
+
 /* Write a horizontal span of color-index pixels with a boolean mask. */
-void dd_write_index_span( GLuint n, GLint x, GLint y,const GLuint index[], const GLubyte mask[] )
+static void write_index_span( GLuint n, GLint x, GLint y,
+                              const GLuint index[],
+                              const GLubyte mask[] )
 {
   int i;
   char *Mem=Current->ScreenMem+y*Current->ScanWidth+x;
@@ -389,11 +472,14 @@ void dd_write_index_span( GLuint n, GLint x, GLint y,const GLuint index[], const
     if (mask[i])
       Mem[i]=index[i];
 }
+
+
+
 /*
  * Write a horizontal span of pixels with a boolean mask.  The current
  * color index is used for all pixels.
  */
-void dd_write_monoindex_span(GLuint n,GLint x,GLint y,const GLubyte mask[])
+static void write_monoindex_span(GLuint n,GLint x,GLint y,const GLubyte mask[])
 {
   int i;
   char *Mem=Current->ScreenMem+y*Current->ScanWidth+x;
@@ -402,17 +488,11 @@ void dd_write_monoindex_span(GLuint n,GLint x,GLint y,const GLubyte mask[])
     if (mask[i])
       Mem[i]=Current->pixel;
 }
-/* Read a horizontal span of color-index pixels. */
-void dd_read_index_span( GLuint n, GLint x, GLint y, GLuint index[])
-{
-  int i;
-  char *Mem=Current->ScreenMem+y*Current->ScanWidth+x;
-  assert(Current->rgb_flag==GL_FALSE);
-  for (i=0; i<n; i++)
-    index[i]=Mem[i];
-}
+
+
+
 /* Write a horizontal span of color pixels with a boolean mask. */
-void dd_write_color_span( GLuint n, GLint x, GLint y,
+static void write_color_span( GLuint n, GLint x, GLint y,
 			  const GLubyte red[], const GLubyte green[],
 			  const GLubyte blue[], const GLubyte alpha[],
 			  const GLubyte mask[] )
@@ -422,25 +502,40 @@ void dd_write_color_span( GLuint n, GLint x, GLint y,
     int i;
     HDC DC=DD_GETDC;
     y=FLIP(y);
-    for (i=0; i<n; i++)
-      if (mask[i])
-        SetPixel(DC,x+i,y,RGB(red[i],green[i],blue[i]));
+    if (mask) {
+       for (i=0; i<n; i++)
+         if (mask[i])
+           SetPixel(DC,x+i,y,RGB(red[i],green[i],blue[i]));
+    }
+    else {
+       for (i=0; i<n; i++)
+         SetPixel(DC,x+i,y,RGB(red[i],green[i],blue[i]));
+    }
     DD_RELEASEDC;
   }
   else
   {
     int i;
     char *Mem=Current->ScreenMem+y*Current->ScanWidth+x;
-    for (i=0; i<n; i++)
-      if (mask[i])
-        Mem[i]=GetNearestPaletteIndex(Current->hPal,RGB(red[i],green[i],blue[i]));
+    if (mask) {
+       for (i=0; i<n; i++)
+         if (mask[i])
+           Mem[i]=GetNearestPaletteIndex(Current->hPal,RGB(red[i],green[i],blue[i]));
+    }
+    else {
+       for (i=0; i<n; i++)
+         Mem[i]=GetNearestPaletteIndex(Current->hPal,RGB(red[i],green[i],blue[i]));
+    }
   }
 }
+
+
+
 /*
  * Write a horizontal span of pixels with a boolean mask.  The current color
  * is used for all pixels.
  */
-void dd_write_monocolor_span( GLuint n, GLint x, GLint y,const GLubyte mask[])
+static void write_monocolor_span( GLuint n, GLint x, GLint y,const GLubyte mask[])
 {
   int i;
   HDC DC=DD_GETDC;
@@ -451,30 +546,123 @@ void dd_write_monocolor_span( GLuint n, GLint x, GLint y,const GLubyte mask[])
       SetPixel(DC,x+i,y,Current->pixel);
   DD_RELEASEDC;
 }
-/* Read an array of color pixels. */
-void dd_read_color_pixels( GLuint n, const GLint x[], const GLint y[],
-				  GLubyte red[], GLubyte green[],
-				  GLubyte blue[], GLubyte alpha[] )
+
+
+
+/**********************************************************************/
+/*****                   Array-based pixel drawing                *****/
+/**********************************************************************/
+
+
+/* Write an array of pixels with a boolean mask. */
+static void write_index_pixels( GLuint n, const GLint x[], const GLint y[],
+                                const GLuint index[], const GLubyte mask[] )
+{
+   int i;
+   assert(Current->rgb_flag==GL_FALSE);
+   for (i=0; i<n; i++) {
+      if (mask[i]) {
+         char *Mem=Current->ScreenMem+y[i]*Current->ScanWidth+x[i];
+         *Mem = index[i];
+      }
+   }
+}
+
+
+
+/*
+ * Write an array of pixels with a boolean mask.  The current color
+ * index is used for all pixels.
+ */
+static void write_monoindex_pixels( GLuint n,
+                                    const GLint x[], const GLint y[],
+                                    const GLubyte mask[] )
+{
+   int i;
+   assert(Current->rgb_flag==GL_FALSE);
+   for (i=0; i<n; i++) {
+      if (mask[i]) {
+         char *Mem=Current->ScreenMem+y[i]*Current->ScanWidth+x[i];
+         *Mem = Current->pixel;
+      }
+   }
+}
+
+
+
+/* Write an array of pixels with a boolean mask. */
+static void write_color_pixels( GLuint n, const GLint x[], const GLint y[],
+                                const GLubyte r[], const GLubyte g[],
+                                const GLubyte b[], const GLubyte a[],
+                                const GLubyte mask[] )
 {
   int i;
-  COLORREF Color;
   HDC DC=DD_GETDC;
   assert(Current->rgb_flag==GL_TRUE);
   for (i=0; i<n; i++)
-  {
-    Color=GetPixel(DC,x[i],FLIP(y[i]));
-    red[i]=GetRValue(Color);
-    green[i]=GetGValue(Color);
-    blue[i]=GetBValue(Color);
-    alpha[i]=255;
-  }
+    if (mask[i])
+      SetPixel(DC,x[i],FLIP(y[i]),RGB(r[i],g[i],b[i]));
   DD_RELEASEDC;
-  memset(alpha,0,n*sizeof(GLint));
 }
+
+
+
+/*
+ * Write an array of pixels with a boolean mask.  The current color
+ * is used for all pixels.
+ */
+static void write_monocolor_pixels( GLuint n,
+                                    const GLint x[], const GLint y[],
+                                    const GLubyte mask[] )
+{
+  int i;
+  HDC DC=DD_GETDC;
+  assert(Current->rgb_flag==GL_TRUE);
+  for (i=0; i<n; i++)
+    if (mask[i])
+      SetPixel(DC,x[i],FLIP(y[i]),Current->pixel);
+  DD_RELEASEDC;
+}
+
+
+
+/**********************************************************************/
+/*****            Read spans/arrays of pixels                     *****/
+/**********************************************************************/
+
+
+/* Read a horizontal span of color-index pixels. */
+static void read_index_span( GLuint n, GLint x, GLint y, GLuint index[])
+{
+  int i;
+  char *Mem=Current->ScreenMem+y*Current->ScanWidth+x;
+  assert(Current->rgb_flag==GL_FALSE);
+  for (i=0; i<n; i++)
+    index[i]=Mem[i];
+}
+
+
+
+
+/* Read an array of color index pixels. */
+static void read_index_pixels( GLuint n, const GLint x[], const GLint y[],
+                               GLuint indx[], const GLubyte mask[] )
+{
+  int i;
+  assert(Current->rgb_flag==GL_FALSE);
+  for (i=0; i<n; i++) {
+     if (mask[i]) {
+        indx[i]=*(Current->ScreenMem+y[i]*Current->ScanWidth+x[i]);
+     }
+  }
+}
+
+
+
 /* Read a horizontal span of color pixels. */
-void dd_read_color_span( GLuint n, GLint x, GLint y,
-		         GLubyte red[], GLubyte green[],
-			 GLubyte blue[], GLubyte alpha[] )
+static void read_color_span( GLuint n, GLint x, GLint y,
+                             GLubyte red[], GLubyte green[],
+                             GLubyte blue[], GLubyte alpha[] )
 {
   int i;
   COLORREF Color;
@@ -492,82 +680,89 @@ void dd_read_color_span( GLuint n, GLint x, GLint y,
   DD_RELEASEDC;
   memset(alpha,0,n*sizeof(GLint));
 }
-/**********************************************************************/
-/*****            Array-based pixel drawing and reading           *****/
-/**********************************************************************/
-/* Write an array of pixels with a boolean mask. */
-void dd_write_index_pixels( GLuint n, const GLint x[], const GLint y[],
-				  const GLuint index[], const GLubyte mask[] )
-{
-  int i;
-  assert(Current->rgb_flag==GL_FALSE);
-  for (i=0; i<n; i++)
-  {
-    if (mask[i])
-    {
-      dd_index(index[i]);
-      dd_draw_pixel(x[i],y[i]);
-    }
-  }
-}
-/*
- * Write an array of pixels with a boolean mask.  The current color
- * index is used for all pixels.
- */
-void dd_write_monoindex_pixels( GLuint n,
-				       const GLint x[], const GLint y[],
-				       const GLubyte mask[] )
-{
-  int i;
-  assert(Current->rgb_flag==GL_FALSE);
-  for (i=0; i<n; i++)
-  {
-    if (mask[i])
-      dd_draw_pixel(x[i],y[i]);
-  }
-}
-/* Read an array of color index pixels. */
-void dd_read_index_pixels( GLuint n,
-				  const GLint x[], const GLint y[],
-				  GLuint indx[] )
-{
-  int i;
-  assert(Current->rgb_flag==GL_FALSE);
-  for (i=0; i<n; i++)
-    indx[i]=*(Current->ScreenMem+y[i]*Current->ScanWidth+x[i]);
-}
-/* Write an array of pixels with a boolean mask. */
-void dd_write_color_pixels( GLuint n, const GLint x[], const GLint y[],
-				   const GLubyte r[], const GLubyte g[],
-				   const GLubyte b[], const GLubyte a[],
-				   const GLubyte mask[] )
+
+
+/* Read an array of color pixels. */
+static void read_color_pixels( GLuint n, const GLint x[], const GLint y[],
+                               GLubyte red[], GLubyte green[],
+                               GLubyte blue[], GLubyte alpha[],
+                               const GLubyte mask[] )
 {
   int i;
+  COLORREF Color;
   HDC DC=DD_GETDC;
   assert(Current->rgb_flag==GL_TRUE);
-  for (i=0; i<n; i++)
-    if (mask[i])
-      SetPixel(DC,x[i],FLIP(y[i]),RGB(r[i],g[i],b[i]));
+  for (i=0; i<n; i++) {
+     if (mask[i]) {
+        Color=GetPixel(DC,x[i],FLIP(y[i]));
+        red[i]=GetRValue(Color);
+        green[i]=GetGValue(Color);
+        blue[i]=GetBValue(Color);
+        alpha[i]=255;
+     }
+  }
   DD_RELEASEDC;
+  memset(alpha,0,n*sizeof(GLint));
 }
-/*
- * Write an array of pixels with a boolean mask.  The current color
- * is used for all pixels.
- */
-void dd_write_monocolor_pixels( GLuint n,
-				       const GLint x[], const GLint y[],
-    		       const GLubyte mask[] )
+
+
+
+/**********************************************************************/
+/**********************************************************************/
+
+
+
+static void setup_DD_pointers( void )
 {
-  int i;
-  HDC DC=DD_GETDC;
-  assert(Current->rgb_flag==GL_TRUE);
-  for (i=0; i<n; i++)
-    if (mask[i])
-      SetPixel(DC,x[i],FLIP(y[i]),Current->pixel);
-  DD_RELEASEDC;
+   DD.finish = finish;
+   DD.flush = flush;
+
+   DD.clear_index = clear_index;
+   DD.clear_color = clear_color;
+   DD.clear = clear;
+
+   DD.index = set_index;
+   DD.color = set_color;
+   DD.index_mask = index_mask;
+   DD.color_mask = color_mask;
+
+   DD.logicop = logicop;
+   DD.dither = dither;
+
+   DD.set_buffer = set_buffer;
+   DD.buffer_size = buffer_size;
+
+   DD.get_points_func = choose_points_function;
+   DD.get_line_func = choose_line_function;
+   DD.get_polygon_func = choose_polygon_function;
+
+   /* Pixel/span writing functions: */
+   DD.write_color_span       = write_color_span;
+   DD.write_monocolor_span   = write_monocolor_span;
+   DD.write_color_pixels     = write_color_pixels;
+   DD.write_monocolor_pixels = write_monocolor_pixels;
+   DD.write_index_span       = write_index_span;
+   DD.write_monoindex_span   = write_monoindex_span;
+   DD.write_index_pixels     = write_index_pixels;
+   DD.write_monoindex_pixels = write_monoindex_pixels;
+
+   /* Pixel/span reading functions: */
+   DD.read_index_span = read_index_span;
+   DD.read_color_span = read_color_span;
+   DD.read_index_pixels = read_index_pixels;
+   DD.read_color_pixels = read_color_pixels;
 }
+
+
+
+/**********************************************************************/
+/*****                  WMesa API Functions                       *****/
+/**********************************************************************/
+
+
+
 #define PAL_SIZE 256
-void GetPalette(HPALETTE Pal,RGBQUAD *aRGB)
+static void GetPalette(HPALETTE Pal,RGBQUAD *aRGB)
 {
 	int i;
 	HDC hdc;
@@ -623,41 +818,25 @@ void GetPalette(HPALETTE Pal,RGBQUAD *aRGB)
     aRGB[i].rgbReserved=Palette.aEntries[i].peFlags;
   }
 }
-WMesaContext WMesaCreateContext(HWND hWnd,HPALETTE Pal,GLboolean rgb_flag,GLboolean db_flag)
+
+
+
+WMesaContext WMesaCreateContext( HWND hWnd, HPALETTE Pal, GLboolean rgb_flag,
+                                 GLboolean db_flag )
 {
   BITMAPINFO *Rec;
   HDC DC;
   RECT CR;
   WMesaContext c;
-  DD.draw_pixel=dd_draw_pixel;
-  DD.draw_line=dd_draw_line;
-  DD.draw_polygon=dd_draw_polygon;
-  DD.write_color_span=dd_write_color_span;
-  DD.write_monocolor_span=dd_write_monocolor_span;
-  DD.write_color_pixels=dd_write_color_pixels;
-  DD.write_monocolor_pixels=dd_write_monocolor_pixels;
-  DD.write_index_span=dd_write_index_span;
-  DD.write_monoindex_span=dd_write_monoindex_span;
-  DD.write_index_pixels=dd_write_index_pixels;
-  DD.write_monoindex_pixels=dd_write_monoindex_pixels;
-  DD.read_index_span=dd_read_index_span;
-  DD.read_color_span=dd_read_color_span;
-  DD.read_index_pixels=dd_read_index_pixels;
-  DD.read_color_pixels=dd_read_color_pixels;
+
   c = (struct wmesa_context *) calloc(1,sizeof(struct wmesa_context));
   if (!c)
     return NULL;
-  /* allocate a new Mesa context */
-  c->gl_ctx = gl_new_context(NULL);
+
   c->Window=hWnd;
   if (rgb_flag==GL_FALSE)
   {
-    /* COLOR-INDEXED WINDOW:
-     * Even if the visual is TrueColor or DirectColor we treat it as
-     * being color indexed.  This is weird but might be useful to someone.
-     */
     c->rgb_flag = GL_FALSE;
-    c->gl_ctx->RGBAflag = GL_FALSE;
     c->pixel = 1;
     db_flag=GL_TRUE; // WinG requires double buffering
     //c->gl_ctx->BufferDepth = windepth;
@@ -665,16 +844,7 @@ WMesaContext WMesaCreateContext(HWND hWnd,HPALETTE Pal,GLboolean rgb_flag,GLbool
   else
   {
     c->rgb_flag = GL_TRUE;
-    c->gl_ctx->RGBAflag = GL_TRUE;
     c->pixel = 0;
-    c->gl_ctx->RedScale   = c->rmult = 255;
-    c->gl_ctx->GreenScale = c->gmult = 255;
-    c->gl_ctx->BlueScale  = c->bmult = 255;
-    c->gl_ctx->AlphaScale = c->amult = 255;
-    c->rshift=0;
-    c->gshift=0;
-    c->bshift=0;
-    c->ashift=0;
   }
   GetClientRect(c->Window,&CR);
   c->width=CR.right;
@@ -682,8 +852,6 @@ WMesaContext WMesaCreateContext(HWND hWnd,HPALETTE Pal,GLboolean rgb_flag,GLbool
   if (db_flag)
   {
     c->db_flag = 1;
-    c->gl_ctx->Color.DrawBuffer = GL_BACK;
-    c->gl_ctx->Pixel.ReadBuffer = GL_BACK;
     /* Double buffered */
     if (c->rgb_flag==GL_TRUE)
     {
@@ -721,12 +889,20 @@ WMesaContext WMesaCreateContext(HWND hWnd,HPALETTE Pal,GLboolean rgb_flag,GLbool
   {
     /* Single Buffered */
     c->db_flag = 0;
-    c->gl_ctx->Color.DrawBuffer = GL_FRONT;
-    c->gl_ctx->Pixel.ReadBuffer = GL_FRONT;
   }
-  c->gl_ctx->DBflag = db_flag;
+
+  /* allocate a new Mesa context */
+  c->gl_ctx = gl_new_context( rgb_flag,
+                              255.0, 255.0, 255.0, 255.0,
+                              db_flag, NULL);
+
+  setup_DD_pointers();
+
   return c;
 }
+
+
+
 void WMesaDestroyContext( WMesaContext c )
 {
    gl_destroy_context( c->gl_ctx );
@@ -738,14 +914,22 @@ void WMesaDestroyContext( WMesaContext c )
    }
    free( (void *) c );
 }
+
+
+
 void WMesaMakeCurrent( WMesaContext c )
 {
    gl_set_context( c->gl_ctx );
    Current = c;
-   if (Current->gl_ctx->Viewport.Width==0)
+   setup_DD_pointers();
+   if (Current->gl_ctx->Viewport.Width==0) {
       /* initialize viewport to window size */
       gl_viewport( 0, 0, Current->width, Current->height );
+   }
 }
+
+
+
 void WMesaSwapBuffers( void )
 {
   // *** Perhaps the DC should be saved in WMesaContext?
@@ -760,6 +944,9 @@ void WMesaSwapBuffers( void )
     ReleaseDC(Current->Window,DC);
   }
 }
+
+
+
 void WMesaPaletteChange(HPALETTE Pal)
 {
   if (Current && Current->rgb_flag==GL_FALSE)
