@@ -22,9 +22,28 @@
 
 
 /*
-$Id: xform.c,v 1.16 1995/07/24 20:35:20 brianp Exp $
+$Id: xform.c,v 1.22 1995/10/31 19:40:35 brianp Exp $
 
 $Log: xform.c,v $
+ * Revision 1.22  1995/10/31  19:40:35  brianp
+ * removed INLINE macro, more trouble than it was worth
+ *
+ * Revision 1.21  1995/10/16  15:26:56  brianp
+ * replaced dd_buffer_info with DD.buffer_size call
+ *
+ * Revision 1.20  1995/09/30  15:35:13  brianp
+ * added test for identity texture matrix in gl_mult_matrix
+ *
+ * Revision 1.19  1995/09/27  18:31:40  brianp
+ * added gl_transform_normals
+ *
+ * Revision 1.18  1995/09/26  16:05:26  brianp
+ * removed gl_transform_point, gl_transform_normal
+ * added gl_transform_points
+ *
+ * Revision 1.17  1995/09/13  14:42:23  brianp
+ * added logic to test for identity texture matrix
+ *
  * Revision 1.16  1995/07/24  20:35:20  brianp
  * replaced memset() with MEMSET() and memcpy() with MEMCPY()
  *
@@ -101,12 +120,6 @@ $Log: xform.c,v $
 #include "stencil.h"
 
 
-#ifdef __GNUC__
-#  define INLINE inline
-#else
-#  define INLINE
-#endif
-
 
 #ifndef M_PI
 #  define M_PI (3.1415926)
@@ -142,7 +155,7 @@ static void print_matrix( const GLfloat m[16] )
  * Input:  a, b - matrices to multiply
  * Output:  product - product of a and b
  */
-static INLINE void
+static void
 matmul( GLfloat *product, const GLfloat *a, const GLfloat *b )
 {
    /* This matmul was contributed by Thomas Malik */
@@ -253,47 +266,99 @@ void gl_compute_modelview_inverse( void )
 
 
 /*
- * Transform a 4-element column vector (4x1 matrix) by a 4x4 matrix.  This
- * function is used for transforming vertexes and texture coordinates.
- * Mathematically,  q = m * p.
- * Input:  m - transformation matrix
- *         p - input vector
- * Output:  q - transformed vector
+ * Apply a transformation matrix to an array of coordinates:
+ *   for i in 0 to n-1 do   q[i] = m * p[i]
+ * where p[i] and q[i] are 4-element column vectors and m is a 16-element
+ * transformation matrix.
  */
-void gl_transform_point( GLfloat q[4], const GLfloat m[16], const GLfloat p[4] )
+void gl_transform_points( GLuint n, GLfloat q[][4], const GLfloat m[16],
+                          GLfloat p[][4] )
 {
-#define M(row,col)  m[col*4+row]
-   q[0] = M(0,0) * p[0] + M(0,1) * p[1] + M(0,2) * p[2] + M(0,3) * p[3];
-   q[1] = M(1,0) * p[0] + M(1,1) * p[1] + M(1,2) * p[2] + M(1,3) * p[3];
-   q[2] = M(2,0) * p[0] + M(2,1) * p[1] + M(2,2) * p[2] + M(2,3) * p[3];
-   q[3] = M(3,0) * p[0] + M(3,1) * p[1] + M(3,2) * p[2] + M(3,3) * p[3];
-#undef M
+   /* This function has been carefully crafted to maximize register usage
+    * and use loop unrolling with IRIX 5.3's cc.  Hopefully other compilers
+    * will like this code too.
+    */
+   {
+      GLuint i;
+      GLfloat m0 = m[0];
+      GLfloat m4 = m[4];
+      GLfloat m8 = m[8];
+      GLfloat m12 = m[12];
+      GLfloat m1 = m[1];
+      GLfloat m5 = m[5];
+      GLfloat m9 = m[9];
+      GLfloat m13 = m[13];
+      for (i=0;i<n;i++) {
+         GLfloat p0 = p[i][0], p1 = p[i][1], p2 = p[i][2], p3 = p[i][3];
+         q[i][0] = m0 * p0 + m4  * p1 + m8 * p2 + m12 * p3;
+         q[i][1] = m1 * p0 + m5  * p1 + m9 * p2 + m13 * p3;
+      }
+   }
+   {
+      GLuint i;
+      GLfloat m2 = m[2];
+      GLfloat m6 = m[6];
+      GLfloat m10 = m[10];
+      GLfloat m14 = m[14];
+      GLfloat m3 = m[3];
+      GLfloat m7 = m[7];
+      GLfloat m11 = m[11];
+      GLfloat m15 = m[15];
+      for (i=0;i<n;i++) {
+         GLfloat p0 = p[i][0], p1 = p[i][1], p2 = p[i][2], p3 = p[i][3];
+         q[i][2] = m2 * p0 + m6 * p1 + m10 * p2 + m14 * p3;
+         q[i][3] = m3 * p0 + m7 * p1 + m11 * p2 + m15 * p3;
+      }
+   }
 }
 
 
 
 /*
- * Transform a normal vector from object coordinates to eye coordinates.
- * Mathematically, m = n * ModelViewInv
- * Input:  n - the input normal vector
- * Output:  m - the transformed normal vector
+ * Apply a transformation matrix to an array of normal vectors:
+ *   for i in 0 to n-1 do  v[i] = u[i] * m
+ * where u[i] and v[i] are 3-element row vectors and m is a 16-element
+ * transformation matrix.
+ * If the normalize flag is true the normals will be scaled to length 1.
  */
-void gl_transform_normal( GLfloat m[3], const GLfloat n[3] )
+void gl_transform_normals( GLuint n, GLfloat v[][3], const GLfloat m[16],
+                           GLfloat u[][3], GLboolean normalize )
 {
-   /* transform by inverse of modelview matrix */
-#define INV(row,col)  CC.ModelViewInv[col*4+row]
-   m[0] = n[0] * INV(0,0) + n[1] * INV(1,0) + n[2] * INV(2,0);
-   m[1] = n[0] * INV(0,1) + n[1] * INV(1,1) + n[2] * INV(2,1);
-   m[2] = n[0] * INV(0,2) + n[1] * INV(1,2) + n[2] * INV(2,2);
-#undef INV
-
-   if (CC.Transform.Normalize) {
-      /* scale to unit length */
-      GLfloat len = sqrt( m[0]*m[0] + m[1]*m[1] + m[2]*m[2] );
-      if (len>0.00001) {
-	 m[0] /= len;
-	 m[1] /= len;
-	 m[2] /= len;
+   if (normalize) {
+      /* Transform normals and scale to unit length */
+      GLuint i;
+      GLfloat m0 = m[0],  m4 = m[4],  m8 = m[8];
+      GLfloat m1 = m[1],  m5 = m[5],  m9 = m[9];
+      GLfloat m2 = m[2],  m6 = m[6],  m10 = m[10];
+      for (i=0;i<n;i++) {
+         GLdouble tx, ty, tz;
+         {
+            GLfloat ux = u[i][0],  uy = u[i][1],  uz = u[i][2];
+            tx = ux * m0 + uy * m1 + uz * m2;
+            ty = ux * m4 + uy * m5 + uz * m6;
+            tz = ux * m8 + uy * m9 + uz * m10;
+         }
+         {
+            GLdouble len, scale;
+            len = sqrt( tx*tx + ty*ty + tz*tz );
+            scale = (len>0.00001) ? (1.0 / len) : 1.0;
+            v[i][0] = tx * scale;
+            v[i][1] = ty * scale;
+            v[i][2] = tz * scale;
+         }
+      }
+   }
+   else {
+      /* Just transform normals, don't scale */
+      GLuint i;
+      GLfloat m0 = m[0],  m4 = m[4],  m8 = m[8];
+      GLfloat m1 = m[1],  m5 = m[5],  m9 = m[9];
+      GLfloat m2 = m[2],  m6 = m[6],  m10 = m[10];
+      for (i=0;i<n;i++) {
+         GLfloat ux = u[i][0],  uy = u[i][1],  uz = u[i][2];
+         v[i][0] = ux * m0 + uy * m1 + uz * m2;
+         v[i][1] = ux * m4 + uy * m5 + uz * m6;
+         v[i][2] = ux * m8 + uy * m9 + uz * m10;
       }
    }
 }
@@ -440,7 +505,7 @@ void gl_viewport( GLint x, GLint y, GLsizei width, GLsizei height )
 #endif
 
    /* ask device driver for size of output buffer */
-   dd_buffer_info( &buf_width, &buf_height, &buf_depth );
+   (*DD.buffer_size)( &buf_width, &buf_height, &buf_depth );
 
    /* see if size of device driver buffer has changed */
    newsize = CC.BufferWidth!=buf_width || CC.BufferHeight!=buf_height;
@@ -500,6 +565,26 @@ void glViewport( GLint x, GLint y, GLsizei width, GLsizei height )
 
 
 
+
+/*
+ * Determine if the given matrix is the identity matrix.
+ */
+static GLboolean is_identity( const GLfloat m[16] )
+{
+   if (   m[0]==1.0F && m[4]==0.0F && m[ 8]==0.0F && m[12]==0.0F
+       && m[1]==0.0F && m[5]==1.0F && m[ 9]==0.0F && m[13]==0.0F
+       && m[2]==0.0F && m[6]==0.0F && m[10]==1.0F && m[14]==0.0F
+       && m[3]==0.0F && m[7]==0.0F && m[11]==0.0F && m[15]==1.0F) {
+      return GL_TRUE;
+   }
+   else {
+      return GL_FALSE;
+   }
+}
+
+
+
+
 void glPushMatrix( void )
 {
    if (CC.CompileFlag) {
@@ -588,6 +673,7 @@ void glPopMatrix( void )
 	    MEMCPY( CC.TextureMatrix,
 		    CC.TextureStack[CC.TextureStackDepth],
 		    16*sizeof(GLfloat) );
+            CC.IdentityTexMat = is_identity( CC.TextureMatrix );
 	    break;
       }
    }
@@ -611,6 +697,7 @@ void gl_load_matrix( const GLfloat *m )
 	 break;
       case GL_TEXTURE:
 	 MEMCPY( CC.TextureMatrix, m, 16*sizeof(GLfloat) );
+         CC.IdentityTexMat = is_identity( CC.TextureMatrix );
 	 break;
    }
 }
@@ -671,6 +758,7 @@ void gl_mult_matrix( const GLfloat *m )
 	 break;
       case GL_TEXTURE:
 	 matmul( CC.TextureMatrix, CC.TextureMatrix, m );
+         CC.IdentityTexMat = is_identity( CC.TextureMatrix );
 	 break;
    }
 }
